###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 2003-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: alias.tcl,v 1.85 2005/01/04 03:36:43 tothwolf Exp $
##

# name: alias
# version: 0.5.5
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: alias.conf
# description: Adds dcc commands '.alias' and '.unalias' to simplify \
#	using multiple commands.
# requires: listMaxElementLength idx2type findBinds expandText


# FIXME: add more options? 'chalias'?
#        possibly store in array/list instead of xtra?
#
#        alias <handle/pattern> '* - global', '+f - friend flagged', etc
#        unalias ''
#
# alias command="cmd1 $1 $2; cmd2 $1 $3"
# aliasfield {command1 {{cmd1 $1 $2} {cmd2 $1 $3}}} {command2 {{cmd3 $1 $2}}}


##
## Add aliases: dcc
##
## Args:    hand, idx, arg
## Returns: 1
##
proc dcc:alias {hand idx arg} {
  variable configData

# FIXME: display usage when args are invalid
  set arg [string trimright $arg]

  # Display all aliases
  if {![string compare "" $arg]} then {
    set commandList [getuser $hand XTRA $configData(aliasfield)]

    if {[string compare "" $commandList]} then {
      set fmtlen [listMaxElementLength $commandList]

      putdcc $idx "Aliases:"
      foreach commandData [lsort $commandList] {
        foreach {command aliases} $commandData {break}
        putdcc $idx "[format "%-${fmtlen}s  %s" $command [join $aliases "; "]]"
      }
    } else {
      putdcc $idx "No aliases are defined."
    }

  # Add/display specific alias(es)
  } else {

    # This processes multiple aliases, and can handle setting and
    # displaying of aliases at the same time. This is very similar
    # to the bash alias command.

    # Process arg and turn it into a valid list
# FIXME: handle both "" and ''
    set regexp { ?([^= ]*(=("[^"]*"|[^ ]*)?)?)}
    while {([regexp -- $regexp $arg -> alias]) && \
           ([string compare "" $arg])} {
      regsub -- $regexp $arg "" arg
      if {[string compare "" $alias]} then {

        # Split command="aliases", but allow "aliases" to contain '='.
        if {[regexp -- {([^=]+)=(.*)} $alias -> command aliases]} then {

          #
          # Add specific alias
          #

          set trimAliases ""

# FIXME: handle both "" and ''
          # Trim off outer "", but leave any others intact.
          regexp -- {([^"]*[^"]+)} $aliases -> trimAliases

          set newAliases ""

          foreach tmpAlias [split $trimAliases \;] {
            lappend newAliases [string trim $tmpAlias]
          }

# FIXME: this does not work properly
#
#          # Split off alias at ';'
#          set regexp {(.*[^\\;])\;}
#          while {([regexp -- $regexp $trimAliases -> tmpAlias]) && \
#                 ([string compare "" $trimAliases])} {
#            regsub -- $regexp $trimAliases "" trimAliases
#            lappend newAliases [string trim $tmpAlias]
#          }
#          if {[string compare "" $trimAliases]} then {
#            lappend newAliases [string trim $trimAliases]
#          }

          set commandList [getuser $hand XTRA $configData(aliasfield)]
          set tmpCommandList [list $command $newAliases]
          set newCommandList ""

          # Remove old command alias from commandList if it exists
          if {[string compare "" $commandList]} then {
            foreach commandData $commandList {
              if {[string compare [lindex $commandData 0] $command]} then {
                lappend newCommandList $commandData
              }
            }
          }

          lappend newCommandList $tmpCommandList
          setuser $hand XTRA [string toupper $configData(aliasfield)] $newCommandList

          putdcc $idx "Added alias '$command' = '[join $newAliases "; "]'"

        } else {

          #
          # Display specific alias
          #

          set commandList [getuser $hand XTRA $configData(aliasfield)]

          if {[string compare "" $commandList]} then {
            set found 0
            foreach commandData $commandList {
              foreach {command aliases} $commandData {break}
              if {![string compare $alias $command]} then {
                putdcc $idx "${command}=\"[join $aliases "; "]\""
                set found 1
                break
              }
            }
            if {!$found} then {
              putdcc $idx "'$alias' not found"
            }
          } else {
            putdcc $idx "'$alias' not found"
          }
        }
      }
    }
  }
  return 1
}

##
## Remove aliases: dcc
##
## Args:    hand, idx, arg
## Returns: 1
##
proc dcc:unalias {hand idx arg} {
  variable configData

  set arg [split [string trimright $arg]]

  if {![string compare "" $arg]} then {
    putdcc $idx "Usage: unalias <command>"
  } else {
    set commandList [getuser $hand XTRA $configData(aliasfield)]

    if {[string compare "" $commandList]} then {
      set modified 0

      foreach command $arg {
        set removed 0
        set index 0

        foreach commandData $commandList {
          if {![string compare [lindex $commandData 0] $command]} then {
            set commandList [lreplace $commandList $index $index]
            set removed 1
            set modified 1
            break
          }
          incr index
        }

        if {$removed} then {
          putdcc $idx "Removed alias '$command'"
        } else {
          putdcc $idx "'$command': not an alias"
        }
      }

      if {$modified} then {
        setuser $hand XTRA [string toupper $configData(aliasfield)] $commandList
      }

    } else {
      putdcc $idx "No aliases are defined."
    }
    return 1
  }
  return 0
}

##
## Process dcc commands and watch for aliases
##
## Args:    idx, text
## Returns: text if command not matched
##          "" otherwise
##
proc filt:checkAliases {idx text} {
  variable configData
  variable ProcessingAliases

  if {![info exists ProcessingAliases($idx)]} then {
    set hand [idx2hand $idx]
    set aliasCommandList [getuser $hand XTRA $configData(aliasfield)]

    if {[string compare "" $aliasCommandList]} then {
      set aliasCommand [lindex $text 0]
      set aliasCommandArgs [join [lrange [split $text] 1 end]]
      set aliasResult [expandText $aliasCommand $aliasCommandList 0 foundAliases]

      # No alias match, return command processing to eggdrop
      if {$aliasResult == 0} then {
        return $text

      # Ambiguous alias match
      } elseif {$aliasResult == -1} then {
        putdcc $idx "Ambiguous command."
        return ""

      # Exact alias match
      } elseif {$aliasResult == 1} then {
        foreach {aliasCommand newAliases} $foundAliases {break}

      # Partial alias match
      } elseif {$aliasResult == 2} then {
        set idxtype [string tolower [idx2type $idx]]

# FIXME: bind matching should also check flags

        if {[string compare chat $idxtype]} then {
          set bindResult [findBinds dcc [string trimleft $aliasCommand .] foundBind]
        } elseif {[string compare files $idxtype]} then {
          set bindResult [findBinds fil [string trimleft $aliasCommand .] foundBind]
        } else {
          set bindResult 0
        }

        foreach {aliasCommand newAliases} $foundAliases {break}

        # Ambiguous bind match or,
        # Partial bind match and bind and alias commands are not the same.
        if {($bindResult == -1) || \
            (($bindResult == 2) && \
             ([string compare [string tolower $foundBind] \
               [string tolower [string trimleft $aliasCommand .]]]))} then {
          putdcc $idx "Ambiguous command."
          return ""

        # Exact bind match, go with bind *only*
        # if the full commands are not the same.
        } elseif {($bindResult == 1) && \
                  ([string compare [string tolower $foundBind] \
                    [string tolower [string trimleft $aliasCommand .]]])} then {
          return $text
        }
      }

      set aliases [expandAliases $aliasCommand $aliasCommandArgs $newAliases $aliasCommandList]
      if {[string compare "" $aliases]} then {

        set ProcessingAliases($idx) 1

        foreach simulCommand $aliases {
          dccsimul $idx $simulCommand
        }

        unset ProcessingAliases($idx)
        return ""
      }
    }
  }
  return $text
}

##
## Expand the given command to a list of commands from an alias command list
##
## Args:    command, commandArgs, aliases, commandList, args
## Returns: list of commands if the given command is an alias
##          nothing otherwise
##
proc expandAliases {command commandArgs aliases commandList args} {

  # Prevent endless loops of aliases.
  # This is trapped with code below.
  if {[lsearch -exact [set loop [lindex $args 0]] $command] != -1} then {
    regsub -all -- { } $loop { -> } loop
    error "Alias endless loop: $loop -> $command"
  }

  set index 0
  foreach tmpAlias $aliases {

    # '{cmd1 $1 $2}'
    set tmpCommand [lindex $tmpAlias 0]
    set tmpArgs [join [lrange $tmpAlias 1 end]]
    set oldArgs $tmpArgs

    # Don't endless loop within an alias:
    #   'fubar="test1; test2; fubar; test3"'
    if {![string compare $tmpCommand $command]} then {
      continue
    }

    # Substitute any desired args
    set tmpArgs [substAliasArgs $tmpArgs $commandArgs]

    # Command is another alias, recursive mode...
    set newaliases [listAliases $tmpCommand $commandList]
    if {[string compare "" $newaliases]} then {

      # Catch endless loop errors
      if {![catch {set tmpaliases [expandAliases $tmpCommand $tmpArgs $newaliases $commandList [concat $loop $command]]}]} then {
        set aliases [lreplace $aliases $index $index]
        set tmpindex $index
        foreach tmpalias $tmpaliases {
          set aliases [linsert $aliases $tmpindex $tmpalias]
          incr tmpindex
        }
      }

    } elseif {[string compare $oldArgs $tmpArgs]} then {
      set aliases [lreplace $aliases $index $index "$tmpCommand $tmpArgs"]
    }
    incr index
  }
  return $aliases
}

##
## List aliases from an alias command list for the given command
##
## Args:    command, commandList
## Returns: list of aliases if the command is found
##          nothing otherwise
##
proc listAliases {command commandList} {
  foreach commandAlias $commandList {
    foreach {tmpCommand tmpAliases} $commandAlias {break}
    if {![string compare $command $tmpCommand]} then {
      return $tmpAliases
    }
  }
  return ""
}

##
## Substitute alias argument markers with arguments
##
## Args:    aliases, arguments
## Returns: substituted aliases
##
proc substAliasArgs {aliases arguments} {
# FIXME: way to escape '$<num>'
# FIXME: allow range of arguments, '${1-3}', '${2-}', '${1,3-4}', etc
  # Replace each $1, $2, $3, etc with proper argument
  set index 1
  foreach arg $arguments {
    regsub -all -- "\\$${index}" $aliases $arg aliases
    incr index
  }

  # Replace special $@ with all arguments
  regsub -all -- {\$@} $aliases $arguments aliases

  # Remove any leftover '$<num>'
  regsub -all -- {\$[0-9]+} $aliases "" aliases
  return [string trim $aliases]
}

##
## Trace configuration changes
##
## Args:    name1, name2, operation
## Returns: nothing
##
proc TraceConfigData {name1 name2 op} {
  variable configData
  variable oldConfigData

  if {[string compare $oldConfigData($name2) $configData($name2)]} then {
    switch -exact -- $name2 {
      aliasfield {
        moveFields $oldConfigData($name2) [string toupper $configData($name2)]
      }
    }
    set oldConfigData($name2) $configData($name2)
  }
  return
}


##
## Init module specific things
##
## Args:    none
## Returns: nothing
##
proc ModulePreInit {} {
  global numversion

  if {$numversion < 1061400} then {
    error "This module requires Eggdrop 1.6.14 or newer."
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  global dcc-flood-thr
  variable configData
  variable oldConfigData

# FIXME: this is an ugly hack
  if {[info exists dcc-flood-thr]} then {
    set dcc-flood-thr 20
  }

  foreach {name data} [array get configData] {
    if {![info exists oldConfigData($name)]} then {
      set oldConfigData($name) $data
    }
  }
  trace variable configData w [namespace code TraceConfigData]
  return
}

##
## Destroy module specific things before unload
##
## Args:    none
## Returns: nothing
##
proc ModuleDestroy {} {
  variable configData

  trace vdelete configData w [namespace code TraceConfigData]
  return
}

# Configuration defaults, values and descriptions
array set configDefaults {
  aliasfield {word cmdaliases {} field {field used to store user defined aliases}}
}

# Bind defaults
array set bindDefaults {
  dcc:alias {{dcc -|- alias {} {}}}
  dcc:unalias {{dcc -|- unalias {} {}}}
  filt:checkAliases {{filt -|- * {} {}}}
}

# Help Data
array set helpData {
  dcc:alias {{FIXME} {show/add command aliases}}
  dcc:unalias {{FIXME} {remove command aliases}}
}
