###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: auth.tcl,v 1.32 2005/01/23 03:44:59 tothwolf Exp $
##

# name: auth
# version: 1.5.1
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: auth.conf
# description: Provides a login/logout authentication system and functions \
#	other modules can use for secure /msg and public commands.
# requires: wpLog commandChar putList putnot convertSeconds formatTime \
#	splitStringList clearFields wipeFields moveFields whoisFields \
#	validHandle floodCheck checkKnownUser
# provides: authUserCheck authUserAdd authUserDel authUserNickchange \
#	authUserClearChannel authUserClearExpired authCheck


##
## Login for a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 0
##
proc msg:login {nick uhost hand arg} {
  variable configData

  if {([floodCheck $nick $uhost login]) && \
      (![isbotnick $nick]) && ([validHandle $hand])} then {
    set arg [split [string trimright $arg]]
    set pass [lindex $arg 0]
    set chan [lindex $arg 1]
    if {[passwdok $hand ""]} then {
      putnot $nick [mc "You must set a password before you can use the '%s' command." login]
    } elseif {![string compare "" $pass]} then {
      putnot $nick [mc "Usage: %s%s <password> \[channel\]" [commandChar msg] login]
    } elseif {[string compare "" $chan]} then {
      if {[passwdok $hand $pass]} then {
        if {[validchan $chan]} then {
          if {[botonchan $chan]} then {
            if {[onchan $nick $chan]} then {
              if {[authUserCheck $nick $uhost $hand $chan] != 1} then {
                authUserAdd $nick $uhost $hand $chan
                putnot $nick [mc "Login successful for %s." $chan]
              } else {
                putnot $nick [mc "You are already logged in for %s." $chan]
              }
            } else {
              putnot $nick [mc "You are not on %s." $chan]
            }
          } else {
            putnot $nick [mc "I'm not on %s." $chan]
          }
        } else {
          putnot $nick [mc "I don't monitor %s." $chan]
        }
      } else {
        wpLog o * [mc "Invalid login from: %s!%s (%s) for %s" $nick $uhost $hand $chan]
        putnot $nick [mc "Login failed for %s." $chan]
      }
    } else {
      set botonchan 0
      set onchan 0
      if {[passwdok $hand $pass]} then {
        foreach chan [channels] {
          if {[botonchan $chan]} then {
            set botonchan 1
            if {[onchan $nick $chan]} then {
              set onchan 1
              if {[authUserCheck $nick $uhost $hand $chan] != 1} then {
                authUserAdd $nick $uhost $hand $chan
                lappend list1 $chan,
              } else {
                lappend list2 $chan,
              }
            }
          }
        }
        if {$botonchan} then {
          if {$onchan} then {
            if {[info exists list1]} then {
              putList putnot $nick [splitStringList $list1 160 "," ", "] [mc "Login successful for: "] [mc "(cont): "]
            } elseif {[info exists list2]} then {
              putList putnot $nick [splitStringList $list2 160 "," ", "] [mc "You are already logged in for: "] [mc "(cont): "]
            }
          } else {
            putnot $nick [mc "You are not on any channels that I monitor."]
          }
        } else {
          putnot $nick [mc "I'm not on any channels."]
        }
      } else {
        wpLog o * [mc "Invalid login from: %s!%s (%s)" $nick $uhost $hand]
        putnot $nick [mc "Login failed."]
      }
    }
    putloglev c * "($nick!$uhost) !$hand! login"
  }
  return 0
}

##
## Logout for a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 0
##
proc msg:logout {nick uhost hand arg} {
  variable configData

  if {([floodCheck $nick $uhost logout]) && \
      (![isbotnick $nick]) && ([validHandle $hand])} then {
    set arg [split [string trimright $arg]]
    set pass [lindex $arg 0]
    set chan [lindex $arg 1]
    if {[passwdok $hand ""]} then {
      putnot $nick [mc "You must set a password before you can use the '%s' command." logout]
    } elseif {![string compare "" $pass]} then {
      putnot $nick [mc "Usage: %s%s <password> \[channel\]" [commandChar msg] logout]
    } elseif {[string compare "" $chan]} then {
      if {[passwdok $hand $pass]} then {
        if {[validchan $chan]} then {
          if {[botonchan $chan]} then {
            if {[onchan $nick $chan]} then {
              if {[authUserCheck $nick $uhost $hand $chan] == 1} then {
                authUserDel $nick $uhost $hand $chan
                putnot $nick [mc "Logout successful for %s." $chan]
              } else {
                putnot $nick [mc "You were never logged in for %s." $chan]
              }
            } else {
              putnot $nick [mc "You are not on %s." $chan]
            }
          } else {
            putnot $nick [mc "I'm not on %s." $chan]
          }
        } else {
          putnot $nick [mc "I don't monitor %s." $chan]
        }
      } else {
        wpLog o * [mc "Invalid logout from: %s!%s (%s) for %s" $nick $uhost $hand $chan]
        putnot $nick [mc "Logout failed for %s." $chan]
      }
    } else {
      set botonchan 0
      set onchan 0
      if {[passwdok $hand $pass]} then {
        foreach chan [channels] {
          if {[botonchan $chan]} then {
            set botonchan 1
            if {[onchan $nick $chan]} then {
              set onchan 1
              if {[authUserCheck $nick $uhost $hand $chan] == 1} then {
                authUserDel $nick $uhost $hand $chan
                lappend list $chan,
              }
            }
          }
        }
        if {$botonchan} then {
          if {$onchan} then {
            if {[info exists list]} then {
              putList putnot $nick [splitStringList $list 160 "," ", "] [mc "Logout successful for: "] [mc "(cont): "]
            } else {
              putnot $nick [mc "You were not logged in on any channels."]
            }
          } else {
            putnot $nick [mc "You are not on any channels that I monitor."]
          }
        } else {
          putnot $nick [mc "I'm not on any channels."]
        }
      } else {
        wpLog o * [mc "Invalid logout from: %s!%s (%s)" $nick $uhost $hand]
        putnot $nick [mc "Logout failed."]
      }
    }
    putloglev c * "($nick!$uhost) !$hand! logout"
  }
  return 0
}

##
## Login for a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:login {hand idx arg} {
  variable configData

  set chan [lindex [split [string trimright $arg]] 0]
  if {[string compare "" $chan]} then {
    if {[validchan $chan]} then {
      if {[botonchan $chan]} then {
        if {[handonchan $hand $chan]} then {
          set nick [hand2nick $hand $chan]
          set uhost [getchanhost $nick $chan]
          if {[authUserCheck $nick $uhost $hand $chan] != 1} then {
            authUserAdd $nick $uhost $hand $chan
            putdcc $idx [mc "Login successful for %s." $chan]
          } else {
            putdcc $idx [mc "You are already logged in for %s." $chan]
          }
        } else {
          putdcc $idx [mc "You are not on %s." $chan]
        }
      } else {
        putdcc $idx [mc "I'm not on %s." $chan]
      }
    } else {
      putdcc $idx [mc "I don't monitor %s." $chan]
    }
  } else {
    set botonchan 0
    set handonchan 0
    foreach chan [channels] {
      if {[botonchan $chan]} then {
        set botonchan 1
        if {[handonchan $hand $chan]} then {
          set handonchan 1
          set nick [hand2nick $hand $chan]
          set uhost [getchanhost $nick $chan]
          if {[authUserCheck $nick $uhost $hand $chan] != 1} then {
            authUserAdd $nick $uhost $hand $chan
            lappend list1 $chan,
          } else {
            lappend list2 $chan,
          }
        }
      }
    }
    if {$botonchan} then {
      if {$handonchan} then {
        if {[info exists list1]} then {
          putList putdcc $idx [splitStringList $list1 160 "," ", "] [mc "Login successful for: "] [mc "(cont): "]
        } elseif {[info exists list2]} then {
          putList putdcc $idx [splitStringList $list2 160 "," ", "] [mc "You are already logged in for: "] [mc "(cont): "]
        }
      } else {
        putdcc $idx [mc "You are not on any channels that I monitor."]
      }
    } else {
      putdcc $idx [mc "I'm not on any channels."]
    }
  }
  return 1
}

##
## Logout for a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:logout {hand idx arg} {
  variable configData

  set chan [lindex [split [string trimright $arg]] 0]
  if {[string compare "" $chan]} then {
    if {[validchan $chan]} then {
      if {[botonchan $chan]} then {
        if {[handonchan $hand $chan]} then {
          set nick [hand2nick $hand $chan]
          set uhost [getchanhost $nick $chan]
          if {[authUserCheck $nick $uhost $hand $chan] == 1} then {
            authUserDel $nick $uhost $hand $chan
            putdcc $idx [mc "Logout successful for %s." $chan]
          } else {
            putdcc $idx [mc "You were never logged in for %s." $chan]
          }
        } else {
          putdcc $idx [mc "You are not on %s." $chan]
        }
      } else {
        putdcc $idx [mc "I'm not on %s." $chan]
      }
    } else {
      putdcc $idx [mc "I don't monitor %s." $chan]
    }
  } else {
    set botonchan 0
    set handonchan 0
    foreach chan [channels] {
      if {[botonchan $chan]} then {
        set botonchan 1
        if {[handonchan $hand $chan]} then {
          set handonchan 1
          set nick [hand2nick $hand $chan]
          set uhost [getchanhost $nick $chan]
          if {[authUserCheck $nick $uhost $hand $chan] == 1} then {
            authUserDel $nick $uhost $hand $chan
            lappend list $chan,
          }
        }
      }
    }
    if {$botonchan} then {
      if {$handonchan} then {
        if {[info exists list]} then {
          putList putdcc $idx [splitStringList $list 160 "," ", "] [mc "Logout successful for: "] [mc "(cont): "]
        } else {
          putdcc $idx [mc "You are not logged in on any channels."]
        }
      } else {
        putdcc $idx [mc "You are not on any channels that I monitor."]
      }
    } else {
      putdcc $idx [mc "I'm not on any channels."]
    }
  }
  return 1
}

##
## Remove auth if user parts a channel
##
## Args:    nick, user@host, handle, channel, part message
## Returns: 0
##
proc part:auth {nick uhost hand chan {msg ""}} {
  if {![isbotnick $nick]} then {
    if {([validHandle $hand]) && \
        ([authUserCheck $nick $uhost $hand $chan])} then {
      authUserDel $nick $uhost $hand $chan
    }
  } else {
    # bot parted $chan, clear all auth data for all users on $chan
    authUserClearChannel $nick $uhost $hand $chan
  }
  return 0
}

##
## Remove auth if user is kicked from a channel
##
## Args:    nick uhost hand chan knick reas
## Returns: 0
##
proc kick:auth {nick uhost hand chan knick reas} {
  if {![isbotnick $knick]} then {

# FIXME: !!! $hand and $uhost do not match $knick
#        would a raw binding be the only way to fix $uhost?
#
# if {[validHandle $hand]} then { .... }
#
#    if {[authUserCheck $knick $uhost $hand $chan]} then {
#      authUserDel $knick $uhost $hand $chan
#    }

  } else {
    # bot kicked from $chan, clear all auth data for all users on $chan
    authUserClearChannel $knick "" "" $chan
  }
  return 0
}

#
# FIXME: detect eggdrop signoff, remove all auth data
#

##
## Remove auth if user signs off
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 0
##
proc sign:auth {nick uhost hand chan arg} {
  # eggdrop can't see its own signoff with sign bindings
  if {(![isbotnick $nick]) && ([validHandle $hand]) && \
      ([authUserCheck $nick $uhost $hand $chan])} then {
    authUserDel $nick $uhost $hand ""
  }
  return 0
}

##
## Remove auth if user signs off due to a netsplit
##
## Args:    nick, user@host, handle, channel
## Returns: 0
##
proc splt:auth {nick uhost hand chan} {
  # eggdrop can't see its own signoff with split bindings
  if {(![isbotnick $nick]) && ([validHandle $hand]) && \
      ([authUserCheck $nick $uhost $hand $chan])} then {
    authUserDel $nick $uhost $hand ""
  }
  return 0
}

##
## Change auth when a user changes nicks
##
## Args:    nick, user@host, handle, channel, newnick
## Returns: 0
##
proc nick:auth {nick uhost hand chan newnick} {
  if {(![isbotnick $nick]) && ([validHandle $hand])} then {
    if {[set check [authUserCheck $nick $uhost $hand $chan]] == 1} then {
      authUserNickchange $nick $uhost $hand $chan $newnick
    } elseif {$check == -1} then {
      authUserDel $nick $uhost $hand $chan
    }
  }
  return 0
}

##
## Remove expired logins
##
## Args:    minute, hour, day, month, year
## Returns: 0
##
proc time:10Minutes {minute hour day month year} {
  authUserClearExpired
  return 0
}

##
## Check auth data for a user
##
## Args:    nick, uhost, hand, chan, args
## Returns: 1 if user is auth'd
##          0 if user is not auth'd
##          -1 if auth has expired
##
proc authUserCheck {nick uhost hand chan args} {
  variable configData
  variable authdata

  # auth data exists at all?
  if {[info exists authdata($nick!$uhost)]} then {
    foreach {authhand authtime authchans} $authdata($nick!$uhost) {break}
    # global type checking if chan == "" and allow for no handle check
    if {((![string compare "" $hand]) || \
         (![string compare $hand $authhand])) && \
        ((![string compare "" $chan]) || \
         ([lsearch -exact $authchans [string tolower $chan]] != -1))} then {
      if {(!$configData(expire)) || \
          ([expr [clock seconds] - $authtime] < [convertSeconds from minutes $configData(expire)])} then {
        return 1
      } else {
        return -1
      }
    }
  }
  return 0
}

##
## Add auth data for a user
##
## Args:    nick, uhost, hand, chan, args
## Returns: nothing
##
proc authUserAdd {nick uhost hand chan args} {
  variable configData
  variable authdata

  # NOTE: channel and handle required here
  set time [clock seconds]
  if {![info exists authdata($nick!$uhost)]} then {
    set channels [string tolower $chan]
  } else {
    foreach {authhand authtime authchans} $authdata($nick!$uhost) {break}
    # don't add channel more than once
    if {[lsearch -exact $authchans [set lchan [string tolower $chan]]] == -1} then {
      set channels "$authchans $lchan"
    }
  }
  if {[info exists channels]} then {
    set authdata($nick!$uhost) [list $hand $time $channels]
    # FIXME: This may be shared across a botnet.
    #        Real auth is checked against the authdata array above,
    #        but this is nice to have in '.whois' output
    setuser $hand XTRA [string toupper $configData(authfield)] $nick!$uhost
    regsub -all -- { } $channels {, } channels
    setuser $hand XTRA [string toupper $configData(chanfield)] $channels
    setuser $hand XTRA [string toupper $configData(loginfield)] [formatTime $time]
    if {$configData(expire)} then {
      setuser $hand XTRA [string toupper $configData(expirefield)] \
       [formatTime [expr $time + [convertSeconds from minutes $configData(expire)]]]
    } else {
      clearFields $hand $configData(expirefield)
    }
  }
  return
}

##
## Remove auth data for a user
##
## Args:    nick, uhost, hand, chan, args
## Returns: nothing
##
proc authUserDel {nick uhost hand chan args} {
  variable configData
  variable authdata

  if {[info exists authdata($nick!$uhost)]} then {
    foreach {authhand authtime authchans} $authdata($nick!$uhost) {break}
    if {[string compare "" $chan]} then {
      # remove channel if it exists
      if {[set index [lsearch -exact $authchans [string tolower $chan]]] != -1} then {
        if {[llength $authchans] <= 1} then {
          unset authdata($nick!$uhost)
          clearFields $authhand "$configData(authfield) $configData(chanfield) $configData(loginfield) $configData(expirefield)"
        } else {
          set channels [lreplace $authchans $index $index]
          set authdata($nick!$uhost) [list $authhand $authtime $channels]
          regsub -all -- { } $channels {, } channels
          setuser $authhand XTRA [string toupper $configData(chanfield)] $channels
        }
      }
    } else {
      unset authdata($nick!$uhost)
      clearFields $authhand "$configData(authfield) $configData(chanfield) $configData(loginfield) $configData(expirefield)"
    }
  }
  return
}

##
## Update auth data for a user
##
## Args:    nick, uhost, hand, chan, newnick, args
## Returns: nothing
##
proc authUserNickchange {nick uhost hand chan newnick args} {
  variable authdata

  if {[info exists authdata($nick!$uhost)]} then {
    set authdata($newnick!$uhost) $authdata($nick!$uhost)
    unset authdata($nick!$uhost)
  }
  return
}

##
## Clear auth data for a channel
##
## Args:    nick, uhost, hand, chan, args
## Returns: nothing
##
proc authUserClearChannel {nick uhost hand chan args} {
  variable configData
  variable authdata

  if {[string compare "" [array names authdata]]} then {
    foreach {name data} [array get authdata] {
      foreach {authhand authtime authchans} $data {break}
      # remove channel if it exists
      if {[set index [lsearch -exact $authchans [string tolower $chan]]] != -1} then {
        if {[llength $authchans] <= 1} then {
          unset authdata($nick!$uhost)
          clearFields $authhand "$configData(authfield) $configData(chanfield) $configData(loginfield) $configData(expirefield)"
        } else {
          set channels [lreplace $authchans $index $index]
          set authdata($nick!$uhost) [list $authhand $authtime $channels]
          regsub -all -- { } $channels {, } channels
          setuser $authhand XTRA [string toupper $configData(chanfield)] $channels
        }
      }
    }
  }
  return
}

##
## Clear all expired auth data
##
## Args:    none
## Returns: nothing
##
proc authUserClearExpired {} {
  variable configData
  variable authdata

  if {[string compare "" [array names authdata]]} then {
    foreach {name data} [array get authdata] {
      foreach {authhand authtime authchans} $data {break}
      if {($configData(expire)) && \
          ([expr [clock seconds] - $authtime] >= [convertSeconds from minutes $configData(expire)])} then {
        unset authdata($name)
        clearFields $authhand "$configData(authfield) $configData(chanfield) $configData(loginfield) $configData(expirefield)"
      }
    }
  }
  return
}

##
## Check to see if a user is logged in, and notice them if not
##
## Args:    type {pub,msg,dcc,fil}, nick, user@host, handle, channel,
##           command
## Returns: 1 if user is logged in, or mode is dcc or fil
##          0 otherwise
##
proc authCheck {type command nick uhost hand chan} {
  switch -exact -- $type {
    pub -
    msg {
      switch -exact -- $type {
        pub {
          set Log "<<$nick>> !$hand!"
        }
        msg {
          set Log "($nick!$uhost) !$hand!"
        }
      }
      switch -exact -- [checkKnownUser $nick $hand] {
        -1 {
          wpLog c * [mc "%s Failed %s (%s is a bot)" $Log $command $hand]
        }
        0 {
          wpLog c * [mc "%s Failed %s (unknown user)" $Log $command]
        }
        1 {
          set check [authUserCheck $nick $uhost $hand $chan]
          if {$check == 1} then {
            return 1
          } else {
            if {$check == -1} then {
              wpLog c * [mc "%s Failed %s (login expired)" $Log $command]
              putnot $nick [mc "Your login has expired, please login again."]
            } else {
              wpLog c * [mc "%s Failed %s (not logged in)" $Log $command]
              putnot $nick [mc "You must login to use the command: %s%s" [commandChar $type] $command]
            }
            putnot $nick [mc "To login, type: %s%s <password> \[channel\]" [commandChar msg] login]
          }
        }
      }
    }
    dcc -
    fil {
      return 1
    }
    default {
      error [mc "Unknown auth type: %s" $type]
    }
  }
  return 0
}

##
## Trace configuration changes
##
## Args:    name1, name2, operation
## Returns: nothing
##
proc TraceConfigData {name1 name2 op} {
  variable configData
  variable oldConfigData
  variable authdata

  if {[string compare $oldConfigData($name2) $configData($name2)]} then {
    switch -exact -- $name2 {
      expire {
        if {$configData(expire)} then {
          if {[string compare "" [array names authdata]]} then {
            foreach {name data} [array get authdata] {
              foreach {authhand authtime authchans} $data {break}
              setuser $authhand XTRA [string toupper $configData(expirefield)] \
               [formatTime [expr $authtime + [convertSeconds from minutes $configData(expire)]]]
            }
          }
        } else {
          wipeFields $configData(expirefield)
        }
      }
      authfield -
      chanfield -
      loginfield -
      expirefield {
        moveFields $oldConfigData($name2) [string toupper $configData($name2)]
        whoisFields update $configData($name2) $oldConfigData($name2)
      }
    }
    set oldConfigData($name2) $configData($name2)
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  variable configData
  variable oldConfigData

  foreach {name data} [array get configData] {
    if {![info exists oldConfigData($name)]} then {
      set oldConfigData($name) $data
    }
  }
  whoisFields update "$configData(authfield) $configData(chanfield) $configData(loginfield) $configData(expirefield)" \
                     "$oldConfigData(authfield) $configData(chanfield) $oldConfigData(loginfield) $oldConfigData(expirefield)"
  trace variable configData w [namespace code TraceConfigData]
  return
}

##
## Destroy module specific things before unload
##
## Args:    none
## Returns: nothing
##
proc ModuleDestroy {} {
  variable configData

  whoisFields remove "$configData(authfield) $configData(chanfield) $configData(loginfield) $configData(expirefield)"
  trace vdelete configData w [namespace code TraceConfigData]
  return
}

# Configuration defaults, values and descriptions
array set configDefaults {
  expire {integer 240 {} minutes {minutes until login expires}}
  authfield {word authuser {} field {field used to display login user@host}}
  chanfield {word authchans {} field {field used to display login channels}}
  loginfield {word authlogin {} field {field used to display login time for a user}}
  expirefield {word authexpire {} field {field used to display login expire time for a user}}
}

# Bind defaults
array set bindDefaults {
  msg:login {{msg -|- login {} {}} {msg -|- logon {} {}}}
  msg:logout {{msg -|- logout {} {}} {msg -|- logoff {} {}}}
  dcc:login {{dcc -|- login {} {}} {dcc -|- logon {} {}}}
  dcc:logout {{dcc -|- logout {} {}} {dcc -|- logoff {} {}}}
  sign:auth {{sign -|- * {} {}}}
  part:auth {{part -|- * {} {}}}
  nick:auth {{nick -|- * {} {}}}
  splt:auth {{splt -|- * {} {}}}
  time:10Minutes {{time -|- {?0 * * * *} {} {}}}
}

# Help Data
array set helpData {
  msg:login {{<password> [channel]} {login for a channel}}
  msg:logout {{<password> [channel]} {logout for a channel}}
  dcc:login {{[channel]} {login for a channel}}
  dcc:logout {{[channel]} {logout for a channel}}
}
