###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: botping.tcl,v 1.14 2005/01/08 20:20:15 tothwolf Exp $
##

# name: botping
# version: 1.5.2
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Adds a dcc command called `.ping' and botnet support for \
#	sending/replying to ping requests from other bots.
# requires: isNumber clockSecondsClicks formatElapsedTime


##
## Send ping request to a bot or the botnet: dcc
##
## Args:    handle, idx, botname
## Returns: 1
##
proc dcc:ping {hand idx arg} {
  variable DataIdx

  set bot [lindex [split [string trimright $arg]] 0]
  if {![string compare "" $bot]} then {
    if {[string compare "" [bots]]} then {
      set DataIdx($hand) $idx
      putallbots "ping $hand [clockSecondsClicks]"
    } else {
      putdcc $idx [mc "Error: No bots linked."]
    }
  } elseif {[islinked $bot]} then {
    set DataIdx($hand) $idx
    putbot $bot "ping $hand [clockSecondsClicks]"
  } else {
    putdcc $idx [mc "Error: %s is not connected to the botnet." $bot]
  }
  return 1
}

##
## Reply to a ping request on the botnet: bot
##
## Args:    bot, command, arg
## Returns: 0
##
proc bot:ping {bot cmd arg} {
  putbot $bot "pong $arg"
  return 0
}

##
## Display ping replys from bots: bot
##
## Args:    bot, command, arg
## Returns: 0
##
proc bot:pong {bot cmd arg} {
  variable DataIdx

  set now [clockSecondsClicks]
  set arg [split $arg]
  set hand [lindex $arg 0]
  set seconds [lindex $arg 1]
  set clicks [lindex $arg 2]
  if {([validuser $hand]) && ([isNumber $seconds]) &&
      ([isNumber $clicks]) && ([info exists DataIdx($hand)])} then {
    putdcc $DataIdx($hand) [mc "Ping reply from %s: %s" $bot [formatElapsedTime "$seconds $clicks" $now]]
  }
  return 0
}

##
## Clear invalid idx data from DataIdx array
##
## Args:    minute, hour, day, month, year
## Returns: 0
##
proc time:hourly {minute hour day month year} {
  variable DataIdx

  if {[string compare "" [set tmp [array get DataIdx]]]} then {
    foreach {hand idx} $tmp {
      if {(![valididx $idx]) || \
          ([string compare [string tolower $hand] \
            [string tolower [idx2hand $idx]]])} then {
        catch {
          unset DataIdx($hand)
        }
      }
    }
  }
  return 0
}

# Bind defaults
array set bindDefaults {
  dcc:ping {{dcc mt|- ping {} {}}}
  bot:ping {{bot -|- ping {} {}}}
  bot:pong {{bot -|- pong {} {}}}
  time:hourly {{time -|- {00 * * * *} {} {}}}
}

# Help Data
array set helpData {
  dcc:ping {{[bot]} {ping the given bot or all bots if none given}}
}
