###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: faqserv.tcl,v 1.17 2005/01/04 03:36:43 tothwolf Exp $
##

# name: faqserv
# version: 1.6.5.3
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: faqserv.conf
# description:
# requires: this_module_needs_to_be_converted commandChar \
#	arrayFindElementName arraySearch countControlProc \
#	countControlProcUhost


# set data information
set data(faqdata) "1 $conf(faqdatafile) faqdata"

# load data
module_data load "" 1 0 "1 $conf(faqdatafile) faqdata"

# show the given faq topic; pub
proc pub:faq {nick uhost hand chan arg} {
  return [faq pub putmsg:putnot $chan:$nick $nick $uhost $hand $chan $arg]
}

# show the given faq topic; msg
proc msg:faq {nick uhost hand arg} {
  return [faq msg putnot $nick $nick $uhost $hand "" $arg]
}

# show the given faq topic; dcc
proc dcc:faq {hand idx arg} {
  return [faq dcc putdcc $idx "" "" $hand "" $arg]
}

# show the given faq topic
proc faq {type action dest nick uhost hand chan arg} {
  variable faqdata
  set what [lindex [split [string trimright $arg]] 0]
  if {![string compare "" $what]} then {
    put $action $dest "Usage: [commandChar $type]faq <topic>" 1
  } else {
    set topic [arrayFindElementName faqdata $what]
    if {[string compare "" $topic]} then {
      put $action $dest "Info for $topic: [join $faqdata($topic)]"
    } else {
      put $action $dest "Faq topic $what does not exist." 1
    }
  }
  return 1
}

# add a faq topic to the faq list; pub
proc pub:addfaq {nick uhost hand chan arg} {
  return [addfaq pub putnot $nick $nick $uhost $hand $chan $arg]
}

# add a faq topic to the faq list; msg
proc msg:addfaq {nick uhost hand arg} {
  return [addfaq msg putnot $nick $nick $uhost $hand "" $arg]
}

# add a faq topic to the faq list; dcc
proc dcc:addfaq {hand idx arg} {
  return [addfaq dcc putdcc $idx "" "" $hand "" $arg]
}

# add a faq topic to the faq list
proc addfaq {type action dest nick uhost hand chan arg} {
  variable faqdata
  variable faqdata_changed
  if {[check_auth $type $nick $uhost $hand $chan addfaq]} then {
    set arg [split [string trimright $arg]]
    set what [lindex $arg 0]
    set content [join [lrange $arg 1 end]]
    if {(![string compare "" $what]) || (![string compare "" $content])} then {
      put $action $dest "Usage: [commandChar $type]addfaq <topic> <content>"
    } else {
      set topic [arrayFindElementName faqdata $what]
      if {[string compare "" $topic]} then {
        set faqdata($topic) [list $content]
        set faqdata_changed 1
        put $action $dest "Added faq topic: $topic"
      } else {
        put $action $dest "Faq $topic already exists."
      }
    }
    return 1
  }
  return 0
}

# remove a faq topic from the faq list; pub
proc pub:delfaq {nick uhost hand chan arg} {
  return [delfaq pub putnot $nick $nick $uhost $hand $chan $arg]
}

# remove a faq topic from the faq list; msg
proc msg:delfaq {nick uhost hand arg} {
  return [delfaq msg putnot $nick $nick $uhost $hand "" $arg]
}

# remove a faq topic from the faq list; dcc
proc dcc:delfaq {hand idx arg} {
  return [delfaq dcc putdcc $idx "" "" $hand "" $arg]
}

# remove a faq topic from the faq list
proc delfaq {type action dest nick uhost hand chan arg} {
  variable faqdata
  variable faqdata_changed
  if {[check_auth $type $nick $uhost $hand $chan delfaq]} then {
    set what [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $what]} then {
      put $action $dest "Usage: [commandChar $type]delfaq <topic>"
    } else {
      set topic [arrayFindElementName faqdata $what]
      if {[string compare "" $topic]} then {
        unset faqdata($topic)
        set faqdata_changed 1
        put $action $dest "Removed faq topic: $topic"
      } else {
        put $action $dest "Faq $what does not exist."
      }
    }
    return 1
  }
  return 0
}

# change a faq topic in the faq list; pub
proc pub:chfaq {nick uhost hand chan arg} {
  return [chfaq pub putnot $nick $nick $uhost $hand $chan $arg]
}

# change a faq topic in the faq list; msg
proc msg:chfaq {nick uhost hand arg} {
  return [chfaq msg putnot $nick $nick $uhost $hand "" $arg]
}

# change a faq topic in the faq list; dcc
proc dcc:chfaq {hand idx arg} {
  return [chfaq dcc putdcc $idx "" "" $hand "" $arg]
}

# change a faq topic in the faq list
proc chfaq {type action dest nick uhost hand chan arg} {
  variable faqdata
  variable faqdata_changed
  if {[check_auth $type $nick $uhost $hand $chan addfaq]} then {
    set arg [split [string trimright $arg]]
    set what [lindex $arg 0]
    set content [join [lrange $arg 1 end]]
    if {(![string compare "" $what]) || (![string compare "" $content])} then {
      put $action $dest "Usage: [commandChar $type]chfaq <topic> <content>"
    } else {
      set topic [arrayFindElementName faqdata $what]
      if {[string compare "" $topic]} then {
        set faqdata($topic) [list $content]
        set faqdata_changed 1
        put $action $dest "Changed faq topic: $topic"
      } else {
        put $action $dest "Faq $what does not exist."
      }
    }
    return 1
  }
  return 0
}

# answer a chat request; pub
proc pub:faqs_chat {nick uhost hand chan arg} {
  return [faqs_chat pub putnot $nick $nick $uhost $hand $chan $arg]
}

# answer a chat request; msg
proc msg:faqs_chat {nick uhost hand arg} {
  return [faqs_chat msg putnot $nick $nick $uhost $hand "" $arg]
}

# send dcc chat request
proc faqs_chat {type action dest nick uhost hand chan arg} {
  variable conf
  variable port

  if {![countControlProcUhost [namespace code "control_faqs $uhost"]]} then {
    if {($conf(port)) && ([info exists port])} then {
      putchat $nick $port
    }
  }
  return 1
}

# put the idx under control of the script
proc listen_faqs {idx} {
  variable conf

  if {[countControlProc [namespace code control_faqs]] < $conf(maxusers)} then {
    if {[countControlProcUhost [namespace code control_faqs] [idx2uhost $idx]] <= 1} then {
      putdcc $idx "Entering faq system..."
      control $idx [namespace code control_faqs]
      putdcc $idx "faqs>"
    } else {
      killdcc $idx
    }
  } else {
    putdcc $idx "Sorry, too many users are using the faq system, please try again later."
    killdcc $idx
  }
  return 0
}

# start a faq session if the user is already on the partyline; dcc
proc dcc:faqs {hand idx arg} {
  variable conf

  if {[countControlProc [namespace code control_faqs]] < $conf(maxusers)} then {
    putdcc $idx "Entering faq system..."
    control $idx [namespace code control_faqs]
    putdcc $idx "faqs>"
  } else {
    putdcc $idx "Sorry, too many users are using the faq system, please try again later."
  }
  return 1
}

# interact with the user
proc control_faqs {idx text} {
  variable conf
  variable faqdata
  variable faqdata_changed

  set hand [idx2hand $idx]
  if {![validuser $hand]} then {
    set uhost [idx2uhost $idx]
  }
  set text [split [string trimright $text]]
  set command [string trimleft [lindex $text 0] .]
  switch -exact [string tolower $command] {
    list {
      set topics [array names faqdata]
      if {[string compare "" $topics]} then {
        putdcc $idx "Faq topics avaliable:"
        foreach i [lsort $topics] {
          putdcc $idx "  $i"
        }
      } else {
        putdcc $idx "Error: no topics in database."
      }
    }
    show {
      set what [lindex $text 1]
      if {![string compare "" $what]} then {
        putdcc $idx "Usage: show <topic>"
      } else {
        set topic [arrayFindElementName faqdata $what]
        if {[string compare "" $topic]} then {
          putdcc $idx "Info for $topic: [join $faqdata($topic)]"
        } else {
          putdcc $idx "Faq topic $what does not exist."
        }
      }
    }
    search {
      set what [string trimright [lrange $text 1 end]]
      if {![string compare "" $what]} then {
        putdcc $idx "Usage: search <pattern>"
      } elseif {[string length $what] >= $conf(minsearch)} then {
        set results [arraySearch faqdata $what]
        if {[string compare "" $results]} then {
          putdcc $idx "Search results:"
          foreach i $results {
            set topic [lindex $i 0]
            set indexes [lrange $i 1 end]
            set result [join $faqdata($topic)]
            foreach j $indexes {
              set result [lreplace $result $j $j "\002[lindex $result $j]\002"]
            }
            putdcc $idx "$topic: $result"
          }
        } else {
          putdcc $idx "No match for: [join $what]"
        }
      } else {
        putdcc $idx "Search pattern must be at least $conf(minsearch) characters long."
      }
    }
    help {
      set what [lindex $text 1]
      if {![string compare "" $what]} then {
        set what help
      }
      switch -exact $what {
        list {
          putdcc $idx "###  list"
          putdcc $idx "  Lists all faq topics"
        }
        show {
          putdcc $idx "###  show <topic>"
          putdcc $idx "  shows the given faq topic"
        }
        search {
          putdcc $idx "###  search <word>"
          putdcc $idx "  searches all faq topics for the given word"
        }
        help {
          putdcc $idx "Commands for faq system:"
          putdcc $idx "  list"
          putdcc $idx "  show    <topic>"
          putdcc $idx "  search  <word>"
          putdcc $idx "  help   \[command\]"
          putdcc $idx "  quit"
          putdcc $idx "You can get help on individual commands: 'help <command>'"
        }
        quit {
          putdcc $idx "###  quit"
          putdcc $idx "  Quits the faq system and returns you to the partyline."
        }
        default {
          putdcc $idx "No help available on that."
        }
      }
    }
    quit {
      putdcc $idx "*** Ja mata!"
      putdcc $idx "Returning you to command mode..."
      return 1
    }
    default {
      putdcc $idx "What?  You need 'help'"
    }
  }
  if {![info exists uhost]} then {
    wpLog 2 * "#$hand# $command [lrange $text 1 end]"
  } else {
    wpLog 2 * "($uhost) $command [lrange $text 1 end]"
  }
  putdcc $idx "faqs>"
  return 0
}

# make sure things are set right when loaded
#if {$conf(port)} then {
#  set port [listen $conf(port) script [namespace code listen_faqs]]
#}

# Configuration defaults, values and descriptions
array set configDefaults {
  faqdatafile {word faqserv-faq.data {} file {file that faq data is stored in}}
  maxusers {integer 10 {} number {max users that can be in faq at once}}
  minsearch {integer 3 {} number {min number of characters for a search pattern}}
  port {integer 30000 {} port {port to accept telnet and dcc connections on}}
}

# Bind defaults
array set bindDefaults {
  pub:faq {{pub -|- faq {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:addfaq {{pub m|- addfaq {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:delfaq {{pub m|- delfaq {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:chfaq {{pub m|- chfaq {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:faqs_chat {{pub -|- faqs {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:faq {{msg -|- faq {} {}}}
  msg:addfaq {{msg m|- addfaq {} {}}}
  msg:delfaq {{msg m|- delfaq {} {}}}
  msg:chfaq {{msg m|- chfaq {} {}}}
  msg:faqs_chat {{msg -|- faqs {} {}}}
  dcc:faq {{dcc -|- faq {} {}}}
  dcc:addfaq {{dcc m|- addfaq {} {}}}
  dcc:delfaq {{dcc m|- delfaq {} {}}}
  dcc:chfaq {{dcc m|- chfaq {} {}}}
  dcc:faqs {{dcc -|- faqs {} {}}}
}

# Help Data
array set helpData {
  pub:faq {{<topic>} {shows the given faq number or topic}}
  pub:addfaq {{<topic> <content>} {adds a topic to the database with the given content}}
  pub:delfaq {{<topic>} {removes a topic from the database}}
  pub:chfaq {{<topic> <content>} {changes the content of a topic}}
  pub:faqs {{} {enters the faq server}}
  msg:faq {{<topic>} {shows the given faq number or topic}}
  msg:addfaq {{<topic> <content>} {adds a topic to the database with the given content}}
  msg:delfaq {{<topic>} {removes a topic from the database}}
  msg:chfaq {{<topic> <content>} {changes the content of a topic}}
  msg:faqs {{} {enters the faq server}}
  dcc:faq {{<topic>} {shows the given faq number or topic}}
  dcc:addfaq {{<topic> <content>} {adds a topic to the database with the given content}}
  dcc:delfaq {{<topic>} {removes a topic from the database}}
  dcc:chfaq {{<topic> <content>} {changes the content of a topic}}
  dcc:faqs {{} {enters the faq server}}
}
