###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: help.tcl,v 1.9 2005/01/04 03:36:43 tothwolf Exp $
##

# name: help
# version: 0.1
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Provides help engine and commands.
# requires: commandChar listModules


# helpData(type:command) {{usage} {{help text para 1} {help text para2}}}




proc loadHelp {module} {
  variable NamespaceParent
  variable HelpData

  if {[info exists ${NamespaceParent}::${module}::helpData]} then {
    foreach {name data} [array get ${NamespaceParent}::${module}::helpData] {
      foreach {usage help} $data {break}
      if {[info exists HelpData($name)]} then {
        set found 0
        set index 0
          if {![string compare $module $moduleTmp]} then {
            set HelpData($name) [lreplace $HelpData($name) $index $index "$module $usage $help"]
            set found 1
            break
          }
          incr index
        }
        if {!$found} then {
          lappend HelpData($name) "$module $usage $help"
        }
      } else {
        set HelpData($name) [list $module $usage $help]
      }
    }
  }
  return
}


proc unloadHelp {module} {
  variable HelpData

  foreach {name data} [array get HelpData] {
    set found 0
    set index 0
    foreach oldhelp $data {
      foreach {moduleTmp usageTmp helpTmp} $oldhelp {break}
      if {![string compare $module $moduleTmp]} then {
        set data [lreplace $data $index $index]
        set found 1
      }
    }
    if {$found} then {
      set HelpData($name) $data
    }
  }
  return
}


#  foreach module [listModules 1] {
#    loadHelp $module
#  }


##########################################################################


##
## Help engine
##
## Args:
## Returns:
##
proc wphelp {type action dest nick uhost hand chan arg} {
  variable NamespaceParent

  set what [lindex [split $arg] 0]

  # Default help listing
  if {(![string compare "" $what]) || (![string compare help $what])} then {
    set lowerWhat [string tolower $what]

    # Loaded modules
    foreach module [listModules 1] {
      if {[info exists ${NamespaceParent}::${module}::helpData]} then {
        foreach command [lsort [array names ${NamespaceParent}::${module}::helpData]] {

          if {[info exists ${NamespaceParent}::${module}::bindDefaults(command)]} then {






    foreach i [lsort [array names ${NamespaceParent}::mod]] {
      foreach j [join [lsort [array names ${NamespaceParent}::${i}::help_${type}]]] {
        set flags [lindex [set ${NamespaceParent}::${i}::help_${type}($j)] 2]
        if {[validuser $hand]} then {
          if {[matchattr $hand $flags]} then {
            lappend list($i) $j
          } elseif {((![string compare pub $type]) || \
                     (![string compare dcc $type])) && \
                    ((([validchan $chan]) && \
                      ([matchattr $hand $flags $chan])) || \
                     ([find_chan_flag $hand $flags]))} then {
            lappend list($i) $j
          }
        } elseif {![string compare "-" [lindex [split $flags |] 0]]} then {
          lappend list($i) $j
        }
      }
    }
    if {[llength [array names list]] >= 1} then {
      put $action $dest "Wolfpack module help:"
      foreach i [lsort [array names list]] {
        put $action $dest "Module $i:"
        formatColumns $action $dest $list($i) 5 13 "  " "  "
      }
    } else {
      return 0
    }


  } else {

    foreach i [lsort [array names ${NamespaceParent}::mod]] {
      set topic [array_find ${NamespaceParent}::${i}::help_${type} $what]
      set ok 0
      if {[string compare "" $topic]} then {
        set flags [lindex [set ${NamespaceParent}::${i}::help_${type}($topic)] 2]
        if {[validuser $hand]} then {
          if {[matchattr $hand $flags]} then {
            set ok 1
            break
          } elseif {((![string compare pub $type]) || \
                     (![string compare dcc $type])) && \
                    ((([validchan $chan]) && \
                      ([matchattr $hand $flags $chan])) || \
                     ([find_chan_flag $hand $flags]))} then {
            set ok 1
            break
          }
        } elseif {![string compare "-" [lindex [split $flags |] 0]]} then {
          set ok 1
          break
        }
      }
    }
    if {$ok} then {
      put $action $dest "###  $topic [lindex [set ${NamespaceParent}::${i}::help_${type}($topic)] 3]"
      formatLists $action $dest [lindex [set ${NamespaceParent}::${i}::help_${type}($topic)] 4] 60 " " " " " " "   " "   "
    } else {
      return 0
    }
  }
  return 1
}

# help frontend; pub
proc pub:help {nick uhost hand chan arg} {
  return [help pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
}

# help frontend; msg
proc msg:help {nick uhost hand arg} {
  return [help msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
}

# help frontend; dcc
proc dcc:help {hand idx arg} {
  return [help dcc putdcc $idx putdcc $idx "" "" $hand [lindex [console $idx] 0] $arg]
}

# help frontend; fil
proc fil:help {hand idx arg} {
  return [help fil putdcc $idx putdcc $idx "" "" $hand "" $arg]
}

# help frontend
proc help {type action dest nick uhost hand chan arg} {
  if {[validHandle $hand]} then {
    set arg [split [string trimright $arg]]
    set mode 0
    set fallthru 0
    set return 1
    switch -regexp [lindex $arg 0] {
      {(^$|^help$)} {
        set mode 0
        set what $arg
      }
      {^eggdrop$} {
        set mode 1
        set what [join [lrange $arg 1 end]]
      }
      {^wolfpack$} { 
        set mode 2
        set what [join [lrange $arg 1 end]]
      }
      default {
        set mode 3
        set what $arg
      }
    }
    if {$mode == 3} then {
      if {![wphelp $type $action $dest $nick $uhost $hand $chan $what]} then {
        set fallthru 1
      }
    }
    if {($mode == 0) || ($mode == 1) || ($fallthru)} then {
      switch -exact $type {
        msg {
          *msg:help $nick $uhost $hand $what
        }
        dcc {
          *dcc:help $hand $dest $what
          set return 0
        }
        fil {
          *fil:help $hand $dest $what
          set return 0
        }
        pub {
          if {$mode == 1} then {
            put $action $dest "No help avaliable on that."
          }
        }
      }
    }
    if {($mode == 0) || ($mode == 2)} then {
      if {![wphelp $type $action $dest $nick $uhost $hand $chan $what]} then {
        put $action $dest "No help available on that."
      }
    }
    return $return
  }
  return 0
}




# Bind defaults
array set bindDefaults {
  pub:help {{pub -|- help {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:help {{msg -|- help {} {}}}
  dcc:help {{dcc -|- help {} {}}}
  fil:help {{fil -|- help {} {}}}
}
