###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: ircaway.tcl,v 1.12 2005/01/04 03:36:43 tothwolf Exp $
##

# name: ircaway
# version: 1.4
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Provides dcc, msg, and secure public ircaway commands that \
#	can be used to set the bot /away.
# requires: authCheck put commandChar


##
## Set away status: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1
##
proc pub:ircaway {nick uhost hand chan arg} {
  return [ircaway pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Set away status: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1
##
proc msg:ircaway {nick uhost hand arg} {
  return [ircaway msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Set away status: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:ircaway {hand idx arg} {
  return [ircaway dcc putdcc $idx "" "" $hand "" $arg]
}

##
## Set away status
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1
##
proc ircaway {type action dest nick uhost hand chan arg} {
  if {[authCheck $type ircaway $nick $uhost $hand $chan]} then {
    set reason [string trimright $arg]
    if {![string compare "" $reason]} then {
      putserv "AWAY :"
      put $action $dest "Bot is no longer marked as away"
    } else {
      putserv "AWAY :$reason"
      put $action $dest "Bot is now marked as away: $reason"
    }
    return 1
  }
  return 0
}

# Bind defaults
array set bindDefaults {
  pub:ircaway {{pub m|- ircaway {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:ircaway {{msg m|- ircaway {} {}}}
  dcc:ircaway {{dcc m|- ircaway {} {}}}
}

# Help Data
array set helpData {
  pub:ircaway {{[reason]} {sets the bot away on irc with the given reason, or unaway if none given}}
  msg:ircaway {{[reason]} {sets the bot away on irc with the given reason, or unaway if none given}}
  dcc:ircaway {{[reason]} {sets the bot away on irc with the given reason, or unaway if none given}}
}
