###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: ircrelay.tcl,v 1.17 2005/01/04 03:36:43 tothwolf Exp $
##

# name: ircrelay
# version: 0.4.6.1
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: ircrelay.conf
# description:
# requires: this_module_needs_to_be_converted commandChar
# provides: irc2dcc dcc2irc


# set data information
set data(relaydata) "1 $conf(relaydatafile) relaydata"

# load data
module_data load "" 1 0 "1 $conf(relaydatafile) relaydata"

# returns dcc channel number associated with the irc channel name
proc irc2dcc {chan} {
  variable conf
  variable relaydata
  if {!$conf(isolate)} then {
    set chan [string tolower $chan]
    foreach i [array names relaydata] {
      if {![string compare $chan [string tolower $i]]} then {
        return $relaydata($i)
      }
    }
  }
  return
}

# returns irc channel name associated with the dcc channel number
proc dcc2irc {chan} {
  variable conf
  variable relaydata
  if {!$conf(isolate)} then {
    foreach i [array names relaydata] {
      if {![string compare $chan $relaydata($i)]} then {
        return $i
      }
    }
  }
  return
}

# tell someone in the irc channel whos in the dcc channel
proc pub:look {nick uhost hand chan arg} {
  if {[validNick $nick]} then {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      foreach i [whom $tmp] {
        lappend list [lindex $i 0],
      }
      if {[info exists list]} then {
        formatLists putnot $nick [string trimright [join $list] ,] 160 , , " " "Users: " "(cont): "
      } else {
        putnot $nick "No users."
      }
    } else {
      putnot $nick "$chan isn't linked."
    }
    return 1
  }
  return 0
}

# tell someone in the dcc channel whos in the irc channel; dcc
proc dcc:look {hand idx arg} {
  global botnick
  set tmp [dcc2irc [getchan $idx]]
  if {[string compare "" $tmp]} then {
    foreach i [chanlist $tmp)] {
      if {[string compare $botnick $i]} then {
        lappend list $i,
      }
    }
    if {[info exists list]} then {
      formatLists putdcc $idx [string trimright [join $list] ,] 160 , , " " "Users: " "(cont): "
    } else {
      putdcc $idx "No users."
    }
  } else {
    putdcc $idx "$chan isn't linked."
  }
  return 1
}

# relay irc channel chat to the dcc channel
proc pubm:relay {nick uhost hand chan text} {
  if {[validNick $nick]} then {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "<<$nick>> $text"
    }
  }
  return 0
}

# relay irc channel actions to the dcc channel
proc ctcp:action_relay {nick uhost hand dest key arg} {
  if {[validNick $nick]} then {
    set tmp [irc2dcc $dest]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "** $nick $arg"
    }
  }
  return 0
}

# relay irc channel joins to the dcc channel
proc join:relay {nick uhost hand chan} {
  if {[validNick $nick]} then {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "$nick joined $chan"
    }
  }
  return 0
}

# relay irc channel parts to the dcc channel
proc part:relay {nick uhost hand chan} {
  if {[validNick $nick]} then {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "$nick left $chan"
    }
  }
  return 0
}

# relay irc channel signoffs to the dcc channel
proc sign:relay {nick uhost hand chan reas} {
  if {[validNick $nick]} then {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "$nick left irc ($reas)"
    }
  }
  return 0
}

# relay irc channel topic changes to the dcc channel
proc topc:relay {nick uhost hand chan topic} {
  set tmp [irc2dcc $chan]
  if {[string compare "" $tmp]} then {
    dccputchan $tmp "$nick set the topic on $chan to $topic"
  }
  return 0
}

# relay irc channel kicks to the dcc channel
proc kick:relay {nick uhost hand chan knick reas} {
  set tmp [irc2dcc $chan]
  if {[string compare "" $tmp]} then {
    dccputchan $tmp "$knick has been kicked off $chan by $nick ($reas)"
  }
  return 0
}

# relay irc channel nick changes to the dcc channel
proc nick:relay {nick uhost hand chan newnick} {
  set tmp [irc2dcc $chan]
  if {[string compare "" $tmp]} then {
    dccputchan $tmp "$nick is now known as $newnick"
  }
  return 0
}

# relay irc channel mode changes to the dcc channel
proc mode:relay {nick uhost hand chan mode arg} {
  set tmp [irc2dcc $chan]
  if {[string compare "" $tmp]} then {
    if {![string compare "" $arg]} then {
      dccputchan $tmp "$nick set mode \"$mode\" on $chan"
    } else {
      dccputchan $tmp "$nick set mode \"$mode $arg\" on $chan"
    }
  }
  return 0
}

# relay dcc channel joins to the irc channel
proc chjn:relay {bot hand chan flag idx from} {
  set tmp [dcc2irc $chan]
  if {[string compare "" $tmp]} then {
    putdcc $idx "You have entered $tmp"
    putmsg $tmp "\002>>>\002 $hand entered the clearing"
  }
  return 0
}

# relay dcc channel parts to the irc channel
proc chpt:relay {bot hand idx chan} {
  set tmp [dcc2irc $chan]
  if {[string compare "" $tmp]} then {
    putdcc $idx "You have left $tmp"
    putmsg $tmp "\002>>>\002 $hand left the clearing"
  }
  return 0
}

# relay dcc channel chat to the irc channel
proc chat:relay {nick chan text} {
  set tmp [dcc2irc $chan]
  if {[string compare "" $tmp]} then {
    putmsg $tmp "\002<\002$nick\002>\002 $text"
  }
  return 0
}

# relay dcc channel actions to the irc channel
proc act:relay {nick chan text} {
  set tmp [dcc2irc $chan]
  if {[string compare "" $tmp]} then {
    putmsg $tmp "\002*\002 $nick $text"
  }
  return 0
}

# relay dcc channel nick changes to the irc channel
proc nkch:relay {nick newnick} {
  set idx [hand2idx $newnick]
  if {[valididx $idx]} then {
    set tmp [dcc2irc [getchan $idx]]
    if {[string compare "" $tmp]} then {
      putmsg $tmp "\002>>>\002 $nick is now known as $newnick"
    }
  }
  return 0
}

# Configuration defaults, values and descriptions
array set configDefaults {
  isolate {switch 0 {} {0/1} {set to 1 to isolate irc <-> dcc traffic}}
  relaydatafile {word ircrelay-chan.data {} file {file to store relay config data in}}
}

# Bind defaults
array set bindDefaults {
  pub:look {{pub -|- look {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  dcc:look {{dcc -|- look {} {}}}
  pubm:relay {{pubm -|- * {} {}}}
  ctcp:action_relay {{ctcp -|- ACTION {} {}}}
  join:relay {{join -|- * {} {}}}
  part:relay {{part -|- * {} {}}}
  sign:relay {{sign -|- * {} {}}}
  topc:relay {{topc -|- * {} {}}}
  kick:relay {{kick -|- * {} {}}}
  nick:relay {{nick -|- * {} {}}}
  mode:relay {{mode -|- * {} {}}}
  chjn:relay {{chjn -|- * {} {}}}
  chpt:relay {{chpt -|- * {} {}}}
  chat:relay {{chat -|- * {} {}}}
  act:relay {{act -|- * {} {}}}
  nkch:relay {{nkch -|- * {} {}}}
}

# Help Data
array set helpData {
  pub:look {{} {shows users who are in dcc}}
  dcc:look {{} {shows users who are in the channel}}
}
