###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: jester.tcl,v 1.16 2005/01/14 06:30:19 tothwolf Exp $
##

# name: jester
# version: 1.0.7.1
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: jester.conf
# description:
# requires: this_module_needs_to_be_converted commandChar


# FIXME: set missing varliables to defaults
if {![info exists chandata_last]} then {set chandata_last ""}
if {![info exists jokedata_last]} then {set jokedata_last ""}
if {![info exists listactive]} then {set listactive 0}

# set data information
set data(chandata) "1 $conf(chandatafile) chandata"
set data(jokedata) "1 $conf(jokedatafile) jokedata"
set data(tempdata) "1 $conf(tempdatafile) tempdata"

# load data
module_data load "" 1 0 "1 $conf(chandatafile) chandata"
module_data load "" 1 0 "1 $conf(jokedatafile) jokedata"
module_data load "" 1 0 "1 $conf(tempdatafile) tempdata"

# look for a place to say our joke, say it if we are already on that channel.
proc annoy_channel {} {
  variable conf
  variable joinchan
  set chan [random_index chandata chandata_last]
  if {[string compare "" $chan]} then {
    if {[validchan $chan]} then {
      set joke [random_index jokedata jokedata_last]
      if {[string compare "" $joke]} then {
        tell_joke $chan $joke
      }
    } else {
      if {[add_channel $chan]} then {
        set joinchan $chan
        timer $conf(delay) [namespace code [list remove_channel $chan]]
      }
    }
  }
  return 0
}

# say our joke when we join a channel
proc join:tell_joke {nick uhost hand chan} {
  global botnick
  variable joinchan
  if {(![string compare $botnick $nick]) && \
      (![string compare [string tolower $joinchan] [string tolower $chan]])} then {
    set joke [random_index jokedata jokedata_last]
    if {[string compare "" $joke]} then {
      utimer 2 [namespace code [list tell_joke $chan $joke]]
    }
    set joinchan ""
  }
  return 0
}

# check number of users in a channel and say our joke
proc tell_joke {chan joke} {
  variable conf
  if {([validchan $chan]) && ([botonchan $chan])} then {
    if {[llength [chanlist $chan]] >= $conf(minusers)} then {
      foreach i $joke {
        putmsg $chan $i
      }
      return 1
    }
  }
  return 0
}

# add a channel and set default options if it dosen't exist
proc add_channel {chan} {
  variable conf
  variable tempdata
  variable tempdata_changed
  if {![validchan $chan]} then {
    lappend tempdata $chan
    set tempdata_changed 1
    channel add $chan
    if 1 "channel set $chan $conf(chanoptions)"
    return 1
  }
  return 0
}

# remove a channel if it exists
proc remove_channel {chan} {
  variable tempdata
  variable tempdata_changed
  if {[validchan $chan]} then {
    set index [lsearch -exact $tempdata $chan]
    if {$index != -1} then {
      channel remove $chan
      set tempdata [lreplace $tempdata $index $index]
      set tempdata_changed 1
      return 1
    }
  }
  return 0
}

# remove channels we shouldn't be in
proc check_channels {} {
  variable tempdata
  variable tempdata_changed
  set tempchans [string tolower [join $tempdata]]
  foreach i [string tolower [channels]] {
    set index [lsearch -exact $tempchans $i]
    if {$index != -1} then {
      channel remove $i
      set tempdata [lreplace $tempdata $index $index]
      set tempdata_changed 1
    }
  }
  return 0
}

# watch for invite requests
proc raw:invite {from key arg} {
  variable conf
  variable joinchan
  variable timers
  set chan [string trimleft [lindex [split $arg] 1] :]
  if {![validchan $chan]} then {
    if {[add_channel $chan]} then {
      set joinchan $chan
      set lchan [string tolower $chan]
      set timers($lchan) [timer $conf(delay) "[namespace code [list remove_channel $chan]]; catch {unset timers($lchan)}"
    }
  }
  return 0
}

# stay if asked
proc pub:stay {nick uhost hand chan arg} {
  variable timers
  if {[isop $nick $chan]} then {
    set lchan [string tolower $chan]
    if {[info exists timers($lchan)]} then {
      killtimer $timers($lchan)
      unset timers($lchan)
    }
  }
  return 1
}

# leave if asked
proc pub:leave {nick uhost hand chan arg} {
  variable tempdata
  variable tempdata_changed
  if {[isop $nick $chan]} then {
    set index [lsearch -exact $tempdata $chan]
    if {$index != -1} then {
      channel remove $i
      set tempdata [lreplace $tempdata $index $index]
      set tempdata_changed 1
    }
  }
  return 1
}

# get a new channel list
proc get_channels {} {
  variable listactive
  set listactive 1
  putserv "LIST"
  return 0
}

# begin of channel list
proc raw:321 {from key arg} {
  variable listactive
  variable listdata
  if {$listactive} then {
    set listdata ""
  }
  return 0
}

# channel list
proc raw:322 {from key arg} {
  variable listactive
  variable listdata
  if {$listactive} then {
    set chan [lindex [split $arg] 1]
    lappend listdata $chan
  }
  return 0
}

# end of channel list
proc raw:323 {from key arg} {
  variable listactive
  variable listdata
  variable chandata
  variable chandata_changed
  if {$listactive} then {
    set listactive 0
    set chandata ""
    foreach i $listdata {
      set ok 1
      foreach j [string tolower $conf(exclude)] {
        if {[regexp -nocase -- ".*$j.*" $i]} then {
          set ok 0
        }
      }
      if {($ok) && ([string compare "*" $chan]) && \
          ((!$conf(maxlength)) || \
           ([string length $i] <= $conf(maxlength)))} then {
        lappend chandata $i
      }
    }
    unset listdata
    set chandata_changed 1
  }
  return 0
}

# find a new channel and annoy it every 5 minutes
proc time:min_5 {min hour day month year} {
  annoy_channel
  return 0
}

# remove channels we shouldn't be in and get a new channel list every 12 hours
proc time:hour_12 {min hour day month year} {
  check_channels
#  get_channels
  return 0
}

# make sure things are set right when loaded
#if {![llength [join $chandata]]} then {
#  utimer 30 [namespace code get_channels]
#}

# Configuration defaults, values and descriptions
array set configDefaults {
  chandatafile {word jester-chan.data {} file {file that will store list channel data}}
  jokedatafile {word jester-joke.data {} file {file that will store joke data}}
  tempdatafile {word jester-temp.data {} file {file that will store temp channel data}}
  delay {integer 1 {} minutes {minutes to delay before removing a channel that was added}}
  minusers {integer 2 {} number {min number of users that must be in channel to tell a joke}}
  exclude {string {} exclude {! boy daughter dog fuck gay girl hack hax lesbian little monkey mp3 porn sex shell son teen wares warez xxx} {keywords to exclude from channel data}}
  maxlength {integer 20 {} length {max length of a channel to join}}
  chanoptions {string {} options {-clearbans -enforcebans +dynamicbans +userbans -autoop -bitch -greet -protectops -statuslog -stopnethack -revenge -secret -shared -autovoice -cycle -seen +dontkickops -wasoptest} {default options for added channels}}
}

# Bind defaults
array set bindDefaults {
  pub:stay {{pub -|- stay {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:leave {{pub -|- leave {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  join:tell_joke {{join -|- * {} {}}}
  raw:invite {{raw -|- INVITE {} {}}}
  raw:321 {{raw -|- 321 {} {}}}
  raw:322 {{raw -|- 322 {} {}}}
  raw:323 {{raw -|- 323 {} {}}}
  time:min_5 {{time -|- {?0 * * * *} {} {}} {time -|- {?5 * * * *} {} {}}}
  time:hour_12 {{time -|- {00 12 * * *} {} {}} {time -|- {00 00 * * *} {} {}}}
}

# Help Data
array set helpData {
  pub:stay {{} {make bot stay in this channel}}
  pub:leave {{} {make bot leave this channel}}
}
