###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: moonphase.tcl,v 1.18 2005/01/04 03:36:43 tothwolf Exp $
##

# name: moonphase
# version: 1.6.6.3
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: moonphase.conf
# description:
# requires: this_module_needs_to_be_converted commandChar


# set data information
set data(moondata) "1 $conf(moondatafile) moondata"

# load data
module_data load "" 1 0 "1 $conf(moondatafile) moondata"

# show moon data; pub
proc pub:moon {nick uhost hand chan arg} {
  return [moon pub putmsg $chan $nick $uhost $hand $chan $arg]
}

# show moon data; msg
proc msg:moon {nick uhost hand arg} {
  return [moon msg putnot $nick $nick $uhost $hand "" $arg]
}

# show moon data; dcc
proc dcc:moon {hand idx arg} {
  return [moon dcc putdcc $idx "" "" $hand "" $arg]
}

# show moon data
proc moon {type action dest nick uhost hand chan arg} {
  variable moondata
  if {[validHandle $hand]} then {
    if {(![string compare none $moondata(face)]) && \
        (![string compare none $moondata(phase)])} then {
      put $action $dest "Phase of the Moon for $moondata(date): Retrieving Data..."
    } elseif {$moondata(found)} then {
      put $action $dest "Phase of the Moon for $moondata(date): $moondata(face) with $moondata(percent) of the Moon's surface illuminated."
    } else {
      put $action $dest "Phase of the Moon for $moondata(date): $moondata(phase)"
      if {(![string compare "Full Moon" $moondata(phase)]) && \
          ([string compare "" $chan])} then {
#FIXME: redo
#        catch {
#          sound:send $chan "[sound:random howl] howls at the full moon"
#        }
      }
    }
    return 1
  }
  return 0
}

# get moon data; pub
proc pub:getmoon {nick uhost hand chan arg} {
  if {[auth_check pub $nick $uhost $hand $chan getmoon]} then {
    putnot $nick "Retrieving moon phase data..."
    getmoon
    return 1
  }
  return 0
}

# get moon data; msg
proc msg:getmoon {nick uhost hand arg} {
  if {[auth_check pub $nick $uhost $hand $chan getmoon]} then {
    putnot $nick "Retrieving moon phase data..."
    getmoon
    return 1
  }
  return 0
}

# get moon data; dcc
proc dcc:getmoon {hand idx arg} {
  putdcc $idx "Retrieving moon phase data..."
  getmoon
  return 1
}

# get moon data
proc getmoon {} {
  variable conf
  variable moondata
  variable NamespaceParent

  set time [clock format [clock seconds] -format "%d %m %Y"]
  set day [lindex $time 0]
  set month [lindex $time 1]
  set year [lindex $time 2]
  set content(string) "FFX=2&ID=WP&y=$year&m=$month&d=$day&xx0=$conf(xx0)&xx1=$conf(xx1)&xx2=$conf(xx2)&yy0=$conf(yy0)&yy1=$conf(yy1)&yy2=$conf(yy2)&zz0=$conf(zz0)&zz1=$conf(zz1)&place=$conf(place)&ZZZ=END"
  regsub -all -- { } $content(string) {+} content(string)
  set content(length) [string length $content(string)]
  set moondata(date) [clock format [clock seconds] -format "%a %b %d %Y"]
  set moondata(found) 0
  set moondata(face) none
  set moondata(percent) 0%
  set moondata(phase) none
  if {$conf(debug)} then {
    set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] w]
    close $fd
  }
  if {[catch {set idx [connect $conf(host) $conf(port)]} result]} then {
    wpLog o * "Error: unable to connect to $conf(host):$conf(port)': $result"
  } else {
    control $idx [namespace code control_getmoon]
    putdcc $idx "POST $conf(path) HTTP/1.0"
    putdcc $idx "Pragma: no-cache"
    putdcc $idx "From: [contact_address]"
    putdcc $idx "User-Agent: wolfpack.tcl\[moonphase.tcl\]/ver"
    putdcc $idx "Content-Length: $content(length)"
    putdcc $idx ""
    putdcc $idx "$content(string)"
    putdcc $idx ""
    return 1
  }
  return 0
}

# grab the data we want
proc control_getmoon {idx text} {
  variable conf
  variable moondata
  variable moondata_changed
  variable NamespaceParent
  set strip "<p> </p> &nbsp;"
  set newtext [replaceExpr [replaceExpr $text $strip] "  " " "]
  if {(!$moondata(found)) && \
      ([regexp -nocase -- {.*Phase of the Moon.*} $newtext])} then {
    set moondata(face) [lrange $newtext 7 8]
    set moondata(found) 1
    set moondata_changed 1
  }
  if {($moondata(found)) && \
      ([regexp -nocase -- {.*% of the Moon.*} $newtext])} then {
    set moondata(percent) [lindex $newtext 1]
    set moondata_changed 1
  }
  if {(!$moondata(found)) && \
      ([regexp -nocase -- {.*New Moon.*} $newtext])} then {
    set moondata(phase) "New Moon"
    set moondata_changed 1
  }
  if {(!$moondata(found)) && \
      ([regexp -nocase -- {.*First quarter.*} $newtext])} then {
    set moondata(phase) "First quarter"
    set moondata_changed 1
  }
  if {(!$moondata(found)) && \
      ([regexp -nocase -- {.*Full Moon.*} $newtext])} then {
    set moondata(phase) "Full Moon"
    set moondata_changed 1
  }
  if {(!$moondata(found)) && \
      ([regexp -nocase -- {.*Last quarter.*} $newtext])} then {
    set moondata(phase) "Last quarter"
    set moondata_changed 1
  }
  if {$conf(debug)} then {
    set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] a]
    puts $fd $text
    close $fd
  }
}

# check/update moon data once an hour
proc time:hourly {min hour day month year} {
  variable moondata

  set date [clock format [clock seconds] -format "%a %b %d %Y"]
  if {(![info exists moondata(date)]) || \
      ([string compare $date $moondata(date)])} then {
    getmoon
  }
  return 0
}


############################################################

# Configuration defaults, values and descriptions
array set configDefaults {
  moondatafile {word moonphase-cache.data {} file {file that will store the moon phase data}}
  debug {switch 0 {} {0/1} {write to a debug file (this WILL slow things down)}}
  debugfile {word moonphase.debug {} file {file to write raw html to}}
  host {word www.usno.navy.mil {} host {http server host}}
  path {word /cgi-bin/aa_pap {} url {cgi url for post}}
  place {string "Jasper National Park" {} name {(Place Name) name of place}}
  port {integer 80 {} port {http server port}}
  xx0 {integer -1 {} {east/west} {(Longitude) \"1\" east, \"-1\" west}}
  xx1 {integer 118 {} degrees {(Longitude) \"xxx\" degrees}}
  xx2 {integer 01 {} minutes {(Longitude) \"xx\" minutes}}
  yy0 {integer 1 {} {north/south} {(Latitude) \"1\" north, \"-1\" south}}
  yy1 {integer 52 {} degrees {(Latitude) \"yy\" degrees}}
  yy2 {integer 04 {} minutes {(Latitude) \"yy\" minutes}}
  zz0 {integer -1 {} {east/west} {(Time Zone) \"1\" east of Greenwich, \"-1\" west of Greenwich}}
  zz1 {integer 08 {} hour {(Time Zone) \"00\" hour}}
}

# Bind defaults
array set bindDefaults {
  pub:moon {{pub -|- moon {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:getmoon {{pub m|- getmoon {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:moon {{msg -|- moon {} {}}}
  msg:getmoon {{msg m|- getmoon {} {}}}
  dcc:moon {{dcc -|- moon {} {}}}
  dcc:getmoon {{dcc m|- getmoon {} {}}}
  time:hourly {{time -|- {00 * * * *} {} {}}}
}

# Help Data
array set helpData {
  pub:moon {{} {shows current moon phase}}
  pub:getmoon {{} {updates moon phase data}}
  msg:moon {{} {shows current moon phase}}
  msg:getmoon {{} {updates moon phase data}}
  dcc:moon {{} {shows current moon phase}}
  dcc:getmoon {{} {updates moon phase data}}
}
