###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: netserv.tcl,v 1.12 2005/01/04 03:36:43 tothwolf Exp $
##

# name: netserv
# version: 1.0
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: netserv.conf
# description: Adds the bot's server to the network variable (40 chars max) \
#	so that the `.botinfo' command will display the bot's current server.
# provides: updateNetwork


##
## Update $network variable to show bot's current server
##
## Args:    none
## Returns: nothing
##
proc updateNetwork {} {
  global network server
  variable configData

  if {([info exists network]) && ([info exists server])} then {
    if {($configData(keepnetwork)) && \
        ([string compare "" $network])} then {
      set newnetwork [lindex [split $network :] 0]
      if {[string compare "" $server]} then {
        append newnetwork ": [lindex [split $server :] 0]"
      } elseif {[llength $network] >= 2} then {
        append newnetwork ": (no server)"
      }
      if {[string compare $newnetwork $network]} then {
        set network [string range $newnetwork 0 39]
      }
    } else {
      set network [string range $server 0 39]
    }
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  global init-server

  if {(![info exists init-server]) || \
      (![string compare "" ${init-server}])} then {
    set init-server [namespace code updateNetwork]
  } elseif {[lsearch -exact ${init-server} \
             [namespace code updateNetwork]] == -1} then {
    append init-server " ; [namespace code updateNetwork]"
  }
  return
}

##
## Destroy module specific things before unload
##
## Args:    none
## Returns: nothing
##
proc ModuleDestroy {} {
  global network server init-server

  set index [lsearch -exact ${init-server} \
             [namespace code updateNetwork]]
  if {$index != -1} then {
    set sindex [expr $index - 1]
    if {[string compare ";" [lindex ${init-server} $sindex]]} then {
      set sindex $index
    }
    set init-server [lreplace ${init-server} $sindex $index ""]
  }
  if {([info exists network]) && \
      ([string compare "" $network])} then {
    set tmp [split $network :]
    set oldserver [lindex $tmp 1]
    if {(![string compare $oldserver [lindex [split $server :] 0]]) || \
        (![string compare $oldserver "(no server)"])} then {
      set network [lindex $tmp 0]
    }
  }
  return
}

# Configuration defaults, values and descriptions
array set configDefaults {
  keepnetwork {switch 1 {} {0/1} {keep irc network name in the network var when adding server name?}}
}
