###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: portctrl.tcl,v 1.14 2005/01/04 03:36:43 tothwolf Exp $
##

# name: portctrl
# version: 1.2
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: portctrl.conf
# description: Adds a dcc command called `.portctrl' that can be used to \
#	enable/disable the bot's listen port(s).
# requires: listListenPorts utimerId dccCheckOwner
# provides: portControl


##
## Enable/disable listen ports: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:portctrl {hand idx arg} {
  variable configData

  if {![dccCheckOwner $hand $idx]} then {
    return 0
  }

  set arg [string trim [lindex [split $arg] 0]]
  set types [join [lrange $arg 1 end]]
  if {![string compare "" $types]} then {
    set types $configData(types)
  }

  switch -exact -- [string tolower [lindex $arg 0]] {
    enable {
      portControl enable $types
      putdcc $idx "Listen port types enabled: $types"
    }
    disable {
      portControl disable $types
      putdcc $idx "Listen port types disabled: $types"
    }
    default {
      putdcc $idx "Usage: portctrl <enable|disable> \[port types\]"
      putdcc $idx "Default types: $configData(types)"
    }
  }
  return 1
}

##
## Enable/disable listen ports
##
## Args:    mode
## Returns: nothing
##
proc portControl {mode {types ""}} {
  variable configData
  variable DisabledPorts

  if {![string compare "" $types]} then {
    set types $configData(types)
  }
  switch -exact -- $mode {
    enable {
      if {[info exists DisabledPorts]} then {
        if {($configData(timeout)) && \
            ([string compare "" [set timerid [utimerId [namespace code "portControl enable"]]]])} then {
          killutimer $timerid
        }
        foreach list $DisabledPorts {
          foreach {port type mask} $list {break}
          if {[info exists mask]} then {
            listen $port $type $mask
          } else {
            listen $port $type
          }
        }
        unset DisabledPorts
      }
    }
    disable {
      foreach list [listListenPorts] {
        if {([lsearch -exact $DisabledPorts $list] == -1) && \
            ([lsearch -exact $types [lindex $list 1]] != -1)} then {
          lappend DisabledPorts $list
          listen [lindex $list 0] off
        }
      }
      if {$configData(timeout)} then {
        if {[string compare "" [set timerid [utimerId [namespace code "portControl enable"]]]]} then {
          killutimer $timerid
        }
        utimer $configData(timeout) [namespace code [list portControl enable]]
      }
    }
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  variable DisabledPorts

  if {![info exists DisabledPorts]} then {
    set DisabledPorts ""
  }
  return
}

# Configuration defaults, values and descriptions
array set configDefaults {
  types {string {bots all} {} types {what listen port types should be disabled?}}
  timeout {integer 600 {} seconds {enable listen ports after how many seconds? (0 will disable)}}
}

# Bind defaults
array set bindDefaults {
  dcc:portctrl {{dcc n|- portctrl {} {}}}
}

# Help Data
array set helpData {
  dcc:portctrl {{<enable|disable> [types]} {enable/disable listen ports}}
}
