###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 2003-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: tclutils.tcl,v 1.10 2005/01/04 03:36:44 tothwolf Exp $
##

# name: tclutils
# version: 0.1
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Adds useful tcl utility type commands to eggdrop
# requires: listBinds whichCommand dccCheckOwner


##
## Show binds matching the given type and mask
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:tcllistbinds {hand idx arg} {
  if {![dccCheckOwner $hand $idx]} then {
    return 0
  }

  set arg [split [string trimright $arg]]
  set type [lindex $arg 0]
  set mask [lindex $arg 1]

  if {![string compare "" $type]} then {
    putdcc $idx "Usage: tcllistbinds <type> \[command mask\]"
  } else {
    if {![string compare "" $mask]} then {
      set mask "*"
    }
    set binds [listBinds $type $mask]
    if {[string compare "" $binds]} then {
      putdcc $idx "Binds matching type '$type', mask '$mask':"
      foreach bind $binds {
        putdcc $idx "  $bind"
      }
    } else {
      putdcc $idx "No binds matched type '$type' mask '$mask'."
    }
  }
  return 1
}

##
## Show full namespace for a given command
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:tclwhichcommand {hand idx arg} {
  if {![dccCheckOwner $hand $idx]} then {
    return 0
  }

  set command [lindex [split [string trimright $arg]] 0]

  if {![string compare "" $command]} then {
    putdcc $idx "Usage: tclwhichcommand <command>"
  } else {
    set which [whichCommand $command]
    if {[string compare "" $which]} then {
      putdcc $idx "$command -> $which"
    } else {
      putdcc $idx "Command '$command' not found."
    }
  }
  return 1
}

# Bind defaults
array set bindDefaults {
  dcc:tcllistbinds {{dcc n tcllistbinds {} {}}}
  dcc:tclwhichcommand {{dcc n tclwhichcommand {} {}}}
}

# Help Data
array set helpData {
  dcc:tcllistbinds {{<type> [mask]} {show binds matching the given type and mask}}
  dcc:tclwhichcommand {{<command>} {show full namespace for a given command}}
}
