###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wakeup.tcl,v 1.15 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wakeup
# version: 1.4
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: wakeup.conf
# description: Anti-idle for bots that uses a /msg of `wakeup' followed by \
#	a random string of 30-40 characters.
# requires: putmsg randstring utimerExists utimerId moduleBindUnbind


##
## Ignore wakeup message
##
## Args:    nick, uhost, handle, arg
## Returns: 0
##
proc msg:wakeupkey {nick uhost hand arg} {
  return 0
}

##
## Send a message to a bot
##
## Args:    none
## Returns: nothing
##
proc AntiIdle {} {
  global botnick
  variable configData

  # Random text to send out
  set text [randstring [expr [rand $configData(textoffset)] + $configData(textlength)] "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789`~!@#$%^&*()-_=+\\|\[\{\]\}:;'\",<.>/? \t"]

# FIXME: negotiate with bots over the botnet and
#        don't send to just any random bot
# FIXME: randomize 'foreach bot $bots'

  # Search all channels for a bot, send a message if we find one
  foreach chan [channels] {
    if {[string compare "" [set bots [chanlist $chan b]]]} then {
      foreach bot $bots {
        if {![isbotnick $bot]} then {

          # Send a message to the bot we found
          putmsg $bot "$configData(wakeupkey) $text"

          # Start another loop
          StartAntiIdle
          return
        }
      }
    }
  }

  # Send a message to ourself as a last resort
  putmsg $botnick "$configData(wakeupkey) $text"

  # Start another loop
  StartAntiIdle
  return
}

##
## Start an anti-idle loop
##
## Args:    none
## Returns: nothing
##
proc StartAntiIdle {} {
  variable configData

  if {![utimerExists [namespace code AntiIdle]]} then {
    if {$configData(offset)} then {
      utimer [expr $configData(delay) + [rand $configData(offset)]] [namespace code AntiIdle]
    } else {
      utimer $configData(delay) [namespace code AntiIdle]
    }
  }
  return
}

##
## Stop anti-idle loop
##
## Args:    none
## Returns: nothing
##
proc StopAntiIdle {} {
  if {[string compare "" [set timerid [utimerId [namespace code AntiIdle]]]]} then {
    killutimer $timerid
  }
  return
}

##
## Trace configuration changes
##
## Args:    name1, name2, operation
## Returns: nothing
##
proc TraceConfigData {name1 name2 op} {
  variable ModuleName
  variable configData
  variable oldConfigData

  if {[string compare $oldConfigData($name2) $configData($name2)]} then {
    switch -exact -- $name2 {
      wakeupkey {
        moduleBindUnbind unbind $ModuleName "option noauto" [list regsub "%wakeupkey% $oldConfigData(wakeupkey)"]
        moduleBindUnbind bind $ModuleName "option noauto" [list regsub "%wakeupkey% $configData(wakeupkey)"]
      }
    }
    set oldConfigData($name2) $configData($name2)
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  variable ModuleName
  variable configData
  variable oldConfigData

  foreach {name data} [array get configData] {
    if {![info exists oldConfigData($name)]} then {
      set oldConfigData($name) $data
    }
  }
  moduleBindUnbind bind $ModuleName "option noauto" [list regsub "%wakeupkey% $configData(wakeupkey)"]
  StartAntiIdle
  trace variable configData w [namespace code TraceConfigData]
  return
}

##
## Destroy module specific things before unload
##
## Args:    none
## Returns: nothing
##
proc ModuleDestroy {} {
  variable ModuleName
  variable configData

  moduleBindUnbind unbind $ModuleName "option noauto" [list regsub "%wakeupkey% $configData(wakeupkey)"]
  StopAntiIdle
  trace vdelete configData w [namespace code TraceConfigData]
  return
}

# Configuration defaults, values and descriptions
array set configDefaults {
  delay {integer 200 {} seconds {time in seconds added to random offset}}
  offset {integer 300 {} seconds {number of seconds used to generate random offset (0 to disable)}}
  textlength {integer 11 {} characters {number of random characters to send}}
  textoffset {integer 30 {} characters {additional number of random offset characters to send}}
  wakeupkey {word wakeup {} keyword {keyword to bind and send with random text}}
}

# Bind defaults
array set bindDefaults {
  msg:wakeupkey {{msg -|- %wakeupkey% noauto {}}}
}
