###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: weather.tcl,v 1.18 2005/01/04 03:36:44 tothwolf Exp $
##

# name: weather
# version: 1.5.6.3
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: weather.conf
# description:
# requires: this_module_needs_to_be_converted commandChar


# set data information
set data(weathdata) "1 $conf(weathdatafile) weathdata"

# load data
module_data load "" 1 0 "1 $conf(weathdatafile) weathdata"

# show weather data; pub
proc pub:weath {nick uhost hand chan arg} {
  return [weath pub putmsg $chan $nick $uhost $hand $chan $arg]
}

# show weather data; msg
proc msg:weath {nick uhost hand arg} {
  return [weath msg putnot $nick $nick $uhost $hand "" $arg]
}

# show weather data; dcc
proc dcc:weath {hand idx arg} {
  return [weath dcc putdcc $idx "" "" $hand "" $arg]
}

# show weather data
proc weath {type action dest nick uhost hand chan arg} {
  variable weathdata

  if {([validHandle $hand]) && ([validNick $nick])} then {
    put $action $dest "Last Update: [clock format $weathdata(time)], Temperature: $weathdata(temperature), Humidity: $weathdata(humidity), Dewpoint: $weathdata(dewpoint), Wind: $weathdata(wind), Pressure: $weathdata(pressure), Conditions: $weathdata(conditions)"
    return 1
  }
  return 0
}

# show temperature; pub
proc pub:temp {nick uhost hand chan arg} {
  return [temp pub putmsg $chan $nick $uhost $hand $chan $arg]
}

# show temperature; msg
proc msg:temp {nick uhost hand arg} {
  return [temp msg putnot $nick $nick $uhost $hand "" $arg]
}

# show temperature; dcc
proc dcc:temp {hand idx arg} {
  return [temp dcc putdcc $idx "" "" $hand "" $arg]
}

# show temperature
proc temp {type action dest nick uhost hand chan arg} {
  variable weathdata

  if {([validHandle $hand]) && ([validNick $nick])} then {
    put $action $dest "Current temperature: $weathdata(temperature)"
    return 1
  }
  return 0
}

# show wind speed; pub
proc pub:wind {nick uhost hand chan arg} {
  return [wind pub putmsg $chan $nick $uhost $hand $chan $arg]
}

# show wind speed; msg
proc msg:wind {nick uhost hand arg} {
  return [wind msg putnot $nick $nick $uhost $hand "" $arg]
}

# show wind speed; dcc
proc dcc:wind {hand idx arg} {
  return [wind dcc putdcc $idx "" "" $hand "" $arg]
}

# show wind speed
proc wind {type action dest nick uhost hand chan arg} {
  variable weathdata

  if {([validHandle $hand]) && ([validNick $nick])} then {
    put $action $dest "Current wind conditions: $weathdata(wind)"
    return 1
  }
  return 0
}

# show partial weather data on join
proc join:weath {nick uhost hand chan} {
  variable weathdata

  if {([validHandle $hand]) && ([validNick $nick])} then {
    putnot $nick "The temperature is $weathdata(temperature), and the conditions are $weathdata(conditions)."
  }
  return 0
}

# get weather data; pub
proc pub:getweath {nick uhost hand chan arg} {
  if {[auth_check pub $nick $uhost $hand $chan getweath]} then {
    putnot $nick "Retrieving weather data..."
    getweath
    return 1
  }
  return 0
}

# get weather data; msg
proc msg:getweath {nick uhost hand arg} {
  if {[auth_check msg $nick $uhost $hand "" getweath]} then {
    putnot $nick "Retrieving weather data..."
    getweath
    return 1
  }
  return 0
}

# get weather data; dcc
proc dcc:getweath {hand idx arg} {
  putdcc $idx "Retrieving weather data..."
  getweath
  return 1
}

# get weather data
proc getweath {} {
  variable conf
  variable weathdata
  variable weathcount
  variable NamespaceParent

  if {[info exists weathdata(time)]} then {
    set weathdata(time_last) $weathdata(time)
  } else {
    set weathdata(time_last) [clock seconds]
  }
  set weathdata(time) [clock seconds]
  set weathcount(current) 0
  set weathcount(next) 0
  set weathcount(item) none
  if {$conf(debug)} then {
    set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] w]
    close $fd
  }
  if {[catch {set idx [connect $conf(host) $conf(port)]} result]} then {
    wpLog o * "Error: unable to connect to `$conf(host):$conf(port)': $result"
  } else {
    control $idx [namespace code control_getweath]
    putdcc $idx "GET $conf(path) HTTP/1.0"
    putdcc $idx "Pragma: no-cache"
    putdcc $idx "From: [contact_address]"
    putdcc $idx "User-Agent: wolfpack.tcl\[weather.tcl\]/ver"
    putdcc $idx ""
    return 1
  }
  return 0
}

# grab the data we want
proc control_getweath {idx text} {
  variable conf
  variable weathdata
  variable weathdata_changed
  variable weathcount
  variable NamespaceParent

  set strip "</tr> <td> </td> <b> </b> &#176;"
  incr weathcount(current)
  if {([string compare none $weathcount(item)]) && \
      ($weathcount(current) == $weathcount(next))} then {
    if {[info exists weathdata($weathcount(item))]} then {
      set weathdata($weathcount(item)_last) $weathdata($weathcount(item))
    } else {
      set weathdata($weathcount(item)_last) none
    }
    set weathdata($weathcount(item)) [replaceExpr [replaceExpr $text $strip] "  " " "]
    set weathdata_changed 1
  }
  if {[regexp -nocase -- {.*<td>temperature</td>.*} $text]} then {
    set weathcount(item) temperature
    set weathcount(next) [expr $weathcount(current) + 1]
  } elseif {[regexp -nocase -- {.*<td>humidity</td>.*} $text]} then {
    set weathcount(item) humidity
    set weathcount(next) [expr $weathcount(current) + 1]
  } elseif {[regexp -nocase -- {.*<td>dewpoint</td>.*} $text]} then {
    set weathcount(item) dewpoint
    set weathcount(next) [expr $weathcount(current) + 1]
  } elseif {[regexp -nocase -- {.*<td>wind</td>.*} $text]} then {
    set weathcount(item) wind
    set weathcount(next) [expr $weathcount(current) + 1]
  } elseif {[regexp -nocase -- {.*<td>pressure</td>.*} $text]} then {
    set weathcount(item) pressure
    set weathcount(next) [expr $weathcount(current) + 1]
  } elseif {[regexp -nocase -- {.*<td>conditions</td>.*} $text]} then {
    set weathcount(item) conditions
    set weathcount(next) [expr $weathcount(current) + 1]
  }
  if {$conf(debug)} then {
    set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] a]
    puts $fd $text
    close $fd
  }
}

# show if the weather has changed
proc show_changes {chan} {
  variable weathdata
  set reply ""
  if {[string compare $weathdata(conditions_last) $weathdata(conditions)]} then {
    lappend reply "Conditions have changed to: $weathdata(conditions),"
  }
  if {[string compare $weathdata(temperature_last) $weathdata(temperature)]} then {
    lappend reply "Temperature has changed to: $weathdata(temperature),"
  }
  if {[string compare $weathdata(wind_last) $weathdata(wind)]} then {
    lappend reply "Wind has changed to: $weathdata(wind),"
  }
  if {[string compare "" $reply]} then {
    putmsg $chan [string trimright [join $reply] ,]
  }
}

# get current weather data every 15 minutes
proc time:min_15 {min hour day month year} {
  variable conf

  get_weathdata
  if {[validchan $conf(channel)]} then {
    utimer 30 [namespace code [list show_changes $conf(channel)]]
  }
  return 0
}

# Configuration defaults, values and descriptions
array set configDefaults {
  channel {{word {regexp {^#}}} #nochannel {} channel {channel name that will get weather updates}}
  debug {switch 0 {} {0/1} {write to a debug file (this WILL slow things down)}}
  debugfile {word weather.debug {} file {file to write raw html to}}
  host {word www.wunderground.com {} host {http server host}}
  path {word /global/stations/71486.html {} url {url for data}}
  port {integer 80 {} port {http server port}}
  weathdatafile {word weather-cache.data {} file {file that weather data is stored in}}
}

# Bind defaults
array set bindDefaults {
  pub:weath {{pub -|- weath {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:temp {{pub -|- temp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:wind {{pub -|- wind {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:getweath {{pub m|- getweath {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:weath {{msg -|- weath {} {}}}
  msg:temp {{msg -|- temp {} {}}}
  msg:wind {{msg -|- wind {} {}}}
  msg:getweath {{msg m|- getweath {} {}}}
  dcc:weath {{dcc -|- weath {} {}}}
  dcc:temp {{dcc -|- temp {} {}}}
  dcc:wind {{dcc -|- wind {} {}}}
  dcc:getweath {{dcc m|- getweath {} {}}}
  join:weath {{join -|- * {} {}}}
  time:min_15 {{time -|- {00 * * * *} {} {}} {time -|- {15 * * * *} {} {}} {time -|- {30 * * * *} {} {}} {time -|- {45 * * * *} {} {}}}
}

# Help Data
array set helpData {
  pub:weath {{} {shows current weahter conditions}}
  pub:temp {{} {shows current temperature}}
  pub:wind {{} {shows current wind speed and direction}}
  pub:getweath {{} {retrieves weather data}}
  msg:weath {{} {shows current weahter conditions}}
  msg:temp {{} {shows current temperature}}
  msg:wind {{} {shows current wind speed and direction}}
  msg:getweath {{} {retrieves weather data}}
  dcc:weath {{} {shows current weahter conditions}}
  dcc:temp {{} {shows current temperature}}
  dcc:wind {{} {shows current wind speed and direction}}
  dcc:getweath {{} {retrieves weather data}}
}
