###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: dice.tcl,v 1.21 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wrpg-dice
# version: 1.5
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: wrpg-dice.conf
# description: Wolf RPG dice roll command and functions.
# requires: commandChar putmsg isNumber floodCheck
# provides: diceConfig


##
## Dice roll
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1
##
proc pub:roll {nick uhost hand chan arg} {
  variable DiceConfig

# FIXME: should flood checking even be used here?
#  if {[floodCheck $nick $uhost roll]} then {
# FIXME: this should be configurable
    if {[validuser $hand]} then {
      set user $hand
    } else {
      set user $nick
    }
    set count 0
    while {$count < $DiceConfig(dice)} {
      set result [expr [rand $DiceConfig(faces)] + 1]
      if {($result >= $DiceConfig(low1_1)) && ($result <= $DiceConfig(low1_2)) && ($DiceConfig(lowodds1_1))} then {
        if {[expr ([rand $DiceConfig(lowodds1_2)] + 1) != $DiceConfig(lowodds1_1)]} then {
          continue
        }
      } elseif {($result >= $DiceConfig(low2_1)) && ($result <= $DiceConfig(low2_2)) && ($DiceConfig(lowodds2_1))} then {
        if {[expr ([rand $DiceConfig(lowodds2_2)] + 1) != $DiceConfig(lowodds2_1)]} then {
          continue
        }
      } elseif {($result >= $DiceConfig(high1_1)) && ($result <= $DiceConfig(high1_2)) && ($DiceConfig(highodds1_1))} then {
        if {[expr ([rand $DiceConfig(highodds1_2)] + 1) != $DiceConfig(highodds1_1)]} then {
          continue
        }
      } elseif {($result >= $DiceConfig(high2_1)) && ($result <= $DiceConfig(high2_2)) && ($DiceConfig(lowodds2_1))} then {
        if {[expr ([rand $DiceConfig(highodds2_2)] + 1) != $DiceConfig(highodds2_1)]} then {
          continue
        }
      }
      lappend results $result
      incr count
    }
    if {([string compare "" $results]) && \
        ((![string compare "" $DiceConfig(use)]) || \
         (![$DiceConfig(use) $nick $uhost $hand $chan $results]))} then {
      foreach result $results {
        lappend reply "$DiceConfig(bold)$result$DiceConfig(bold),"
      }
      putmsg $chan "$user rolls a [string trimright $reply ,]"
    }
    return 1
#  }
#  return 0
}

##
## Configure dice settings and values
##
## Args:    what {reset|use|config}, args {options|command}
## Returns: nothing
##
proc diceConfig {what args} {
  variable configData
  variable DiceConfig

  set options [lindex $args 0]
  switch -exact -- $what {
    reset {
      if {$configData(bold)} then {
        set DiceConfig(bold) "\002"
      } else {
        set DiceConfig(bold) ""
      }
      set DiceConfig(use) ""
      set DiceConfig(dice) $configData(dice)
      set DiceConfig(faces) $configData(faces)
      foreach option "low1 low2 high1 high2 lowodds1 lowodds2 highodds1 highodds2" {
        foreach "DiceConfig(${option}_1) DiceConfig(${option}_2)" [split $configData($option) :] {break}
      }
    }
    use {
      set DiceConfig(use) $options
    }
    config {
      if {([llength $options] == 16) && ([isNumber [join $options ""]])} then {
        foreach "DiceConfig(dice) DiceConfig(faces) \
                 DiceConfig(low1_1) DiceConfig(low1_2) \
                 DiceConfig(low2_1) DiceConfig(low2_2) \
                 DiceConfig(high1_1) DiceConfig(high1_2) \
                 DiceConfig(high2_1) DiceConfig(high2_2) \
                 DiceConfig(lowodds1_1) DiceConfig(lowodds1_2) \
                 DiceConfig(lowodds2_1) DiceConfig(lowodds2_2) \
                 DiceConfig(highodds1_1) DiceConfig(highodds1_2) \
                 DiceConfig(highodds2_1) DiceConfig(highodds2_2)" $options {break}
      }
    }
  }
  return
}

##
## Trace configuration changes
##
## Args:    name1, name2, operation
## Returns: nothing
##
proc TraceConfigData {name1 name2 op} {
  variable configData
  variable oldConfigData
  variable DiceConfig

  if {[string compare $oldConfigData($name2) $configData($name2)]} then {
    switch -exact -- $name2 {
      bold {
        if {$configData($name2)} then {
          set DiceConfig(bold) "\002"
        } else {
          set DiceConfig(bold) ""
        }
      }
      dice {
        set DiceConfig(dice) $configData(dice)
      }
      faces {
        set DiceConfig(faces) $configData(faces)
      }
      high1 {
        scan $configData(high1) "%\[^:\]:%s" DiceConfig(high1_1) DiceConfig(high1_2)
      }
      high2 {
        scan $configData(high2) "%\[^:\]:%s" DiceConfig(high2_1) DiceConfig(high2_2)
      }
      highodds1 {
        scan $configData(highodds1) "%\[^:\]:%s" DiceConfig(lowodds1_1) DiceConfig(lowodds1_2)
      }
      highodds2 {
        scan $configData(highodds2) "%\[^:\]:%s" DiceConfig(lowodds2_1) DiceConfig(lowodds2_2)
      }
      low1 {
        scan $configData(low1) "%\[^:\]:%s" DiceConfig(low1_1) DiceConfig(low1_2)
      }
      low2 {
        scan $configData(low2) "%\[^:\]:%s" DiceConfig(low2_1) DiceConfig(low2_2)
      }
      lowodds1 {
        scan $configData(lowodds1) "%\[^:\]:%s" DiceConfig(highodds1_1) DiceConfig(highodds1_2)
      }
      lowodds2 {
        scan $configData(lowodds2) "%\[^:\]:%s" DiceConfig(highodds2_1) DiceConfig(highodds2_2)
      }
    }
    set oldConfigData($name2) $configData($name2)
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  variable configData
  variable oldConfigData

  foreach {name data} [array get configData] {
    if {![info exists oldConfigData($name)]} then {
      set oldConfigData($name) $data
    }
  }
  diceConfig reset
  trace variable configData w [namespace code TraceConfigData]
  return
}

##
## Destroy module specific things before unload
##
## Args:    none
## Returns: nothing
##
proc ModuleDestroy {} {
  variable configData

  trace vdelete configData w [namespace code TraceConfigData]
  return
}

# Configuration defaults, values and descriptions
array set configDefaults {
  bold {switch 1 {} {0/1} {display dice results in bold text}}
  dice {integer 2 {} number {how many dice to roll}}
  faces {integer 20 {} number {faces on each die}}
  low1 {group 1:1 {} {low:high} {limits for low1 roll}}
  low2 {group 2:2 {} {low:high} {limits for low2 roll}}
  high1 {group 15:19 {} {low:high} {limits for high1 roll}}
  high2 {group 20:20 {} {low:high} {limits for high2 roll}}
  lowodds1 {group 10:30 {} {x:range} {chance for reroll for x out of range}}
  lowodds2 {group 5:20 {} {x:range} {chance for reroll for x out of range}}
  highodds1 {group 0:0 {} {x:range} {chance for reroll for x out of range}}
  highodds2 {group 1:5 {} {x:range} {chance for reroll for x out of range}}
}

# Bind defaults
array set bindDefaults {
  pub:roll {{pub -|- roll {cmdchr {cmdsub {{commandChar pub} ^}}} {}} {pub -|- ,, {} {}}}
}

# Help Data
array set helpData {
  pub:roll {{} {rolls the dice}}
}
