###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: foodcmds.tcl,v 1.16 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wrpg-foodcmds
# version: 1.0
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description:
# requires: foodData commandChar isNumber grammarA splitStringList


# FIXME: finish cleanup, test and debug

##
## Add the given animal with optional amount to the food list: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:addfood {nick uhost hand chan arg} {
  return [addfood pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Add the given animal with optional amount to the food list: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:addfood {nick uhost hand arg} {
  return [addfood msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Add the given animal with optional amount to the food list: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:addfood {hand idx arg} {
  return [addfood dcc putdcc $idx "" "" $hand "" $arg]
}

##
## Add the given animal with optional amount to the food list
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc addfood {type action dest nick uhost hand chan arg} {
  if {[authCheck $type addfood $nick $uhost $hand $chan]} then {
    set arg [split [string trimright $arg]]
    if {[isNumber [lindex $arg end]]} then {
      set name [join [lrange $arg 0 [expr [llength $arg] - 2]]]
      set amount [lindex $arg end]
    } else {
      set name [join $arg]
      set amount ""
    }
    if {![string compare "" $name]} then {
      put $action $dest "Usage: [commandChar $type]addfood <name> \[amount\]"
    } else {
      foreach {status realname} [foodData add $name $amount] {break}
      switch -exact -- $status {
        2 -
        1 {
          put $action $dest "Added $realname \[$amount\] to the food list."
        }
        0 {
          put $action $dest "I don't know what [grammarA $name] is."
        }
        -1 {
          put $action $dest "Invalid amount of $realname to add."
        }
      }
    }
    return 1
  }
  return 0
}

##
## Remove the given animal or amount from the food list: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:delfood {nick uhost hand chan arg} {
  return [delfood pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Remove the given animal or amount from the food list: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:delfood {nick uhost hand arg} {
  return [delfood msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Remove the given animal or amount from the food list: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:delfood {hand idx arg} {
  return [delfood dcc putdcc $idx "" "" $hand "" $arg]
}

##
## Remove the given animal or amount from the food list
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc delfood {type action dest nick uhost hand chan arg} {
  if {[authCheck $type delfood $nick $uhost $hand $chan]} then {
    set arg [split [string trimright $arg]]
    if {[isNumber [lindex $arg end]]} then {
      set name [join [lrange $arg 0 [expr [llength $arg] - 2]]]
      set amount [lindex $arg end]
    } else {
      set name [join $arg]
    }
    if {![string compare "" $name]} then {
      put $action $dest "Usage: [commandChar $type]delfood <name> \[amount\]"
    } else {
      foreach {status realname} [foodData del $name $amount] {break}
      switch -exact -- $status {
        2 {
          put $action $dest "Removed $realname from the food list."
        }
        1 {
          put $action $dest "Removed $realname \[$amount\] from the food list."
        }
        0 {
          put $action $dest "I don't know what [grammarA $name] is."
        }
        -1 {
          put $action $dest "Invalid amount of $realname to remove."
        }
        -2 {
          put $action $dest "There is no $realname in the food list."
        }
      }
    }
    return 1
  }
  return 0
}

##
## Change the amount for a given animal in the food list: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:chfood {nick uhost hand chan arg} {
  return [chfood pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Change the amount for a given animal in the food list: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:chfood {nick uhost hand arg} {
  return [chfood msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Change the amount for a given animal in the food list: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:chfood {hand idx arg} {
  return [chfood dcc putdcc $idx "" "" $hand "" $arg]
}

##
## Change the amount for a given animal in the food list
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc chfood {type action dest nick uhost hand chan arg} {
  if {[authCheck $type chfood $nick $uhost $hand $chan]} then {
    set arg [split [string trimright $arg]]
    set name [join [lrange $arg 0 [expr [llength $arg] - 2]]]
    set amount [lindex $arg end]
    if {(![string compare "" $name]) || (![string compare "" $amount]) || \
        (![isNumber $amount])} then {
      put $action $dest "Usage: [commandChar $type]chfood <name> <amount>"
    } else {
      foreach {status realname} [foodData set $name $amount] {break}
      switch -exact -- $status {
        1 {
          put $action $dest "Set $realname amount to \[$amount\] in the food list."
        }
        0 {
          put $action $dest "I don't know what [grammarA $name] is."
        }
        -1 {
          put $action $dest "Invalid amount to set $realname to."
        }
        -2 {
          put $action $dest "There is no $realname in the food list."
        }
      }
    }
    return 1
  }
  return 0
}

##
## Display the food list: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listfood {nick uhost hand chan arg} {
  return [listfood pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Display the food list: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listfood {nick uhost hand arg} {
  return [listfood msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
}

##
## Display the food list: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listfood {hand idx arg} {
  return [listfood dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
}

##
## Display the food list
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc listfood {type action dest nick uhost hand chan arg} {
  if {[authCheck $type listfood $nick $uhost $hand $chan]} then {
    set what [split [string trimright $arg]]
    foreach {status data} [foodData list $what] {break}
    switch -exact -- $status {
      3 {
        put $action $dest "Info for $name: [lrange $data 1 end]"
      }
      2 {
        putList $action $dest [splitStringList [lrange $data 2 end] 160 "," ", "] "The following animals are on the food list have $what days left ([lindex $data 1]): " "(cont): "
      }
      1 {
        putList $action $dest [splitStringList [lrange $data 2 end] 160 "," ", "] "The following animals are on the food list ([lindex $data 1]): " "(cont): "
      }
      0 {
        put $action $dest "I don't know what [grammarA $name] is."
      }
      -1 {
        put $action $dest "The food list is empty :("
      }
      -2 {
        put $action $dest "There is no food that has $what days left in the food list."
      }
      -3 {
        put $action $dest "Food will not spoil."
      }
      -4 {
        put $action $dest "There is no $data in the food list."
      }
    }
    return 1
  }
  return 0
}


#############################################################################
#
# FIXME: code from hunt.tcl:

# eat prey to increase user's fp
proc pub:eat {nick uhost hand chan arg} {
  variable conf
  variable fooddata
  variable fooddata_changed

  if {[validHandle $hand]} then {
    set arg [split [string trimright $arg]]
    if {![string length $arg]} then {
      if {$conf(foodspoil)} then {
        foreach i [lsort [array names fooddata]] {
          lappend list "[join $i] \[[lindex $fooddata($i) 0]:[days_left [lindex $fooddata($i) 1] $conf(foodspoil)]\],"
        }
      } else {
        foreach i [lsort [array names fooddata]] {
          lappend list "[join $i] \[[lindex $fooddata($i) 0]\],"
        }
      }
      if {[info exists list]} then {
        formatLists putmsg $chan [string trimright [join $list] ,] 160 , , " " "The food available is: " "(cont): "
      } else {
        putmsg $chan "The food list is empty :("
      }
    } else {
      if {[isNumber [lindex $arg end]]} then {
        set animal [arrayFindElementName huntdata [lrange $arg 0 [expr [llength $arg] - 2]]]
        set amount [lindex $arg end]
      } else {
        set animal [arrayFindElementName huntdata $arg]
        set amount 1
      }
      #FIXME: $animal == ""
      if {[info exists fooddata($animal)]} then {
        set count [lindex $fooddata($animal) 0]
        if {$count > 0} then {
          if {([days_left [lindex $fooddata($animal) 1] $conf(foodspoil)] > 0) || (!$conf(foodspoil))} then {
            if {![matchattr $hand F]} then {
              if {[fp test2 $hand 0] != 2} then {
                if {[expr ([fp limit $hand 0] - [fp list $hand 0]) >= $amount]} then {
                  if {[isNumber $amount]} then {
                    if {[lindex $fooddata($animal) 0] >= $amount} then {
                      #FIXME: need to do this all at once
                      for {set count2 $amount} {$count2 > 0} {incr count2 -1} {
                        if {$count > 1} then {
                          if {[fp incr $hand 1]} then {
                            set fooddata($animal) [lreplace $fooddata($animal) 0 0 [expr [lindex $fooddata($animal) 0] - 1]]
                            set fooddata_changed 1
                            incr count -1
                          }
                        } else {
                          if {[fp incr $hand 1]} then {
                            set fooddata($animal) [lreplace $fooddata($animal) 0 0 0]
                            set fooddata_changed 1
                            incr count -1
                          }
                        }
                      }
                      if {$count < 1} then {
                        if {$conf(foodbury)} then {
                          unset fooddata($animal)
                        }
                        putmsg $chan "$nick eats the last of the [join $animal]."
                      } elseif {$count == 1} then {
                        putmsg $chan "$nick eats some of the [join $animal], there is only enough left for one more wolf."
                      } elseif {$count > 1} then {
                        putmsg $chan "$nick eats some of the [join $animal], there is enough left for [lindex $fooddata($animal) 0] more wolves."
                      }
                    } else {
                      putnot $nick "There is not that much [join $animal] to eat."
                    }
                  } else {
                    putnot $nick "Invalid count of [join $animal] to eat."
                  }
                } else {
                  putmsg $chan "Sorry $nick, but you are too full to eat that much [join $animal]."
                }
              } else {
                putmsg $chan "Sorry $nick, but you are too full to eat."
              }
            } else {
              putnot $nick "You don't need to eat anything."
            }
          } else {
            putmsg $chan "The [join $animal] has spoiled, please bury it."
          }
        } else {
          putmsg $chan "There is nothing but a carcass left of the [join $animal], please bury it."
        }
      } else {
        putmsg $chan "There is no [join $animal] to eat."
      }
    }
    return 1
  }
  return 0
}


# bury prey that has been eaten or has spoiled
proc pub:bury {nick uhost hand chan arg} {
  variable conf
  variable fooddata
  variable fooddata_changed

  if {[validHandle $hand]} then {
    set arg [split [string trimright $arg]]
    if {[string length $arg]} then {
      set animal [arrayFindElementName huntdata $arg]
      if {[string length $animal]} then {
        if {[info exists fooddata($animal)]} then {
          if {([lindex $fooddata($animal) 0] <= 0) || \
              (!$conf(foodspoil)) || \
              ([days_left [lindex $fooddata($animal) 1] $conf(foodspoil)] <= 0)} then {
            putmsg $chan "$nick buries the [join $animal]"
            unset fooddata($animal)
            set fooddata_changed 1
          } else {
            putnot $nick "There is still [join $animal] left."
          }
        } else {
          putnot $nick "Theres no [join $animal] to bury."
        }
      } else {
        putnot $nick "I don't know what [grammar_check2 0 $arg] is."
      }
    } else {
      putnot $nick "You must specify the animal to bury."
    }
    return 1
  }
  return 0
}


##
## Remove spoiled food once an hour
##
## Args:    minute, hour, day, month, year
## Returns: 0
##
proc time:hourly {min hour day month year} {
  variable configData
  variable fooddata
  variable fooddataChanged

  if {$configData(foodbury)} then {
    foreach {name data} [array get fooddata] {
      foreach {amount timestamp} $data {break}
      if {($amount <= 0) || \
          ([daysLeft $timestamp $configData(foodspoil)] <= 0)} then {
        unset fooddata($name)
        set fooddataChanged 1
      }
    }
  }
  return 0
}

#
#############################################################################


# Bind defaults
array set bindDefaults {
  pub:addfood {{pub m|m addfood {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:delfood {{pub m|m delfood {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:chfood {{pub m|m chfood {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listfood {{pub m|m listfood {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:addfood {{msg m|m addfood {} {}}}
  msg:delfood {{msg m|m delfood {} {}}}
  msg:chfood {{msg m|m chfood {} {}}}
  msg:listfood {{msg m|m listfood {} {}}}
  dcc:addfood {{dcc m|m addfood {} {}}}
  dcc:delfood {{dcc m|m delfood {} {}}}
  dcc:chfood {{dcc m|m chfood {} {}}}
  dcc:listfood {{dcc m|m listfood {} {}}}
  pub:eat {{pub P|P eat {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:bury {{pub P|P bury {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  time:hourly {{time -|- {00 * * * *} {} {}}}
}

# Help Data
array set helpData {
  pub:addfood {{<animal> [amount]} {adds an animal to the food database}}
  pub:delfood {{<animal> [amount]} {removes an animal from the food database}}
  pub:chfood {{<animal> <amount>} {changes the amount of an animal in the food database}}
  pub:listfood {{[animal|days]} {lists the food database}}
  msg:addfood {{<animal> [amount]} {adds an animal to the food database}}
  msg:delfood {{<animal> [amount]} {removes an animal from the food database}}
  msg:chfood {{<animal> <amount>} {changes the amount of an animal in the food database}}
  msg:listfood {{[animal|days]} {lists the food database}}
  dcc:addfood {{<animal> [amount]} {adds an animal to the food database}}
  dcc:delfood {{<animal> [amount]} {removes an animal from the food database}}
  dcc:chfood {{<animal> <amount>} {changes the amount of an animal in the food database}}
  dcc:listfood {{[animal|days]} {lists the food database}}
  pub:eat {{[animal] [amount]} {rasies a user's fp and shows the food database}}
  pub:bury {{<animal>} {removes the given animal from the food database}}
}
