###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: huntdata.tcl,v 1.11 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wrpg-huntdata
# version: 1.0
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: wrpg-huntdata.conf
# description:
# requires: dataFormatValue dataFormatReplace arrayFindElementName
# provides: huntData


# FIXME: finish rewrite

##
## Handle hunt data
##
## Args:    action {add|del|set|get|list|exists|realname|format|value}, args
## Returns: data passed from sub procs
##
proc huntData {action args} {
  switch -exact -- $action {
    add {
      return [huntDataAdd $args]
    }
    del {
      return [huntDataDel $args]
    }
    set {
      return [huntDataSet $args]
    }
    get {
      return [huntDataGet $args]
    }
    list {
      return [huntDataList $args]
    }
    exists {
      return [huntDataExists $args]
    }
    realname {
      return [huntDataRealname $args]
    }
    format {
      return [huntDataFormat $args]
    }
    value {
      return [huntDataValue $args]
    }
  }
  return
}

##
## Add hunt data to the hunt list
##
## Args:    args {name, options}
## Returns: 1 if name added to the hunt list
##          0 if name is already in the hunt list
##          -1 if args are invalid
##
proc huntDataAdd {args} {
#  variable dataFormats
#  variable huntdata
#
#  set name [lindex $args 0]
#  set options [lidnex $args 1]
#  foreach {group hits food hunt sound diceopt} $options {break}
#
#  return 0
}

##
## Remove hunt data from the hunt list
##
## Args:    args {name}
## Returns: 
##
proc huntDataDel {args} {
#  variable dataFormats
#  variable huntdata
#
#  return
}

##
## Set hunt data in the hunt list
##
## Args:    args {name, options}
## Returns:
##
proc huntDataSet {args} {
#  variable huntdata
#
#  return
}

##
## Get food data amount from the food list
##
## Args:    args {name}
## Returns: hunt data for 'name'
##
proc huntDataGet {args} {
  variable huntdata

  set name [lindex $args 0]
  if {[info exists huntdata($name)]} then {
    return $huntdata($name)
  }
  return
}

##
## List hunt data from the hunt list
##
## Args:    args {what}
## Returns: 
##
proc huntDataList {args} {
#  variable configData
#  variable dataFormats
#  variable huntdata
#
#  set what [lindex $args 0]
#
#  return
}

##
## Check if hunt data exists in the hunt list
##
## Args:    args {name}
## Returns: 1 if name is in the hunt list
##          0 if name is not in the hunt list
##
proc huntDataExists {args} {
  variable huntdata

  if {[info exists huntdata([lindex $args 0])]} then {
    return 1
  }
  return 0
}

##
## Find realname for hunt data in the hunt list
##
## Args:    args {name}
## Returns: realname if name is in the hunt list
##          "" if name is not in the hunt list
##
proc huntDataRealname {args} {
  variable huntdata

  set realname [arrayFindElementName huntdata [lindex $args 0]]]
  if {[string compare "" $realname]} then {
    return $realname
  }
  return
}

##
## Return huntdata data format
##
## Args:    args
## Returns: huntdata data format
##
proc huntDataFormat {args} {
  variable dataFormats

  return $dataFormats(huntdata)
}

##
## Return data for a particular huntdata field
##
## Args:    args
## Returns: huntdata data format
##
proc huntDataValue {args} {
  variable huntdata
  variable dataFormats

  set realname [huntData realname [lindex $args 0]]
  set what [lindex $args 1]
  if {([string compare "" $realname]) && \
      ([info exists huntdata($realname)])} then {
    return [dataFormatValue $dataFormats(huntdata) $huntdata($realname) $what]
  }
  return
}

# Set data format information
array set dataFormats {
  huntdata "{group 1} {hits 1} {food 1} {hunt 1} {sound 0} {dice {}}"
}

# Set data file information
array set dataOptions {
  huntdata "array wrpg-hunt.data hunt"
}
