###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: pointcmds.tcl,v 1.15 2005/01/23 04:06:52 tothwolf Exp $
##

# name: wrpg-pointcmds
# version: 1.0
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Wolf RPG user hit point and food point dcc, msg and public \
#	commands.
# requires: authCheck commandChar isNumber put putnot checkUser findUser \
#	userPointData userStatusData


##
## Increases a user's hp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:addhp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          addhp incr hp]
}

##
## Increases a user's hp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:addhp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          addhp incr hp]
}

##
## Increases a user's hp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:addhp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg addhp incr hp]
}

##
## Decreases a user's hp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:delhp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          delhp decr hp]
}

##
## Decreases a user's hp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:delhp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          delhp decr hp]
}

##
## Decreases a user's hp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:delhp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg delhp decr hp]
}

##
## Change a user's hp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:chhp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          chhp set hp]
}

##
## Change a user's hp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:chhp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          chhp set hp]
}

##
## Change a user's hp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:chhp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg chhp set hp]
}

##
## List a user's hp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listhp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          listhp list hp]
}

##
## List a user's hp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listhp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          listhp list hp]
}

##
## List a user's hp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listhp {hand idx arg} {
  return [UserPointData dcc putdcc $idx putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg listhp list hp]
}

##
## Increases a user's fp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:addfp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          addfp incr fp]
}

##
## Increases a user's fp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:addfp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          addfp incr fp]
}

##
## Increases a user's fp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:addfp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg addfp incr fp]
}

##
## Decreases a user's fp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:delfp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          delfp decr fp]
}

##
## Decreases a user's fp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:delfp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          delfp decr fp]
}

##
## Decreases a user's fp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:delfp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg delfp decr fp]
}

##
## Change a user's fp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:chfp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          chfp set fp]
}

##
## Change a user's fp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:chfp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          chfp set fp]
}

##
## Change a user's fp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:chfp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg chfp set fp]
}

##
## List a user's fp: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listfp {nick uhost hand chan arg} {
  return [UserPointData pub putnot $nick $nick $uhost $hand $chan $arg \
          listfp list fp]
}

##
## List a user's fp: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listfp {nick uhost hand arg} {
  return [UserPointData msg putnot $nick $nick $uhost $hand "" $arg \
          listfp list fp]
}

##
## List a user's fp: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listfp {hand idx arg} {
  return [UserPointData dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg listfp list fp]
}

##
## Changes/lists a user's hp/fp data
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg,
##          args {command, action, what}
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc UserPointData {type action dest nick uhost hand chan arg args} {
  set command [lindex $args 0]
  if {[authCheck $type $command $nick $uhost $hand $chan]} then {
    set dataAction [lindex $args 1]
    set what [lindex $args 2]
    set arg [split [string trimright $arg]]
    set who [lindex $arg 0]
    set amount [lindex $arg 1]
    if {![string length $amount]} then {
      set amount 1
    }
    if {(![string length $who]) || (![isNumber $amount])} then {
# FIXME: do proper arg checking
      set extra ""
      switch -exact -- $dataAction {
        incr -
        decr {
          set extra " \[amount\]"
        }
        set {
          set extra " <amount>"
        }
        list {
          set extra ""
        }
      }
      put $action $dest "Usage: [commandChar $type]$command <nick|hand>$extra"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have [string toupper $what]."
          }
        }
      } elseif {![userStatusData $user compare status pack]} then {
        put $action $dest "$user is not pack."
      } else {
        if {[set result [userPointData $user $dataAction $what $amount]] != -1} then {
          put $action $dest "[string toupper $what] for $user are: $result"
        } else {
          put $action $dest "Invalid [string toupper $what] amount: $amount"
        }
      }
    }
    return 1
  }
  return 0
}

# Bind defaults
array set bindDefaults {
  pub:addhp {{pub m|m addhp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:delhp {{pub m|m delhp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:chhp {{pub m|m chhp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listhp {{pub m|m listhp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:addfp {{pub m|m addfp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:delfp {{pub m|m delfp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:chfp {{pub m|m chfp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listfp {{pub m|m listfp {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:addhp {{msg m|m addhp {} {}}}
  msg:delhp {{msg m|m delhp {} {}}}
  msg:chhp {{msg m|m chhp {} {}}}
  msg:listhp {{msg m|m listhp {} {}}}
  msg:addfp {{msg m|m addfp {} {}}}
  msg:delfp {{msg m|m delfp {} {}}}
  msg:chfp {{msg m|m chfp {} {}}}
  msg:listfp {{msg m|m listfp {} {}}}
  dcc:addhp {{dcc m|m addhp {} {}}}
  dcc:delhp {{dcc m|m delhp {} {}}}
  dcc:chhp {{dcc m|m chhp {} {}}}
  dcc:listhp {{dcc m|m listhp {} {}}}
  dcc:addfp {{dcc m|m addfp {} {}}}
  dcc:delfp {{dcc m|m delfp {} {}}}
  dcc:chfp {{dcc m|m chfp {} {}}}
  dcc:listfp {{dcc m|m listfp {} {}}}
}

# Help Data
array set helpData {
  pub:addhp {{<nick|hand> <amount>} {adds hp for a user}}
  pub:delhp {{<nick|hand> <amount>} {removes hp from a user}}
  pub:chhp {{<nick|hand> <amount>} {changes a user's hp count}}
  pub:listhp {{<nick|hand>} {lists a user's hp count}}
  pub:addfp {{<nick|hand> <amount>} {adds fp for a user}}
  pub:delfp {{<nick|hand> <amount>} {removes fp from a user}}
  pub:chfp {{<nick|hand> <amount>} {changes a user's fp count}}
  pub:listfp {{<nick|hand>} {lista a user's fp count}}
  msg:addhp {{<hand> <amount>} {adds hp for a user}}
  msg:delhp {{<hand> <amount>} {removes hp from a user}}
  msg:chhp {{<hand> <amount>} {changes a user's hp count}}
  msg:listhp {{<hand>} {lists a user's hp count}}
  msg:addfp {{<hand> <amount>} {adds fp for a user}}
  msg:delfp {{<hand> <amount>} {removes fp from a user}}
  msg:chfp {{<hand> <amount>} {changes a user's fp count}}
  msg:listfp {{<hand>} {lista a user's fp count}}
  dcc:addhp {{<nick|hand> <amount>} {adds hp for a user}}
  dcc:delhp {{<nick|hand> <amount>} {removes hp from a user}}
  dcc:chhp {{<nick|hand> <amount>} {changes a user's hp count}}
  dcc:listhp {{<nick|hand>} {lists a user's hp count}}
  dcc:addfp {{<nick|hand> <amount>} {adds fp for a user}}
  dcc:delfp {{<nick|hand> <amount>} {removes fp from a user}}
  dcc:chfp {{<nick|hand> <amount>} {changes a user's fp count}}
  dcc:listfp {{<nick|hand>} {lista a user's fp count}}
}
