###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: pointstat.tcl,v 1.10 2005/01/23 03:50:48 tothwolf Exp $
##

# name: wrpg-pointstat
# version: 1.0
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Wolf RPG user hit point and food point status support.
# requires: putnot validHandle userPointData userStatusData


##
## Annoy users when they are hungry: join
##
## Args:    nick, user@host, handle, channel
## Returns: 0
##
proc join:checkFp {nick uhost hand chan} {
  if {([validHandle $hand]) && \
      ([userStatusData $hand compare status pack]) && \
      (![userPointData $hand test1 fp])} then {
    if {[string length [set response [checkFp $hand]]]} then {
      putnot $nick $response
    }
  }
  return 0
}

##
## Annoy users when they are hungry: chon
##
## Args:    handle, idx
## Returns: 0
##
proc chon:checkFp {hand idx} {
  if {([userStatusData $hand compare status pack]) && \
      (![userPointData $hand test1 fp])} then {
    if {[string length [set response [checkFp $hand]]]} then {
      putdcc $idx " "
      putdcc $idx ">> $response"
    }
  }
  return 0
}

##
## Annoy users when they are hungry
##
## Args:    handle
## Returns: description of a user's fp state
##
proc checkFp {hand} {
  variable fpstatus

  set fp [userPointData $hand list fp]
  set limit [userPointData $hand limit fp]
  foreach value [lsort -real [array names fpstatus]] {
    if {[expr $limit * $value] >= $fp} then {
      return $fpstatus($value)
    }
  }
  return
}

##
## Init module specific things at load time
##
## Args:    none
## Returns: nothing
##
proc ModuleInit {} {
  variable fpstatus

  # set fp status defaults if none found
  if {(![info exists fpstatus]) || \
      (![llength [array names fpstatus]])} then {
    array set fpstatus {
      0 "You are STARVING, you had better eat soon!"
      0.1 "You are very hungry, you had best eat somthing."
      0.2 "You are pretty hungry, the DCP smells good."
      0.3 "You are hungry, perhaps you should take a look at the food."
      0.4 "You are hungry, a nice piece of deer meat would taste good about now."
      0.5 "You are abit hungry, time for a snack."
    }
  }
  return
}

# FIXME: ugly hack?
# Set datafile information
array set dataOptions {
  fpstatus "array wrpg-pointstat-fp.data {wrpg food point status}"
}

# Bind defaults
array set bindDefaults {
  join:checkFp {{join -|- * {} {}}}
  chon:checkFp {{chon -|- * {} {}}}
}
