###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: statuscmds.tcl,v 1.20 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wrpg-statuscmds
# version: 1.0
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# description: Wolf RPG user status dcc, msg and public commands
# requires: authCheck commandChar findUser checkUser putList splitStringList put \
#	validHandle userPointData userStatusData userAssessmentData \
#	floodCheck


##
## Add pack status for a user: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:addpack {nick uhost hand chan arg} {
  return [addpack pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Add pack status for a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:addpack {nick uhost hand arg} {
  return [addpack msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Add pack status for a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:addpack {hand idx arg} {
  return [addpack dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Add pack status for a user
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc addpack {type action dest nick uhost hand chan arg} {
  if {[authCheck $type addpack $nick $uhost $hand $chan]} then {
    set arg [split [string trimright $arg]]
    set who [lindex $arg 0]
    set pos [join [lrange $arg 1 end]]
    if {![string length $who]} then {
      put $action $dest "Usage: [commandChar $type]addpack <nick|hand> \[position\]"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots can't be pack."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "$user is already pack."
          }
          assessment {
            if {[userStatusData $user set status pack]} then {
              userAssessmentData $user del
              if {[string length $pos]} then {
                userStatusData $user set pos $pos
              } else {
                userStatusData $user check pos
              }
              userStatusData $user check level
              userPointData $user check hp
              userPointData $user check fp
              put $action $dest "Updated status for $user from assessment to pack."
            } else {
              put $action $dest "Error updating status for $user from assessment to pack."
            }
          }
          default {
            if {[userStatusData $user set status pack]} then {
              if {[string length $pos]} then {
                userStatusData $user set pos $pos
              } else {
                userStatusData $user check pos
              }
              userStatusData $user check level
              userPointData $user check hp
              userPointData $user check fp
              put $action $dest "Added pack status for $user."
            } else {
              put $action $dest "Error adding pack status for $user."
            }
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## Remove pack status from a user: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:delpack {nick uhost hand chan arg} {
  return [delpack pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Remove pack status from a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:delpack {nick uhost hand arg} {
  return [delpack msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Remove pack status from a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:delpack {hand idx arg} {
  return [delpack dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Remove pack status from a user
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc delpack {type action dest nick uhost hand chan arg} {
  if {[authCheck $type delpack $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {![string length $who]} then {
      put $action $dest "Usage: [commandChar $type]delpack <nick|hand>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots can't be pack."
          }
        }
      } elseif {![userStatusData $user compare status pack]} then {
        put $action $dest "$user is not pack."
      } elseif {[userStatusData $user clear status]} then {
        userStatusData $user clear pos
        userStatusData $user clear level
        userPointData $user clear hp
        userPointData $user clear fp
        put $action $dest "Removed pack status from $user."
      } else {
        put $action $dest "Error removing pack status for $user."
      }
    }
    return 1
  }
  return 0
}

##
## Display pack info for a user: pub
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listpack {nick uhost hand chan arg} {
  return [listpack pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Display pack info for a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listpack {nick uhost hand arg} {
  return [listpack msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Display pack info for a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listpack {hand idx arg} {
  return [listpack dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Display pack info for a user
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc listpack {type action dest nick uhost hand chan arg} {
  if {[authCheck $type listpack $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {[string length $who]} then {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have pack info."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "Info for $user: HP \[[userPointData $user list hp]\], FP \[[userPointData $user list fp]\], Level \[[userStatusData $user list level]\], Position \[[userStatusData $user list pos]\]"
          }
          assessment {
            put $action $dest "$user is under assessment."
          }
          default {
            put $action $dest "$user is not pack."
          }
        }
      }
    } else {
      if {[string compare "" [set users [userStatusData "" listusers status pack]]]} then {
        set count 0
        foreach user $users {
          append list "$user: HP \[[userPointData $user list hp]\], FP \[[userPointData $user list fp]\], Level \[[userStatusData $user list level]\], Position \[[userStatusData $user list pos]\], "
          incr count
        }
        putList $action $dest [splitStringList $list 160 , ", "] "I know the following pack members ($count): " "(cont): "
      } else {
        put $action $dest "No users are pack."
      }
    }
    return 1
  }
  return 0
}

##
## Add assessment status for a user: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:addas {nick uhost hand chan arg} {
  return [addas pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Add assessment status for a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:addas {nick uhost hand arg} {
  return [addas msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Add assessment status for a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:addas {hand idx arg} {
  return [addas dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Add assessment status for a user
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc addas {type action dest nick uhost hand chan arg} {
  if {[authCheck $type addas $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {![string length $who]} then {
      put $action $dest "Usage: [commandChar $type]addas <nick|hand>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots can't be under assessment."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "$user is already pack."
          }
          assessment {
            put $action $dest "$user is already under assessment."
          }
          default {
            if {[userStatusData $user set status assessment]} then {
              userAssessmentData $user add
              put $action $dest "Added assessment status for $user."
            } else {
              put $action $dest "Error adding assessment status for $user."
            }
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## Remove assessment status from a user: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:delas {nick uhost hand chan arg} {
  return [delas pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Remove assessment status from a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:delas {nick uhost hand arg} {
  return [delas msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Remove assessment status from a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:delas {hand idx arg} {
  return [delas dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Remove assessment status from a user
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc delas {type action dest nick uhost hand chan arg} {
  if {[authCheck $type delas $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {![string length $who]} then {
      put $action $dest "Usage: [commandChar $type]delas <nick|hand>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots can't be under assessment."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "$user is already pack."
          }
          assessment {
            if {[userStatusData $user clear status]} then {
              userAssessmentData $user del
              put $action $dest "Removed assessment status for $user."
            } else {
              put $action $dest "Error removing assessment status for $user."
            }
          }
          default {
            put $action $dest "$user is not under assessment."
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## Display assessment info for a user: pub
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listas {nick uhost hand chan arg} {
  return [listas pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Display assessment info for a user: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listas {nick uhost hand arg} {
  return [listas msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Display assessment info for a user: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listas {hand idx arg} {
  return [listas dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Display assessment info for a user
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc listas {type action dest nick uhost hand chan arg} {
  if {[authCheck $type listpack $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {[string length $who]} then {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have assessment info."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "$user is pack."
          }
          assessment {
            switch -exact -- [set timeleft [userAssessmentData $user timeleft]] {
              -1 {
                put $action $dest "Assessment started for $user: [userAssessmentData $user started]"
              }
              0 {
                put $action $dest "Assessment over for $user."
              }
              default {
                put $action $dest "Assessment time left for $user: [duration $timeleft]"
              }
            }
          }
          default {
            put $action $dest "$user is not under assessment."
          }
        }
      }
    } else {
      if {[string compare "" [set users [userStatusData "" listusers status assessment]]]} then {
        set count 0
        foreach user $users {
          switch -exact -- [set timeleft [userAssessmentData $user timeleft]] {
            -1 {
              append list "$user: started [clock format [userAssessmentData $user time] -format "%a %b %d %Y"], "
            }
            0 {
              append list "$user: assessment over, "
            }
            default {
              append list "$user: [duration $timeleft] left, "
            }
          }
          incr count
        }
        putList $action $dest [splitStringList $list 160 , ", "] "Users under assessment ($count): " "(cont): "
      } else {
        put $action $dest "No users are under assessment."
      }
    }
    return 1
  }
  return 0
}

##
## Change a user's position: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:chpos {nick uhost hand chan arg} {
  return [chpos pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Change a user's position: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:chpos {nick uhost hand arg} {
  return [chpos msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Change a user's position: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:chpos {hand idx arg} {
  return [chpos dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Change a user's position
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc chpos {type action dest nick uhost hand chan arg} {
  if {[authCheck $type chpos $nick $uhost $hand $chan]} then {
    set arg [split [string trimright $arg]]
    set who [lindex $arg 0]
    set pos [join [lrange $arg 1 end]]
    if {(![string length $who]) || (![string length $pos])} then {
      put $action $dest "Usage: [commandChar $type]chpos <nick|hand> <position>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have a pack position."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            if {[string compare none [string tolower $pos]]} then {
              if {[userStatusData $user set pos $pos]} then {
                put $action $dest "Set position for $user to: $pos"
              } else {
                put $action $dest "Error setting position for $user"
              }
            } elseif {[userStatusData $user clear pos]} then {
              put $action $dest "Cleared position for $user."
            } else {
              put $action $dest "Error clearing position for $user"
            }
          }
          assessment {
            put $action $dest "$user is under assessment."
          }
          default {
            put $action $dest "$user is not pack."
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## List a user's position: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listpos {nick uhost hand chan arg} {
  return [listpos pub putnot $nick putnot $nick $nick $uhost $hand \
          $chan $arg]
}

##
## List a user's position: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listpos {nick uhost hand arg} {
  return [listpos msg putnot $nick putnot $nick $nick $uhost $hand \
          "" $arg]
}

##
## List a user's position: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listpos {hand idx arg} {
  return [listpos dcc putdcc $idx putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## List a user's position
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc listpos {type action dest nick uhost hand chan arg} {
  if {[authCheck $type listpos $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {![string length $who]} then {
      put $action $dest "Usage: [commandChar $type]listpos <nick|hand>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have a pack position."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "Position for $user: [userStatusData $user list pos]"
          }
          assessment {
            put $action $dest "$user is under assessment."
          }
          default {
            put $action $dest "$user is not pack."
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## Change a user's level: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:chlev {nick uhost hand chan arg} {
  return [chlev pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Change a user's level: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:chlev {nick uhost hand arg} {
  return [chlev msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Change a user's level: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:chlev {hand idx arg} {
  return [chlev dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Change a user's level
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc chlev {type action dest nick uhost hand chan arg} {
  if {[authCheck $type chlev $nick $uhost $hand $chan]} then {
    set arg [split [string trimright $arg]]
    set who [lindex $arg 0]
    set level [lindex $arg 1]
    if {(![string length $who]) || (![string length $level])} then {
      put $action $dest "Usage: [commandChar $type]chlev <nick|hand> <level>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have a pack level."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            if {[userStatusData $user set level $level]} then {
              put $action $dest "Set level for $user to: $level"
            } else {
              put $action $dest "Error setting level for $user"
            }
          }
          assessment {
            put $action $dest "$user is under assessment."
          }
          default {
            put $action $dest "$user is not pack."
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## List a user's level: pub
##
## Args:    nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc pub:listlev {nick uhost hand chan arg} {
  return [listlev pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## List a user's level: msg
##
## Args:    nick, user@host, handle, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc msg:listlev {nick uhost hand arg} {
  return [listlev msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## List a user's level: dcc
##
## Args:    handle, idx, arg
## Returns: 1
##
proc dcc:listlev {hand idx arg} {
  return [listlev dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## List a user's level
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1 if user is auth'd
##          0 otherwise
##
proc listlev {type action dest nick uhost hand chan arg} {
  if {[authCheck $type listlev $nick $uhost $hand $chan]} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {![string length $who]} then {
      put $action $dest "Usage: [commandChar $type]listlev <nick|hand>"
    } else {
      set user [findUser $who $chan]
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $action $dest "I don't know who $who is."
          }
          -1 {
            put $action $dest "Bots don't have a pack level."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            put $action $dest "Level for $user: [userStatusData $user list level]"
          }
          assessment {
            put $action $dest "$user is under assessment."
          }
          default {
            put $action $dest "$user is not pack."
          }
        }
      }
    }
    return 1
  }
  return 0
}

##
## Show a user's info: pub
##
## Args: nick, user@host, handle, channel, arg
## Returns: 1
##
proc pub:uinfo {nick uhost hand chan arg} {
  return [uinfo pub putnot $nick $nick $uhost $hand $chan $arg]
}

##
## Show a user's info: msg
##
## Args: nick, user@host, handle, arg
## Returns: 1
##
proc msg:uinfo {nick uhost hand arg} {
  return [uinfo msg putnot $nick $nick $uhost $hand "" $arg]
}

##
## Show a user's info: dcc
##
## Args: nick, user@host, handle, arg
## Returns: 1
##
proc dcc:uinfo {hand idx arg} {
  return [uinfo dcc putdcc $idx "" "" $hand \
          [lindex [console $idx] 0] $arg]
}

##
## Show a user's info
##
## Args:    type, action, dest, nick, user@host, handle, channel, arg
## Returns: 1
##
proc uinfo {type action dest nick uhost hand chan arg} {
  if {((![string compare dcc $type]) || \
       ([floodCheck $nick $uhost uinfo])) && \
      ([validHandle $hand])} then {
    set who [lindex [split [string trimright $arg]] 0]
    if {[string length $who]} then {
      if {[matchattr $hand m|m $chan]} then {
        set user [findUser $who $chan]
      } else {
        put $action $dest "Only masters can specify other users."
      }
    } else {
      set user $hand
    }
    if {[set result [checkUser $user]] != 1} then {
      switch -exact -- $result {
        0 {
          put $action $dest "I don't know who $who is."
        }
        -1 {
          put $action $dest "Bots don't have info."
        }
      }
    } else {
      switch -exact -- [userStatusData $user list status] {
        pack {
          set reply "HP \[[userPointData $user list hp]\], FP \[[userPointData $user list fp]\], Level \[[userStatusData $user list level]\], Position \[[userStatusData $user list pos]\]"
        }
        assessment {
          switch -exact -- [set timeleft [userAssessmentData $user timeleft]] {
            -1 {
              set reply "Assessment started: [userAssessmentData $user started]"
            }
            0 {
              set reply "Assessment over"
            }
            default {
              set reply "Assessment time left: [duration $timeleft]"
            }
          }
        }
        default {
          if {![string compare $hand $user]} then {
            if {[string length $nick]} then {
              set uwho $nick
            } else {
              set uwho $hand
            }
            put $action $dest "Sorry $uwho, but I don't have any info for you."
          } else {
            put $action $dest "I don't have any info for $user."
          }
        }
      }
      if {[info exists reply]} then {
        put $action $dest "Info for $user: $reply"
      }
    }
    return 1
  }
  return 0
}

# Bind defaults
array set bindDefaults {
  pub:addpack {{pub m|m addpack {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:delpack {{pub m|m delpack {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listpack {{pub m|m listpack {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:addas {{pub m|m addas {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:delas {{pub m|m delas {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listas {{pub m|m listas {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:chpos {{pub m|m chpos {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listpos {{pub m|m listpos {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:chlev {{pub m|m chlev {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:listlev {{pub m|m listlev {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:uinfo {{pub -|- uinfo {cmdchr {cmdsub {{commandChar pub} ^}}} {}} {pub -|- info {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:addpack {{msg m|m addpack {} {}}}
  msg:delpack {{msg m|m delpack {} {}}}
  msg:listpack {{msg m|m listpack {} {}}}
  msg:addas {{msg m|m addas {} {}}}
  msg:delas {{msg m|m delas {} {}}}
  msg:listas {{msg m|m listas {} {}}}
  msg:chpos {{msg m|m chpos {} {}}}
  msg:listpos {{msg m|m listpos {} {}}}
  msg:chlev {{msg m|m chlev {} {}}}
  msg:listlev {{msg m|m listlev {} {}}}
  msg:uinfo {{msg -|- uinfo {} {}}}
  dcc:addpack {{dcc m|m addpack {} {}}}
  dcc:delpack {{dcc m|m delpack {} {}}}
  dcc:listpack {{dcc m|m listpack {} {}}}
  dcc:addas {{dcc m|m addas {} {}}}
  dcc:delas {{dcc m|m delas {} {}}}
  dcc:listas {{dcc m|m listas {} {}}}
  dcc:chpos {{dcc m|m chpos {} {}}}
  dcc:listpos {{dcc m|m listpos {} {}}}
  dcc:chlev {{dcc m|m chlev {} {}}}
  dcc:listlev {{dcc m|m listlev {} {}}}
  dcc:uinfo {{dcc -|- uinfo {} {}}}
}

# Help Data
array set helpData {
  pub:addpack {{<nick|hand> [position]} {adds pack command access for a user}}
  pub:delpack {{<nick|hand>} {removes pack command access from a user}}
  pub:listpack {{[nick|hand]} {lists users who have pack command access}}
  pub:addas {{<nick|hand>} {adds a user for assessment}}
  pub:delas {{<nick|hand>} {removes a user from assessment}}
  pub:listas {{[nick|hand]} {lists users who are in assessment}}
  pub:chpos {{<nick|hand> <position>} {changes a user's position}}
  pub:listpos {{<nick|hand>} {lists a user's position}}
  pub:chlev {{<nick|hand> <level>} {changes a user's level}}
  pub:listlev {{<nick|hand>} {lists a user's level}}
  pub:uinfo {{[nick|hand]} {shows info for a user}}
  msg:addpack {{<hand> [position]} {adds pack command access for a user}}
  msg:delpack {{<hand>} {removes pack command access from a user}}
  msg:listpack {{[hand]} {lists users who have pack command access}}
  msg:addas {{<hand>} {adds a user for assessment}}
  msg:delas {{<hand>} {removes a user from assessment}}
  msg:listas {{[hand]} {lists users who are in assessment}}
  msg:chpos {{<hand> <position>} {changes a user's position}}
  msg:listpos {{<hand>} {lists a user's position}}
  msg:chlev {{<hand> <level>} {changes a user's level}}
  msg:listlev {{<hand>} {lists a user's level}}
  msg:uinfo {{[hand]} {shows info for a user}}
  dcc:addpack {{<nick|hand> [position]} {adds pack command access for a user}}
  dcc:delpack {{<nick|hand>} {removes pack command access from a user}}
  dcc:listpack {{[nick|hand]} {lists users who have pack command access}}
  dcc:addas {{<nick|hand>} {adds a user for assessment}}
  dcc:delas {{<nick|hand>} {removes a user from assessment}}
  dcc:listas {{[nick|hand]} {lists users who are in assessment}}
  dcc:chpos {{<nick|hand> <position>} {changes a user's position}}
  dcc:listpos {{<nick|hand>} {lists a user's position}}
  dcc:chlev {{<nick|hand> <level>} {changes a user's level}}
  dcc:listlev {{<nick|hand>} {lists a user's level}}
  dcc:uinfo {{[nick|hand]} {shows info for a user}}
}
