###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wrpgfserv.tcl,v 1.15 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wrpgfserv
# version: 0.6.3.3
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: wrpgfserv.conf
# description:
# requires: this_module_needs_to_be_converted commandChar


# list the avaliable sound files; pub
proc pub:sounds {nick uhost hand chan arg} {
  return [sounds pub putnot $nick $nick $uhost $hand $chan $arg]
}

# list the avaliable sound files; msg
proc msg:sounds {nick uhost hand arg} {
  return [sounds msg putnot $nick $nick $uhost $hand "" $arg]
}

# list the avaliable sound files
proc sounds {type action dest nick uhost hand chan arg} {
  global botnick files-path
  variable conf
  if {[validNick $nick]} then {
    set count 0
    foreach i [getfiles $conf(soundpath)] {
      lappend list "$i ([convert_size [file size [file join ${files-path} $conf(soundpath) $i]]]),"
      incr count
    }
    if {[info exists list]} then {
      formatLists $action $dest [string trimright [join $list] ,] 160 , , " " "Sound files avaliable ($count): " "(cont): "
      put $action $dest "To have any of the files sent to you, type: /msg $botnick send <filename>"
    } else {
      put $action $dest "No sound files are currently avaliable."
    }
    return 1
  }
  return 0
}

# send user a sound file when requested; pub
proc pub:send {nick uhost hand chan arg} {
  return [send pub putnot $nick $nick $uhost $hand $chan $arg]
}

# send user a sound file when requested; msg
proc msg:send {nick uhost hand arg} {
  return [send msg putnot $nick $nick $uhost $hand "" $arg]
}

# send user a sound file when requested
proc send {type action dest nick uhost hand chan arg} {
  global files-path
  variable conf

  if {[validNick $nick]} then {
    set arg [split [string trimright $arg]]
    set file [lindex [file split [lindex $arg 0]] end]
    set who [join [lindex $arg 1]]
    if {![string compare "" $file]} then {
      put $action $dest "[commandChar $type]send <file> \[nick\]"
    } else {
      if {![string compare "" $who]} then {
        set who $nick
      }
      if {([info exists files-path]) && ([info exists conf(soundpath)])} then {
        set filename [file join ${files-path} $conf(soundpath) $file]
        if {[file exists $filename]} then {
          wpLog x * "Sending `[file tail $filename]' to $who."
          dccsend $filename $who
        } elseif {![string compare $nick $who]} then {
          wpLog x * "Error: $nick!$uhost requested `$file', but it does not exist."
        } else {
          wpLog x * "Error: $nick!$uhost requested `$file' be sent to $who, but it does not exist."
        }
      }
    }
    return 1
  }
  return 0
}

# send out the rules file when requested; pub
proc pub:rules {nick uhost hand chan arg} {
  return [rules pub putnot $nick $nick $uhost $hand "" $arg]
}

# send out the rules file when requested; msg
proc msg:rules {nick uhost hand arg} {
  return [rules msg putnot $nick $nick $uhost $hand "" $arg]
}

# send out the rules file when requested
proc rules {type action dest nick uhost hand chan arg} {
  if {[validNick $nick]} then {
    global files-path
    variable conf

    set file [file join ${files-path} $conf(textdir) $conf(rulesfile)]
    if {[file exists $file]} then {
      wpLog x * "Sending [file tail $file] to $nick."
      dccsend $file $nick
    } else {
      wpLog x * "Error: Rules file [file tail $file] does not exist."
    }
    return 1
  }
  return 0
}

# Configuration defaults, values and descriptions
array set configDefaults {
  rulesfile {word rules.txt {} file {filename for rules in textdir}}
  soundpath {word wolfpack/sounds/ {} path {path for sound files (relative to files-path)}}
  textdir {word text/ {} path {path for text files (relative to files-path)}}
}

# FIXME: %nick% noauto below

# Bind defaults
array set bindDefaults {
  pub:sounds {{pub -|- sounds {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  pub:send {{pub -|- send {cmdchr {cmdsub {{commandChar pub} ^}}} {}} {pub -|- !%nick% noauto {}}}
  pub:rules {{pub -|- rules {cmdchr {cmdsub {{commandChar pub} ^}}} {}}}
  msg:sounds {{msg -|- sounds {} {}}}
  msg:send {{msg -|- send {} {}} {{msg -|- %nick noauto {}}}
  msg:rules {{msg -|- rules {} {}}}
}

# Help Data
array set helpData {
  pub:sounds {{} {shows sound files avaliable}}
  pub:send {{<soundfile>} {makes the bot send the given filename via dcc}}
  pub:rules {{} {dcc send the rule list to whoever used the command}}
  msg:sounds {{} {shows sound files avaliable}}
  msg:send {{<soundfile>} {makes the bot send the given filename via dcc}}
  msg:rules {{} {dcc send the rule list to whoever used the command}}
}
