###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wrpgsound.tcl,v 1.13 2005/01/04 03:36:44 tothwolf Exp $
##

# name: wrpgsound
# version: 0.6.5.3
# author: Tothwolf <tothwolf@concentric.net>
# license: GNU GPL v2
# config: wrpgsound.conf
# description:
# requires: this_module_needs_to_be_converted
# provides: valid_type random_sound substitute_sound


# set data information
set data(sounddata) "1 $conf(sounddatafile) sounddata"

# load data
module_data load "" 1 0 "1 $conf(sounddatafile) sounddata"

# refresh sound data; dcc
proc dcc:refsound {hand idx arg} {
  set type [lindex [split [string trimright $arg]] 0]
  if {![string compare "" $type]} then {
    wpLog o * "Refreshing sound data..."
    refsound ""
  } else {
    wpLog o * "Refreshing sound data for type `$type'..."
    refsound $type
  }
  return 1
}

# refresh sound data; fil
proc fil:refsound {hand idx arg} {
  set type [lindex [split [string trimright $arg]] 0]
  if {![string compare "" $type]} then {
    wpLog o * "Refreshing sound data..."
    refsound ""
  } else {
    wpLog o * "Refreshing sound data for type `$type'..."
    refsound $type
  }
  return 1
}

#FIXME: do this on a time binding
# refresh the sound data
proc refsound {type} {
  variable conf
  variable sounddata
  variable sounddata_changed
  if {![string compare "" $type]} then {
    set update $conf(types)
  } elseif {[valid_type $type]} then {
    set update $type
  }
  if {[info exists update]} then {
    foreach i $update {
      set list ""
      foreach j [lsort [getfiles $conf(soundpath)]] {
        set desc [string tolower [getdesc $conf(soundpath) $j]]
        if {[regexp -- "%$i.*" $desc]} then {
          lappend list $j
        }
      }
      if {[info exists list]} then {
        set sounddata($i) [string trimright [join $list]]
      } elseif {[info exists sounddata($i)]} then {
        unset sounddata($i)
      }
    }
    set sounddata_changed 1
    return 1
  }
  return 0
}

# returns 1 if the given sound type is valid
proc valid_type {type} {
  variable conf
  if {[lsearch -exact $conf(types) $type] != -1} then {
    return 1
  }
  return 0
}

# returns a random sound of the given type
proc random_sound {type} {
  variable sounddata
  if {[info exists sounddata($type)]} then {
    return [lindex $sounddata($type) [rand [llength $sounddata($type)]]]
  }
  return ""
}

# do sound substitutions for the given string
proc substitute_sound {string} {
  if {[regexp -- {.*%sound:.*} $string]} then {
    for {set index 0} {$index < [llength $string]} {incr index} {
      if {[regexp -- {%sound:.*} [lindex $string $index]]} then {
        set type [lindex [split [lindex $string $index] :] 1]
        set sound [random_sound $type]
        if {[string compare "" $sound]} then {
          set string [lreplace $string $index $index $sound]
        } else {
          set string [lreplace $string $index $index ${type}.wav]
        }
      }
    }
  }
  return $string
}

# Configuration defaults, values and descriptions
array set configDefaults {
  sounddatafile {word wrpgsound-cache.data {} file {file that cached sound data is stored in}}
  soundpath {word wolfpack/sounds/ {} path {path within files-path for sound files}}
  types {string {bark howl whine} {} types {sound types to search file descriptions for}}
}

# Bind defaults
array set bindDefaults {
  dcc:refsound {{dcc M|M refsound {} {}}}
  fil:refsound {{fil M|M refsound {} {}}}
}

# Help Data
array set helpData {
  dcc:refsound {{[type]} {refreshes sound data}}
  fil:refsound {{[type]} {refreshes sound data}}
}
