#! /bin/sh
# \
# Nice little hack to find latest version of tclsh in PATH \
# \
# NOTE: backslash and semicolon placements are important! \
# \
# Check for working 'grep -E' before using 'egrep' \
if echo a | (grep -E '(a|b)') >/dev/null 2>&1; \
then \
  egrep="grep -E"; \
else \
  egrep=egrep; \
fi; \
# Search for tclsh[0-9].[0-9] in each valid dir in PATH \
for dir in $(echo $PATH | sed 's/:/ /g'); \
do \
  if test -d $dir; \
  then \
    files=$(/bin/ls $dir | $egrep '^tclsh[0-9]\.[0-9]$'); \
    if test "$files" != ""; \
    then \
      versions="${versions:+$versions }$(echo $files | sed 's/tclsh//g')"; \
    fi; \
  fi; \
done; \
# Loop over each version to find the latest version of tclsh \
for ver in $versions; \
do \
  tmpver=$(echo $ver | sed 's/\.//g'); \
  if test "$lasttmpver" != ""; \
  then \
    if test "$tmpver" -gt "$lasttmpver"; \
    then \
      lastver=$ver; \
      lasttmpver=$tmpver; \
    fi; \
  else \
    lastver=$ver; \
    lasttmpver=$tmpver; \
  fi; \
done; \
# Use the latest tclsh version found, otherwise fall back to 'tclsh' \
exec tclsh$lastver "$0" "$@"
###############################################################################
##
##  Wolfpack - A modular Tcl system for Eggdrop 1.3.0+ with Tcl 8.0+
##  Copyright (C) 1998-2005  Tothwolf <tothwolf@concentric.net>
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
###############################################################################
##
##  You should not need to edit anything in any of these files.
##
##  './wolfpack.tcl -c' is used to configure initial settings.
##  './wolfpack.tcl --help' will list all avaliable options.
##
##  Use '.wpconf' from the bot's partyline to configure everything else.
##
###############################################################################
##
## $Id: wolfpack.tcl,v 1.71 2005/01/28 10:48:08 tothwolf Exp $
##

# Make sure Tcl version is compatible with this code (we use namespaces)
if {[catch {package require Tcl 8.0}]} then {
  if {[info exists argv0]} then {
    puts "Wolfpack: Error: wolfpack requires Tcl 8.0 or later to load."
  } else {
    putloglev o * "Wolfpack: Error: wolfpack requires Tcl 8.0 or later to load."
  }
  return
}

namespace eval :: {

# Eggdrop doesn't currently set argv0, so we use it to detect load type.
if {![info exists argv0]} then {
  # Fix for buggy Tcl variables in pre 1.3.28 Eggdrop versions
  catch {set numversion}

  # Require eggdrop 1.3.0 or later
  if {(![info exists numversion]) || ($numversion < 1030000)} then {
    putloglev o * "Wolfpack: Error: wolfpack requires Eggdrop 1.3.0 or later to load."
    return
  }
} else {
  # Emulate eggdrop's putloglev when loading with tclsh
  if {![string compare "" [info commands putloglev]]} then {
    proc putloglev {level channel text} {
      puts $text
    }
  }
  # Emulate eggdrop's putdcc when loading with tclsh
  if {![string compare "" [info commands putdcc]]} then {
    proc putdcc {idx text} {
      puts $text
    }
  }
}

##
## Log Tcl messages where they can be seen
##
## Args:    text
## Returns: nothing
##
proc tclLog {text} {
  # Tcl's tclLog has embedded newlines in it's output
  if {[string first \n $text] == -1} then {
    putloglev o * "Wolfpack: \[Tcl\]: $text"
  } else {
    foreach line [split $text \n] {
      putloglev o * "Wolfpack: \[Tcl\]: $line"
    }
  }
  return
}

} ;# namespace ::

namespace eval ::wp {

# Manage package version information
package forget ::wp
package provide ::wp 1.9.9

# set namespace variables
set NamespaceParent [namespace parent]
set NamespaceCurrent [namespace current]
set ModuleName [namespace tail $NamespaceCurrent]
set ModuleVersion [package require $NamespaceCurrent]

##
## WARNING: If you change these, you may render your module database useless!
##

# Max number of lines to scan in a module file
set moduleDatabaseConfig(scanlines) 30

# Max depth to list directories in module path
set moduleDatabaseConfig(maxdepth) 4

# Module database version
set moduleDatabaseConfig(version) 2.0

# Module database header
set moduleDatabaseConfig(header) "Wolfpack module database "

# Module database defaults
set moduleDatabaseConfig(defaults) "{version 0} {description {(no description)}} {load 0}"

# Versioned module database formats
array set moduleDatabaseFormat {
  2.0 "{version config author license description provides requires} {load file md5sum}"
}

# md5 style, command name and result string index
array set md5Format {
  bsd "md5 3"
  gnu "md5sum 0"
}

# Configuration defaults, values and descriptions
array set configDefaults {
  modulepath {word modules/ {} path {Module path (multiple paths may be separated by a ':')}}
  configpath {word wpconf/ {} path {Config path}}
  datapath {word wpdata/ {} path {Data path}}
  moddbfile {word wolfpack.db {} file {Module database file}}
  update {{range {0 1}} 1 {} {0/1} {Automatically update module database}}
  verbose {{range {0 1}} 0 {} {0/1} {Verbose operation}}
  time {{range {0 1}} 0 {} {0/1} {Time module database compare/update/rebuild}}
}

# Command line option defaults
array set optionData {
  cfgfile ""
  config 0
  update 0
  noupdate 0
  rebuild 0
  time 0
  include ""
  exclude ""
  module ""
  verbose 0
  quiet 0
  debug 0
}

array set moduleOptionData {
  exclude ""
  include ""
  module ""
  excludedfiles ""
  modulefiles ""
}

# Exported commands
set ExportedCommands {
  md5Sum
  md5Init
  replaceExpr
  listFiles
  listSubdirs
  findFiles
  createFile
  createDir
  backupFile
  shortFile
  listSave
  listLoad
  arraySave
  arrayLoad
  arraySetAll
  arrayUnsetAll
  arrayFindElementName
  arrayMaxElementDataLength
  listAppendIf
  listMaxElementLength
  splitStringList
  expandText
  arraySearch
  dataFormatDefault
  dataFormatValue
  dataFormatList
  dataFormatReplace
  dataFormatBuild
  dataFormatConvert
  scanModule
  getModuleDatabaseData
  setModuleDatabaseData
  saveModuleDatabase
  loadModuleDatabase
  updateModuleData
  updateModuleDatabase
  rebuildModuleDatabase
  listModules
  moduleExists
  moduleLoaded
  moduleBindUnbind
  moduleLoad
  moduleUnload
  moduleConfigSave
  moduleConfigLoad
  moduleConfigCheckdefs
  moduleConfig
  moduleDataSave
  moduleDataLoad
  moduleDataBackup
  moduleData
  buildModuleCommandTable
  buildCommandTable
  whichCommand
  whichCommandModule
  whichModule
  configExists
  compareVersion
  wpLog
  callTrace
  listNamespaceChildren
  qualifyCommandName
  secondsElapsed
}

# Exported wrappers for built-in Tcl/Eggdrop commands
set ExportedWrappers {
  mc
  bind
  unbind
}

# Copyright and license information
set Copyright "Wolfpack version ${ModuleVersion}, Copyright (C) 1998-2005 Tothwolf"

set License "This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"


##
## Create md5 checksum for a file
##
## Args:    filename
## Returns: md5 checksum if successful
## Errors:  permission denied,
##          no such file,
##          not a file,
##          can't exec md5 command
##
proc md5Sum {file} {
  variable md5Config

  if {![file exists $file]} then {
    error [mc "%s: no such file" $file]
  } else {
    if {![file isfile $file]} then {
      error [mc "%s: not a file" $file]
    } else {
      if {![file readable $file]} then {
        error [mc "%s: permission denied" $file]
      } else {
        if {[catch {set sum [lindex [exec $md5Config(command) $file] $md5Config(index)]} result]} then {
          error "$file: $result"
        } else {
          return $sum
        }
      }
    }
  }
}

##
## Init md5 command
##
## Args:    none
## Returns: 1 if a useable md5 command found
##          0 otherwise
##
proc md5Init {} {
  variable md5Config
  variable md5Format

  foreach type [array names md5Format] {
    foreach {command index} $md5Format($type) {break}
    if {([catch {exec $command ""} result]) && \
        (![regexp -- {^couldn't execute} $result])} then {
      set md5Config(command) $command
      set md5Config(index) $index
      return 1
    }
  }
  return 0
}

##
## Replace all occurances of an expression in a string with the given text
##
## Args:    string, expr, replacement text
## Returns: string
##
proc replaceExpr {string expr {replace ""}} {
  while {[regsub -all -- $expr $string $replace string]} {}
  return $string
}

##
## List files in a path
##
## Args:    path
## Returns: list of files in the given path,
##          nothing if no files in the given path
## Errors:  permission denied,
##          no such directory,
##          not a directory
##
proc listFiles {path} {
  if {![file exists $path]} then {
    error [mc "%s: no such directory" $path]
  } else {
    if {![file isdirectory $path]} then {
      error [mc "%s: not a directory" $path]
    } else {
      if {![file readable $path]} then {
        error [mc "%s: permission denied" $path]
      } else {
        set ret ""
        foreach name [lsort [glob -nocomplain -- [file join $path *]]] {
          if {[file isfile $name]} then {
            lappend ret $name
          }
        }
        return $ret
      }
    }
  }
}

##
## List subdirs in a path
##
## Args:    path
## Returns: list of subdirs in the given path,
##          nothing if no subdirs in the given path
## Errors:  permission denied,
##          no such directory,
##          not a directory
##
proc listSubdirs {path} {
  if {![file exists $path]} then {
    error [mc "%s: no such directory" $path]
  } else {
    if {![file isdirectory $path]} then {
      error [mc "%s: not a directory" $path]
    } else {
      if {![file readable $path]} then {
        error [mc "%s: permission denied" $path]
      } else {
        set ret ""
        foreach name [lsort [glob -nocomplain -- [file join $path *]]] {
          if {[file isdirectory $name]} then {
            lappend ret $name
          }
        }
        return $ret
      }
    }
  }
}

##
## List files with a set ext in a path and its subdirs up to a set depth
##
## Args:    path, max search depth, file extension
## Returns: list of files with a set ext in the given path and its subdirs,
##          nothing if no matching files are found
##
proc findFiles {path depth {ext ""}} {
  set ret ""
  set foundDirs "$path "
  set searchDirs $path
  for {
    set currentDepth 0
  } {($currentDepth <= $depth) || (!$depth)} {
    incr currentDepth
  } {
    set subDirs ""
    foreach dir $searchDirs {
      if {[catch {set dirList [listSubdirs $dir]} result]} then {
        wpLog o * [mc "Error: unable to get file listing: %s" $result]
      } elseif {[string compare "" $dirList]} then {
        append subDirs $dirList " "
      }
    }
    if {![string compare "" $subDirs]} then {
      break
    }
    append foundDirs $subDirs " "
    set searchDirs $subDirs
  }
  foreach dir $foundDirs {
    if {[catch {set files [listFiles $dir]} result]} then {
      wpLog o * [mc "Error: unable to get file listing: %s" $result]
    } else {
      if {[string compare "" $ext]} then {
        foreach file $files {
          if {![string compare $ext \
                [string tolower [file extension $file]]]} then {
            lappend ret $file
          }
        }
      } else {
        set ret $files
      }
    }
  }
  return $ret
}

##
## Check if a file exists, and create it if not
##
## Args:    filename, verbose {-1,0,1}, description, force new file
## Returns: 1 if the file was created successfully
##          0 if the operation failed
##         -1 if the file already exists
##
# FIXME: [mc 'file']
proc createFile {file {verbose 0} {desc "file "} {force 0}} {
  if {($force) || (![file exists $file])} then {
    if {[catch {set fd [open $file w]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to create %s`%s': %s" $desc [file tail $file] $result]
      }
    } else {
      if {(!$force) && ($verbose >= 1)} then {
        wpLog o * [mc "Warning: %s`%s' does not exist -- creating" $desc [file tail $file]]
      }
      close $fd
      return 1
    }
  } elseif {[file isfile $file]} then {
    return -1
  } elseif {$verbose >= 0} then {
    wpLog o * [mc "Error: not a file: %s" $file]
  }
  return 0
}

##
## Check if a directory exists, and create it if not
##
## Args:    directory, verbose {-1,0,1}, description
## Returns: 1 if the directory was created successfully
##          0 if the operation failed
##         -1 if the directory already exists
##
# FIXME: [mc 'directory']
proc createDir {dir {verbose 0} {desc "directory "}} {
  if {![file exists $dir]} then {
    if {[catch {file mkdir $dir} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to create %s`%s': %s" $desc [file tail $dir] $result]
      }
    } else {
      if {$verbose >= 1} then {
        wpLog o * [mc "Warning: %s`%s' does not exist -- creating" $desc [file tail $dir]]
      }
      return 1
    }
  } elseif {[file isdirectory $dir]} then {
    return -1
  } elseif {$verbose >= 0} then {
    wpLog o * [mc "Error: not a directory: %s" $dir]
  }
  return 0
}

##
## Create a backup of the given file with an optional suffix
##
## Args:    filename, suffix, verbose {-1,0,1}
## Returns:  1 if successful
##          -1 if file is 0 in size
##           0 otherwise
##
proc backupFile {file {verbose 0} {suffix ~bak}} {
  if {[string compare "" $suffix]} then {
    if {[file size $file]} then {
      if {[catch {
        file copy -force $file $file${suffix}
      } result]} then {
        if {$verbose >= 0} then {
          wpLog o * [mc "Error: unable to create backup file for `%s': %s" [file tail $file] $result]
        }
      } else {
        return 1
      }
    } else {
      return -1
    }
  }
  return 0
}

##
## Remove directory component from a filename
##
## Args:    file, pathlist
## Returns: file with directory component removed
##
proc shortFile {file {pathlist ""}} {
  if {[string compare "" $pathlist]} then {
    foreach dir $pathlist {
      set dirlength [string length $dir]
      set filedir [string range $file 0 [expr $dirlength - 1]]
      if {![string compare $dir $filedir]} then {
        return [string trimleft [string range $file $dirlength end] /]
      }
    }
  }
  return [string trimleft [string range $file [string last / $file] end] /]
}

##
## Save data from a list into a file
##
## Args:    list, filename, verbose {-1,0,1}, description, access flag
## Returns: 1 is successful,
##          0 otherwise
##
# FIXME: [mc 'file']
proc listSave {listName file {verbose 0} {desc "file "} {access w}} {
  upvar 1 $listName list

  if {[createFile $file $verbose $desc]} then {
    if {[catch {set fd [open $file $access]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to open %s`%s' for writing: %s" $desc $file $result]
      }
    } else {
      if {[info exists list]} then {
        foreach data $list {
          puts $fd [list $data]
        }
      }
      close $fd
      return 1
    }
  }
  return 0
}

##
## Load data into a list from a file
##
## Args:    list, filename, verbose {-1,0,1}, description, ignore regsub
## Returns: 1 if successful,
##          0 otherwise
##
# FIXME: [mc 'file']
proc listLoad {listName file {verbose 0} {desc "file "} {ignore "^#"}} {
  upvar 1 $listName list

  if {[createFile $file $verbose $desc]} then {
    if {[catch {set fd [open $file r]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to open %s`%s' for reading: %s" $desc $file $result]
      }
    } else {
      if {[info exists list]} then {
        unset list
      }
      while {![eof $fd]} {
        set line [replaceExpr [gets $fd] "^ "]
        if {([string compare "" $line]) && \
            (![regexp -- $ignore $line])} then {
          append list $line " "
        }
      }
      close $fd
      return 1
    }
  }
  return 0
}

##
## Save data from an array into a file
##
## Args:    array, filename, verbose {-1,0,1}, description, access flag
## Returns: 1 is successful,
##          0 otherwise
##
# FIXME: [mc 'file']
proc arraySave {arrayName file {verbose 0} {desc "file "} {access w}} {
  upvar 1 $arrayName array

  if {[createFile $file $verbose $desc]} then {
    if {[catch {set fd [open $file $access]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to open %s`%s' for writing: %s" $desc $file $result]
      }
    } else {
      if {[array exists array]} then {
        foreach name [lsort [array names array]] {
          puts $fd "[list $name] [list $array($name)]"
        }
        close $fd
        return 1
      } else {
        close $fd
      }
    }
  }
  return 0
}

##
## Load data into an array from a file
##
## Args:    array, filename, verbose {-1,0,1}, description, ignore regsub
## Returns: 1 if successful,
##          0 otherwise
##
# FIXME: [mc 'file']
proc arrayLoad {arrayName file {verbose 0} {desc "file "} {ignore "^#"}} {
  upvar 1 $arrayName array

  if {[createFile $file $verbose $desc]} then {
    if {[catch {set fd [open $file r]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to open %s`%s' for reading: %s" $desc $file $result]
      }
    } else {
      if {[info exists array]} then {
        unset array
      }
      while {![eof $fd]} {
        set line [replaceExpr [gets $fd] "^ "]
        if {([string compare "" $line]) && \
            (![regexp -- $ignore $line])} then {
          set array([lindex $line 0]) [lindex $line 1]
        }
      }
      close $fd
      return 1
    }
  }
  return 0
}

##
## Set all elements in the given array the the given value
##
## Args:    array name, value
## Returns: 1 if the array exists
##          0 otherwise
##
proc arraySetAll {arrayName {value ""}} {
  upvar 1 $arrayName array

  if {[array exists array]} then {
    foreach name [array names array] {
      set array($name) $value
    }
    return 1
  }
  return 0
}

##
## Unset all elements in the given array
##
## Args:    array name
## Returns: 1 if the array exists
##          0 otherwise
##
proc arrayUnsetAll {arrayName} {
  upvar 1 $arrayName array

  if {[array exists array]} then {
    foreach name [array names array] {
      unset array($name)
    }
    return 1
  }
  return 0
}

##
## Find the given element in an array
##
## Args:    array name, element name
## Returns: case sensitive element name if found,
##          nothing otherwise
##
proc arrayFindElementName {arrayName element} {
  upvar 1 $arrayName array

  set list [lsort [array names array]]
  set index [lsearch -exact [string tolower $list] [string tolower $name]]
  if {$index != -1} then {
    return [lindex $list $index]
  }
  return
}

##
## Return length of longest data in an array at index
##
## Args:    array name
## Returns: length of longest name in an array
##
proc arrayMaxElementDataLength {arrayName index} {
  upvar 1 $arrayName array

  set maxlength 0
  foreach {name data} [array get array] {
    set length [string length [lindex $data $index]]
    if {$length > $maxlength} then {
      set maxlength $length
    }
  }
  return $maxlength
}

##
## Append something to the given list if it is not already in the list
##
## Args:    listVar, what
## Returns: list
##
proc listAppendIf {listVar {what ""}} {
  upvar 1 $listVar list

  if {([string compare "" $what]) &&
      ((![info exists list]) || ([lsearch -exact $list $what] == -1))} then {
    lappend list $what
  }
  return $list
}

##
## Return length of the longest element in a list
##
## Args:    list, index
## Returns: length of longest element in the given list
##
proc listMaxElementLength {list {index 0}} {
  set maxlength 0
  foreach data $list {
    set length [string length [lindex $data $index]]
    if {$length > $maxlength} then {
      set maxlength $length
    }
  }
  return $maxlength
}

##
## Split a string into multiple list elements
##
## Args:    text, max list length, split char, trim chars
## Returns: split list
##
# FIXME: improve this
proc splitStringList {text {splitLength 75} {splitChar " "} {trimChars " "}} {
  # Sanity check splitLength and splitChar
  if {($splitLength >= 1) && ([string compare "" $splitChar])} then {
    set elementSplitLength [expr $splitLength - 1]
    set stringLength [string length $text]	;# Total length of string
    set subStringLength 0			;# Text left over
    set elementLength 0				;# Element length counter
    set elementStartIndex 0			;# Start of split element
    set elementEndIndex 0			;# End of split element
    for {
      set stringIndex 0
    } {$stringIndex < $stringLength} {
      incr stringIndex
    } {
      # If element length greater than/equal to split length,
      # Or element length equal to split length - 1,
      # And character at current string index is splitChar
      if {(($elementLength >= $splitLength) ||
           ($elementLength == $elementSplitLength)) &&
          (![string compare $splitChar \
             [string index $text $stringIndex]])} then {
        # Split substring element from text
        set string [string range $text $elementStartIndex $elementEndIndex]
        # Append substring element list to list
        lappend list [string trim $string $trimChars]
        # Reset element length counter
        set elementLength 0
        # Start split of next element at the end + 1 of the current one
        set elementStartIndex [expr $elementEndIndex + 1]
        # Reset end of next element to the start of the next element
        set elementEndIndex $elementStartIndex
        # Track remaining text length
        set subStringLength [expr $subStringLength + [string length $string]]
      } else {
        # Increment element length
        incr elementLength
        # Increment end of next element
        incr elementEndIndex
      }
    }
    # Append any left over text as a new element
    if {$stringLength > $subStringLength} then {
      lappend list [string trim [string range $text $subStringLength end] $trimChars]
    }
    # Whew...that was alot of work!
    if {[info exists list]} then {
      return $list
    }
  }
  return
}

##
## Expand the given text with a list
##
## Args:    text, list, elementIndex, resultVar
## Returns: 0 if text not found
##          1 if text is exactly matched
##          2 if text is matched
##          -1 if text is ambiguous
##
proc expandText {text list elementIndex resultVar} {
  upvar 1 $resultVar result

  set found 0
  set ambiguous 0
  set lowerText [string tolower $text]
  foreach listElement $list {
    set compareElement [lindex $listElement $elementIndex]

    # Exact match
    if {![string compare $text $compareElement]} then {
      set result $listElement
      return 1

    # Partial match
    } elseif {[string match "$lowerText*" [string tolower $compareElement]]} then {
      if {!$found} then {
        set result $listElement
        set found 1
      } else {
        set ambiguous 1
      }
    }
  }
  if {$ambiguous} then {
    return -1
  } elseif {$found} then {
    return 2
  }
  return 0
}

##
## Search an array for a given word or regexp
##
## Args:    array name, word/regexp
## Returns: list of indexes that match the given word/regexp
##
proc arraySearch {arrayName word} {
  upvar 1 $arrayName array

  set word [string tolower $word]
  set ret ""
  foreach {name data} [array get $array] {
    set string [string tolower $data]
    if {[lsearch -regexp $string $word] != -1} then {
      for {
        set index 0
        set indexes ""
      } {
        if {[regexp -- "$word" $string]} then {
          lappend indexes $index
        }
      } {
        incr index
      }
      lappend ret [list [concat $name $indexes]]
    }
  }
  return $ret
}

##
## Find option default for the given option name in a data list
##
## Args:    data list, option name
## Returns: option default if found,
##          nothing otherwise
##
proc dataFormatDefault {list option} {
  foreach i $list {
    if {![string compare $option [lindex $i 0]]} then {
      return [lindex $i 1]
    }
  }
  return
}

##
## Find option value for the given option name in a data list
##
## Args:    data format, data list, option name
## Returns: option value if found,
##          nothing otherwise
##
proc dataFormatValue {format data option} {
  if {[set index [lsearch -exact $format $option]] != -1} then {
    return [lindex $data $index]
  }
  return
}

##
## Build list of data from the given options
##
## Args:    data format, data list, option names
## Returns: list of data from the given options
##
proc dataFormatList {format data options} {
  set ret ""
  foreach option $options {
    if {[set index [lsearch -exact $format $option]] != -1} then {
      lappend ret [lindex $data $index]
    }
  }
  return $ret
}

##
## Replace option data in the given data list with a new value
##
## Args:    data format, data list, option name, new value
## Returns: data list
##
proc dataFormatReplace {format data option value} {
  if {[set index [lsearch -exact $format $option]] != -1} then {
    return [lreplace $data $index $index $value]
  }
  return $data
}

##
## Create a data format list for a given data format and options
##
## Args:    data format list, options {{option1 value} ...}
## Returns: data format list with options and values in proper order
##
proc dataFormatBuild {format args} {
  set ret ""
  foreach arg $args {
    set tmp([lindex $arg 0]) [lindex $arg 1]
  }
  foreach opt $format {
    if {[info exists tmp($opt)]} then {
      lappend ret $tmp($opt)]
    } else {
      lappend ret [dataFormatDefault $format $opt]
    }
  }
  return $ret
}

##
## Convert a data list from one format to another
##
## Args:    from format, to format, data list
## Returns: data list
##
proc dataFormatConvert {fromFormat toFormat data} {
  set ret ""
  set index 0
  foreach opt $fromFormat {
    set tmp($opt) [lindex $data $index]
    incr index
  }
  foreach opt $toFormat {
    if {[info exists tmp($opt)]} then {
      lappend ret $tmp($opt)
    } else {
      lappend ret [dataFormatDefault $toFormat $opt]
    }
  }
  return $ret
}

##
## Scan the given file for module options
##
## Args:    file, args {only scan for these options}
## Returns: list of module options if the given file is a module,
##          nothing otherwise
## Errors:  unable to open file for reading
##
proc scanModule {file args} {
  variable moduleDatabaseConfig
  variable moduleDatabaseFormat

  if {[catch {set fd [open $file r]} result]} then {
    error $result
  } else {
    set ret ""
    if {![string compare "" $args]} then {
      set baseOptions [lindex $moduleDatabaseFormat([set moduleDatabaseConfig(version)]) 0]
      set extraOptions [lindex $moduleDatabaseFormat([set moduleDatabaseConfig(version)]) 1]
      set scanOptions "name $baseOptions"
      set formatOptions "name $baseOptions $extraOptions"
    } else {
      set scanOptions $args
      set formatOptions $args
    }
    for {
      set lineCount 0
      set optionCount 0
      set continuedLine 0
    } {(![eof $fd]) && ($lineCount <= $moduleDatabaseConfig(scanlines))} {
      incr lineCount
    } {
      gets $fd line
      if {[regexp -- {^# .*:.*} $line]} then {
        set opt [string trimright [lindex $line 1] :]
        if {[lsearch -glob $scanOptions $opt] != -1} then {
          set data [string trimright [string trimleft [string range $line [string first : $line] end] " \t:"] " \t\\"]
          if {![info exists $opt]} then {
            set $opt $data
          } else {
            append $opt " $data"
          }
        }
        if {[regexp -- {\\$} $line]} then {
          set continuedLine 1
        } else {
          set continuedLine 0
        }
      } elseif {($continuedLine) && ([info exists opt])} then {
        append $opt " [string trimright [string trimleft $line " \t#"] " \t\\"]"
        if {![regexp -- {\\$} $line]} then {
          set continuedLine 0
        }
      }
    }
    close $fd
    if {(![string compare "" $args]) && \
        ((![info exists name]) || \
         ([catch {set md5sum [md5Sum $file]}]))} then {
      return
    }
    foreach option $formatOptions {
      if {(![info exists $option]) || \
          (![string compare "" [set $option]])} then {
        set $option [dataFormatDefault $moduleDatabaseConfig(defaults) $option]
      }
      lappend ret [set $option]
    }
    return $ret
  }
}

##
## Get data from module db data array
##
## Args:    module name, data type
## Returns: data for the given module's data type if it exists,
##          nothing otherwise
##
proc getModuleDatabaseData {module type} {
  variable moduleDatabaseConfig
  variable moduleDatabaseFormat
  variable moduleDatabaseData

  if {[moduleExists $module]} then {
    set index [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] $type]
    if {$index != -1} then {
      return [lindex $moduleDatabaseData($module) $index]
    }
  }
  return
}

##
## Set data in module db data array
##
## Args:    module name, data type, data
## Returns: 1 if valid module and data type,
##          0 otherwise
##
proc setModuleDatabaseData {module type data} {
  variable moduleDatabaseConfig
  variable moduleDatabaseFormat
  variable moduleDatabaseData
  variable moduleDatabaseDataChanged

  if {[moduleExists $module]} then {
    set index [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] $type]
    if {$index != -1} then {
      set moduleDatabaseData($module) [lreplace $moduleDatabaseData($module) $index $index [list $data]]
      set moduleDatabaseDataChanged 1
      return 1
    }
  }
  return 0
}

##
## Save module database
##
## Args:    verbose {-1,0,1}
## Returns: 1 if successful,
##          0 otherwise
##
proc saveModuleDatabase {{verbose 0}} {
  variable configData
  variable moduleDatabaseConfig
  variable moduleDatabaseData
  variable moduleDatabaseDataChanged

# FIXME [mc] string ends with ' '
  if {[createFile $configData(moddbfile) $verbose [mc "module database file "]]} then {
    if {[catch {set fd [open $configData(moddbfile) w]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to open module database file `%s' for writing: %s" $configData(moddbfile) $result]
      }
    } else {
      puts $fd "# $moduleDatabaseConfig(header)$moduleDatabaseConfig(version)"
      close $fd
# FIXME [mc] string ends with ' '
      set ret [arraySave moduleDatabaseData $configData(moddbfile) $verbose [mc "module database file "] a]
      if {$ret} then {
        set moduleDatabaseDataChanged 0
      }
      return $ret
    }
  }
  return 0
}

##
## Load module database
##
## Args:    verbose {-1,0,1}
## Returns: 1 if successful,
##          0 otherwise
##
proc loadModuleDatabase {{verbose 0}} {
  variable configData
  variable moduleDatabaseConfig
  variable moduleDatabaseFormat
  variable moduleDatabaseData
  variable moduleDatabaseDataChanged

  if {![file exists $configData(moddbfile)]} then {
    return -1
  } else {
    if {[catch {set fd [open $configData(moddbfile) r]} result]} then {
      if {$verbose >= 0} then {
        wpLog o * [mc "Error: unable to open module database file `%s' for reading: %s" $configData(moddbfile) $result]
      }
    } else {
      set firstline [replaceExpr [gets $fd] "^ "]
      if {[regexp -- "^# $moduleDatabaseConfig(header)" $firstline]} then {
        regsub -- "^# $moduleDatabaseConfig(header)" $firstline "" version
        if {![string compare [set version [string trim $version]] $moduleDatabaseConfig(version)]} then {
          close $fd
          return [arrayLoad moduleDatabaseData $configData(moddbfile) $verbose "module database file "]
        } elseif {[info exists moduleDatabaseFormat($version)]} then {
          if {[info exists moduleDatabaseData]} then {
            unset moduleDatabaseData
          }
          while {![eof $fd]} {
            set line [replaceExpr [gets $fd] "^ "]
            if {([string compare "" $line]) && \
                (![regexp -- {^#} $line])} then {
              set moduleDatabaseData([lindex $line 0]) [dataFormatConvert [join $moduleDatabaseFormat($version)] [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] [lindex $line 1]]
            }
          }
          close $fd
# FIXME: should this be set to 1 when converting database?
          set moduleDatabaseDataChanged 0
          return 1
        } else {
          wpLog o * [mc "Error: unknown module database version: %s" $version]
        }
      } else {
        wpLog o * [mc "Error: unknown module database format: %s" [string trimleft $firstline " \t#"]]
      }
    }
    close $fd
  }
  return 0
}

##
## Add module data to the module database
##
## Args:    file
## Returns: nothing
## Errors:  if can't open file for reading
##
proc updateModuleData {file} {
  variable moduleDatabaseConfig
  variable moduleDatabaseFormat
  variable moduleDatabaseData
  variable moduleDatabaseDataChanged

  if {[catch {set data [scanModule $file]} result]} then {
    error $result
  } else {
    set name [lindex $data 0]
    if {[string compare "" $name]} then {
      if {[moduleExists $name]} then {
        set loadIndex [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] load]
        set moduleDatabaseData($name) [lreplace [lrange $data 1 end] $loadIndex $loadIndex [lindex $moduleDatabaseData($name) $loadIndex]]
      } else {
        set moduleDatabaseData($name) [lrange $data 1 end]
      }
      set moduleDatabaseDataChanged 1
    }
  }
  return
}

##
## Update module database
##
## Args:    verbose {-1,0,1}
## Returns: nothing
##
proc updateModuleDatabase {{verbose 0}} {
  variable modulePath
  variable moduleDatabaseConfig
  variable moduleDatabaseData
  variable moduleDatabaseDataChanged
  variable moduleOptionData

  set moduleList [listModules]

  set foundFiles ""	;# List of '.tcl' files found in module path
  set comparedFiles ""	;# List of compared module files

  # Only update the given module(s) when the '-m <file>' option is used.
  if {[string compare "" $moduleOptionData(modulefiles)]} then {
    set fullSearch 0
    set foundFiles $moduleOptionData(modulefiles)
  } else {
    set fullSearch 1

    # Build up file list
    foreach dir $modulePath {
      foreach file [findFiles $dir $moduleDatabaseConfig(maxdepth) .tcl] {
        lappend foundFiles $file
      }
    }
  }

  # Find removed files
  foreach module $moduleList {
    set file [getModuleDatabaseData $module file]

    # Only update the given modules if the '-m' option was used
    if {($fullSearch) ||
        ([lsearch -exact $moduleOptionData(modulefiles) $file] != -1)} then {
      set shortfile [shortFile $file $modulePath]

      # Remove any invalid module data
      if {[lsearch -exact $foundFiles $file] == -1} then {
        if {([lsearch -exact $moduleOptionData(excludedfiles) $file] != -1)} then {
          if {$verbose >= 1} then {
            wpLog o * [mc "Not removing module data for excluded missing file `%s' (%s)" $shortfile $module]
          }
        } else {
          if {$verbose >= 1} then {
            wpLog o * [mc "Removing module data for missing file `%s' (%s)" $shortfile $module]
          }
          unset moduleDatabaseData($module)
          set moduleDatabaseDataChanged 1
        }

      # Compare existing valid modules
      } else {
        if {([lsearch -exact $moduleOptionData(excludedfiles) $file] != -1)} then {
          if {$verbose >= 1} then {
            wpLog o * [mc "Not comparing excluded file `%s'" $shortfile]
          }
        } else {
          if {$verbose >= 1} then {
            wpLog o * [mc "Comparing file `%s'" $shortfile]
          }

          # Compare md5 from module db and make sure the module hasn't changed
          if {([catch {set md5sum [md5Sum $file]}]) || \
              ([string compare [getModuleDatabaseData $module md5sum] $md5sum])} then {
            if {$verbose >= 1} then {
              wpLog o * [mc "Updating module information for file `%s'" $shortfile]
            }
            if {[catch {updateModuleData $file} result]} then {
              if {$verbose >= 0} then {
                wpLog o * [mc "Warning: unable to open file `%s' for reading: %s" $shortfile $result]
              }
            }
          }
        }
        lappend comparedFiles $file
      }
    }
  }

  # Find new module files
  foreach file $foundFiles {

    # Only update the given modules if the '-m' option was used
    if {($fullSearch) ||
        ([lsearch -exact $moduleOptionData(modulefiles) $file] != -1)} then {
      set shortfile [shortFile $file $modulePath]

      if {[lsearch -exact $comparedFiles $file] == -1} then {
        if {([lsearch -exact $moduleOptionData(excludedfiles) $file] != -1)} then {
          if {$verbose >= 1} then {
            wpLog o * [mc "Not adding module information for excluded file `%s'" $shortfile]
          }
        } else {
          if {$verbose >= 1} then {
            wpLog o * [mc "Adding module information for file `%s'" $shortfile]
          }
          if {[catch {updateModuleData $file} result]} then {
            if {$verbose >= 0} then {
              wpLog o * [mc "Warning: unable to open file `%s' for reading: %s" $shortfile $result]
            }
          }
        }
      }
    }
  }
  return
}

##
## Rebuild module database
##
## Args:    verbose {-1,0,1}
## Returns: nothing
##
proc rebuildModuleDatabase {{verbose 0}} {
  variable modulePath
  variable moduleDatabaseConfig
  variable moduleDatabaseFormat
  variable moduleDatabaseData

  # Copy database data for later use
  if {[info exists moduleDatabaseData]} then {
    set mergeLoad 1
    array set moduleDatabaseDataTmp [array get moduleDatabaseData]
    unset moduleDatabaseData
  } else {
    set mergeLoad 0
  }

  foreach dir $modulePath {
    foreach file [findFiles $dir $moduleDatabaseConfig(maxdepth) .tcl] {
      set shortfile [shortFile $file $modulePath]

      if {$verbose >= 1} then {
        wpLog o * [mc "Scanning file `%s'" $shortfile]
      }
      if {[catch {updateModuleData $file} result]} then {
        if {$verbose >= 0} then {
          wpLog o * [mc "Warning: unable to open file `%s' for reading: %s" $shortfile $result]
        }
      }
    }
  }

  # Merge load data into new database
  if {$mergeLoad} then {
    set loadIndex [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] load]

    foreach module [listModules] {
      set moduleDatabaseData($module) [lreplace $moduleDatabaseData($module) $loadIndex $loadIndex [lindex $moduleDatabaseDataTmp($module) $loadIndex]]
    }
    # Note: Not modifying moduleDatabaseDataChanged here, since
    #       'updateModuleData' will have already done so.
  }
  return
}

##
## List all modules in the database
##
## Args:    none
## Returns: list of modules in module database
##
proc listModules {{loaded 0}} {
  variable moduleDatabaseData
  variable moduleLoadedList

  if {$loaded} then {
    return [lsort $moduleLoadedList]
  }
  return [lsort [array names moduleDatabaseData]]
}

##
## Check if the given module exists
##
## Args:    module name
## Returns: 1 if the given module exists
##          0 otherwise
##
proc moduleExists {module} {
  variable moduleDatabaseData

  if {[info exists moduleDatabaseData($module)]} then {
    return 1
  }
  return 0
}

##
## Check if a module is loaded
##
## Args:    module name
## Returns: 1 if the given module is loaded
##          0 otherwise
##
proc moduleLoaded {module} {
  variable moduleLoadedList

  if {[lsearch -exact $moduleLoadedList $module] != -1} then {
    return 1
  }
  return 0
}

##
## Add/remove bindings for a given module
##
## Args:    mode {bind|unbind}, module,
##           args {{type ...} {option ...} {cmdsub ...} {regsub ...}}
## Returns: nothing
##
## Important variables:
##   argTypes "dcc msg pub ..."
##   argOptions "noauto cmdchr ..."
##   argCmdsub(regexp) "command"
##   argRegsub(regexp) "with"
##   optCmdsub(regexp) "command"
##   optRegsub(regexp) "with"
##
proc moduleBindUnbind {mode module args} {
  variable NamespaceCurrent

  if {[info exists ${NamespaceCurrent}::${module}::bindDefaults]} then {
    # These are for use in calling this proc directly.
    # bindDefaults options are further below
    set argTypes ""
    set argOptions ""
    foreach arg $args {
      switch -exact -- [lindex $arg 0] {
        type {
          # Specific types to match against
          # dcc msg pub ...
          set argTypes [lrange $arg 1 end]
        }
        option {
          # Specific options to match against
          # noauto cmdchr ...
          set argOptions [lrange $arg 1 end]
        }
        cmdsub {
          # Replace 'regexp' with result of 'command'
          # NOTE: 'command' will eventually be processed in calling stack
          foreach {command regexp} [lindex $arg 1] {break}
          # FIXME: better fix for leading '+/$' (?)
          regsub -- {^([+$])} $regexp {\\\1} regexp
          # FIXME: can't do this (ugh...why *can't* we do this???)
          #regsub -- {^([][\\.+*?^$(){}|])} $regexp {\\\1} regexp
          # Try to find 'command' in 'module' namespace
          if {[string compare "" [info commands ${NamespaceCurrent}::${module}::[lindex $command 0]]]} then {
            # Command is module specific or imported
            set argCmdsub($regexp) ${NamespaceCurrent}::${module}::$command
          } else {
            # Must be a global command
            set argCmdsub($regexp) $command
          }
        }
        regsub {
          # Replace regexp 'rwhat' with 'rwith'
          foreach {rwhat rwith} [lindex $arg 1] {break}
          # FIXME: better fix for leading '+/$' (?)
          regsub -- {^([+$])} $rwhat {\\\1} rwhat
          set argRegsub($rwhat) $rwith
        }
      }
    }
    foreach {proc data} [array get ${NamespaceCurrent}::${module}::bindDefaults] {
      foreach bind $data {
        foreach {type flags mask options help} $bind {break}
        # Continue if a specific bind type is requested and not matched
        if {([string compare "" $argTypes]) && \
            ([lsearch -exact $argTypes $type] == -1)} then {
          continue
        }
        # Sanity check!
        # Continue if argOptions specified and bind options don't exist
        if {([string compare "" $argOptions]) && \
            (![string compare "" $options])} then {
          continue
        }
        # These _must_ be clean since they are reused for multiple binds
        set continue 0
        if {[info exists optCmdsub]} then {
          unset optCmdsub
        }
        if {[info exists optRegsub]} then {
          unset optRegsub
        }
        # Process bind specific options
        foreach option $options {
          set optcmd [lindex $option 0]
          # Search 'argOptions' for 'optcmd'
          # Abort bind and continue with next if not found
          if {[string compare "" $argOptions]} then {
            set found 0
            foreach argoption $argOptions {
              if {![string compare $optcmd [lindex $argoption 0]]} then {
                set found 1
              }
            }
            if {!$found} then {
              set continue 1
              break
            }
          }
          switch -exact -- $optcmd {
            noauto {
              # Search for noauto override in argOptions
              if {[lsearch -exact $argOptions noauto] == -1} then {
                # noauto matched and not overriden
                set continue 1
                break
              }
            }
            cmdsub {
              # Replace 'regexp' with result of 'command'
              # NOTE: 'command' will eventually be processed in calling stack
              foreach {command regexp} [lindex $option 1] {break}
              # FIXME: better fix for leading '+/$' (?)
              regsub -- {^([+$])} $regexp {\\\1} regexp
              # Try to find 'command' in 'module' namespace
              if {[string compare "" [info commands ${NamespaceCurrent}::${module}::[lindex $command 0]]]} then {
                # Command is module specific or imported
                set optCmdsub($regexp) ${NamespaceCurrent}::${module}::$command
              } else {
                # Must be a global command
                set optCmdsub($regexp) $command
              }
            }
            regsub {
              # Replace regexp 'rwhat' with 'rwith'
              foreach {rwhat rwith} [lindex $option 1] {break}
              # FIXME: better fix for leading '+/$' (?)
              regsub -- {^([+$])} $rwhat {\\\1} rwhat
              set optRegsub($rwhat) $rwith
            }
          }
        }
        # Abort this bind and continue with the next
        if {$continue} then {
          continue
        }
        # Make optCmdsub regexp substitutions on mask
        if {[array exists optCmdsub]} then {
          foreach {regexp command} [array get optCmdsub] {
            # Process 'command' in calling stack
            regsub -all -- $regexp $mask [uplevel 1 [list if 1 $command]] mask
          }
        }
        # Make optRegsub regexp substitutions on mask
        if {[array exists optRegsub]} then {
          foreach {replacewhat replacewith} [array get optRegsub] {
            regsub -all -- $replacewhat $mask $replacewith mask
          }
        }
        # Make argCmdsub regexp substitutions on mask
        if {[array exists argCmdsub]} then {
          foreach {regexp command} [array get argCmdsub] {
            # Process 'command' in calling stack
            regsub -all -- $regexp $mask [uplevel 1 [list if 1 $command]] mask
          }
        }
        # Make argRegsub regexp substitutions on mask
        if {[array exists argRegsub]} then {
          foreach {replacewhat replacewith} [array get argRegsub] {
            regsub -all -- $replacewhat $mask $replacewith mask
          }
        }
        # Finally!  bind/unbind  ($mode == 'bind' or 'unbind')
        # FIXME: do this inside of a 'namespace eval'
        if {[catch {
          $mode $type $flags $mask ${NamespaceCurrent}::${module}::$proc
        } result]} then {
          wpLog d * [mc "Error: unable to %s %s %s to %s: %s" $mode $type $mask "${NamespaceCurrent}::${module}::$proc" $result]
        }
      }
    }
  }
  return
}

##
## Load a module
##
## Args:    module name, verbose {-1,0,1}, args {loop detection}
## Returns: nothing
## Errors:  if unable to load module
##
proc moduleLoad {module {verbose 0} args} {
  variable NamespaceCurrent
  variable ExportedWrappers
  variable moduleLoadedList

  if {[lsearch -exact [set loop [lindex $args 0]] $module] == -1} then {
    if {[moduleExists $module]} then {
      set preload ""
      set requires [getModuleDatabaseData $module requires]
      foreach required $requires {
        set preloadModule [whichCommandModule $required]
        if {[string compare "" $preloadModule]} then {
          if {([lsearch -exact $preload $preloadModule] == -1) && \
              ([string compare wp $preloadModule]) && \
              (![moduleLoaded $preloadModule])} then {
            lappend preload $preloadModule
          }
        } else {
          error [mc "required command `%s' not found." $required]
        }
      }
      if {[string compare "" $preload]} then {
        foreach premod $preload {
          if {[catch {moduleLoad $premod $verbose [concat $loop $module]} result]} then {
            error $result
          }
        }
      }
      if {[catch {

        # Load module into its own namespace
        set namespaceScript "namespace eval ${NamespaceCurrent}::${module} \{\n"
        append namespaceScript "  source \"[getModuleDatabaseData $module file]\"\n"
        append namespaceScript "\}"
        if 1 $namespaceScript

      } result]} then {
        error $result
      } else {

        # ModulePreInit
        if {[string compare "" [info commands ${NamespaceCurrent}::${module}::ModulePreInit]]} then {
          ${NamespaceCurrent}::${module}::ModulePreInit
        }

        set version [getModuleDatabaseData $module version]

        # FIXME: do this elsewhere?
        # Build list of command wrappers
        set commandWrappers ""
        foreach wrapper $ExportedWrappers {
          append commandWrappers " ${NamespaceCurrent}::$wrapper"
        }

        # Package version management
        package forget $module
        package provide ${NamespaceCurrent}::${module} $version

        # Set standard module variables
        set namespaceScript "namespace eval ${NamespaceCurrent}::${module} \{\n"
        append namespaceScript "  set NamespaceParent \[namespace parent\]\n"
        append namespaceScript "  set NamespaceCurrent \[namespace current\]\n"
        append namespaceScript "  set ModuleName $module\n"
        append namespaceScript "  set ModuleVersion $version\n"
        append namespaceScript "  namespace forget *\n"

        # Install command wrappers in module namespace
        if {[string compare "" $commandWrappers]} then {
          append namespaceScript "  namespace import $commandWrappers\n"
        }

        # Imported commands '# requires: ...'
        if {[string compare "" $requires]} then {
          set namespaceScriptTmp ""
          foreach required $requires {
            if {[string compare "" [set command [whichCommand $required]]]} then {
              append namespaceScriptTmp " $command"
            } else {
# FIXME: unknown command
            }
          }
          if {[string compare "" $namespaceScriptTmp]} then {
            append namespaceScript "  namespace import $namespaceScriptTmp\n"
          }
        }

        # Exported commands '# provides: ...'
        if {[string compare "" [set provides [getModuleDatabaseData $module provides]]]} then {
          set namespaceScriptTmp ""
          foreach provided $provides {
            append namespaceScriptTmp " $provided"
          }
          if {[string compare "" $namespaceScriptTmp]} then {
            append namespaceScript "namespace export $namespaceScriptTmp\n"
          }
        }

        # Close and eval
        append namespaceScript "\}"
        if 1 $namespaceScript

        # Install command wrappers in all child namespaces
        if {[string compare "" $commandWrappers]} then {
          foreach namespaceChild [listNamespaceChildren ${NamespaceCurrent}::${module}] {
            set namespaceScript "namespace eval $namespaceChild \{\n"
            append namespaceScript "  namespace import $commandWrappers\n"
            append namespaceScript "\}"
            if 1 $namespaceScript
          }
        }

        # configData and module specific data
        moduleConfigLoad $module 1
        moduleConfigCheckdefs $module 1
        moduleDataLoad $module 1

        # ModuleInit
        if {[string compare "" [info commands ${NamespaceCurrent}::${module}::ModuleInit]]} then {
          ${NamespaceCurrent}::${module}::ModuleInit
        }

# FIXME: check for bindings? duplicates?
        # Create binds
        moduleBindUnbind bind $module

        # Flag module as loaded
        if {![getModuleDatabaseData $module load]} then {
          setModuleDatabaseData $module load 1
        }

        # Add module to moduleLoadedList
        if {[lsearch -exact $moduleLoadedList $module] == -1} then {
          lappend moduleLoadedList $module
          if {$verbose >= 1} then {
            wpLog o * [mc "Module loaded: %s" $module]
          }
        }
      }
    } else {
      error [mc "No such module: %s" $module]
    }
  } else {
    regsub -all -- { } $loop { -> } loop
    error [mc "Module preload endless loop: %s -> %s" $loop $module]
  }
  return
}

##
## Unload a module
##
## Args:    module name, verbose {-1,0,1}
## Returns: nothing
## Errors:  if unable to completely unload module
##
proc moduleUnload {module {verbose 0}} {
  variable NamespaceCurrent
  variable moduleLoadedList

# FIXME: handle dependant modules and modules that can't be unloaded
  if {[moduleExists $module]} then {
    if {[string compare "" [info commands ${NamespaceCurrent}::${module}::ModuleDestroy]]} then {
      ${NamespaceCurrent}::${module}::ModuleDestroy
    }
# FIXME: check for bindings?
    moduleBindUnbind unbind $module
    moduleConfigSave $module 1
    moduleDataSave $module 1

# FIXME: we should kill off stale timer/utimers here somewhere before unloading

    if {[catch {namespace delete ${NamespaceCurrent}::${module}} result]} then {
      error $result
    } else {
      package forget ${NamespaceCurrent}::${module}
      if {[getModuleDatabaseData $module load] == 1} then {
        setModuleDatabaseData $module load 0
      }
      set index [lsearch -exact $moduleLoadedList $module]
      if {$index != -1} then {
        set moduleLoadedList [lreplace $moduleLoadedList $index $index]
        if {$verbose >= 1} then {
          wpLog o * [mc "Module unloaded: %s" $module]
        }
      }
    }
  } else {
    error [mc "No such module: %s" $module]
  }
  return
}

##
## Save configuration settings for the given module
##
## Args:    module, force {0,1}, verbose {-1,0,1}
## Returns: 1 if settings saved
##          0 otherwise
##
proc moduleConfigSave {module {force 0} {verbose 0}} {
  variable NamespaceCurrent
  variable configData

  if {([string compare "" \
        [set file [getModuleDatabaseData $module config]]]) && \
      ([createDir $configData(configpath)])} then {
    set cfgfile [file join $configData(configpath) $file]
    if {([getModuleDatabaseData $module load]) && \
        (($force) || \
         (![info exists ${NamespaceCurrent}::${module}::configDataChanged]) || \
         ([set ${NamespaceCurrent}::${module}::configDataChanged]))} then {
      if {[arraySave ${NamespaceCurrent}::${module}::configData $cfgfile 0 "$module configuration file "]} then {
        if {$verbose >= 1} then {
          wpLog o * [mc "Writing %s config file..." $module]
        }
        set ${NamespaceCurrent}::${module}::configDataChanged 0
        return 1
      } elseif {$verbose >= 0} then {
        wpLog o * [mc "Error writing %s config file." $module]
      }
    }
  }
  return 0
}

##
## Load configuration settings for the given module
##
## Args:    module, force {0,1}, verbose {-1,0,1}
## Returns: 1 if settings loaded
##          0 otherwise
##
proc moduleConfigLoad {module {force 0} {verbose 0}} {
  variable NamespaceCurrent
  variable configData

  if {([string compare "" \
         [set file [getModuleDatabaseData $module config]]]) && \
       ([createDir $configData(configpath)])} then {
    set cfgfile [file join $configData(configpath) $file]
    if {($force) || \
        (![info exists ${NamespaceCurrent}::${module}::configDataChanged]) || \
        (![set ${NamespaceCurrent}::${module}::configDataChanged])} then {
      if {[arrayLoad ${NamespaceCurrent}::${module}::configData $cfgfile 0 "$module configuration file "]} then {
        if {$verbose >= 1} then {
          wpLog o * [mc "Loading %s config file..." $module]
        }
        set ${NamespaceCurrent}::${module}::configDataChanged 0
        return 1
      } elseif {$verbose >= 0} then {
        wpLog o * [mc "Error loading %s config file." $module]
      }
    }
  }
  return 0
}

##
## Check and set default configuration settings for the given module
##
## Args:    module, force {0,1}, verbose {-1,0,1}
## Returns: nothing
##
proc moduleConfigCheckdefs {module {force 0} {verbose 0}} {
  variable NamespaceCurrent

  if {([array exists ${NamespaceCurrent}::${module}::configDefaults]) && \
      ([string compare "" [getModuleDatabaseData $module config]])} then {
    set Changed 0
    # Unset unknown variables
    foreach name [array names ${NamespaceCurrent}::${module}::configData] {
      if {![info exists ${NamespaceCurrent}::${module}::configDefaults($name)]} then {
        unset ${NamespaceCurrent}::${module}::configData($name)
        set Changed 1
      }
    }
    # Set missing variables to defaults
    foreach {name data} [array get ${NamespaceCurrent}::${module}::configDefaults] {
      if {![info exists ${NamespaceCurrent}::${module}::configData($name)]} then {
        set ${NamespaceCurrent}::${module}::configData($name) [lindex $data 1]
        set Changed 1
      }
    }
# FIXME: do this with a trace?
    if {$Changed} then {
      set ${NamespaceCurrent}::${module}::configDataChanged 1
    }
  }
  return
}

##
## Handle config data for a list of modules
##
## Args:    action {load|save|checkdefs}, module list, force {0,1},
##           verbose {-1,0,1}
## Returns: nothing
##
proc moduleConfig {action modules {force 0} {verbose 0}} {
  if {![string compare * $modules]} then {
    set modules [listModules 1]
  }
  switch -exact -- $action {
    save {
      foreach module $modules {
        if {([moduleExists $module]) && ([moduleLoaded $module])} then {
          moduleConfigSave $module $force $verbose
        }
      }
    }
    load {
      foreach module $modules {
        if {([moduleExists $module]) && ([moduleLoaded $module])} then {
          moduleConfigLoad $module $force $verbose
        }
      }
    }
    checkdefs {
      foreach module $modules {
        if {([moduleExists $module]) && ([moduleLoaded $module])} then {
          moduleConfigCheckdefs $module $force $verbose
        }
      }
    }
  }
  return
}

##
## Save data for the given module
##
## Args:    module, force {0,1}, verbose {-1,0,1}
## Returns: nothing
##
proc moduleDataSave {module {force 0} {verbose 0}} {
  variable NamespaceCurrent
  variable configData

  if {([info exists ${NamespaceCurrent}::${module}::dataOptions]) && \
      ([createDir $configData(datapath)])} then {
    foreach {name data} [array get ${NamespaceCurrent}::${module}::dataOptions] {
      foreach {type file desc} $data {break}
      if {([info exists type]) && ([info exists file]) && \
          ([info exists desc])} then {
        set Changed ${NamespaceCurrent}::${module}::${name}Changed
        if {($force) || ((![info exists $Changed]) || ([set $Changed]))} then {
          if {[${type}Save ${NamespaceCurrent}::${module}::$name [file join $configData(datapath) $file]]} then {
            if {$verbose >= 1} then {
              wpLog o * [mc "Writing %s data file..." $desc]
            }
            set $Changed 0
          } elseif {$verbose >= 0} then {
            wpLog o * [mc "Error writing %s data file!" $desc]
          }
        }
      }
    }
  }
  return
}

##
## Load data for the given module
##
## Args:    module, force {0,1}, verbose {-1,0,1}
## Returns: nothing
##
proc moduleDataLoad {module {force 0} {verbose 0}} {
  variable NamespaceCurrent
  variable configData

  if {([info exists ${NamespaceCurrent}::${module}::dataOptions]) && \
      ([createDir $configData(datapath)])} then {
    foreach {name data} [array get ${NamespaceCurrent}::${module}::dataOptions] {
      foreach {type file desc} $data {break}
      if {([info exists type]) && ([info exists file]) && \
          ([info exists desc])} then {
        set Changed ${NamespaceCurrent}::${module}::${name}Changed
        if {($force) || ((![info exists $Changed]) || ([set $Changed]))} then {
          if {[${type}Load ${NamespaceCurrent}::${module}::$name [file join $configData(datapath) $file]]} then {
            if {$verbose >= 1} then {
              wpLog o * [mc "Reloading %s data file..." $desc]
            }
            set $Changed 0
          } elseif {$verbose >= 0} then {
            wpLog o * [mc "Error reloading %s data file!" $desc]
          }
        }
      }
    }
  }
  return
}

##
## Backup data for the given module
##
## Args:    module, force {0,1}, verbose {-1,0,1}
## Returns: nothing
##
proc moduleDataBackup {module {force 0} {verbose 0}} {
  variable NamespaceCurrent
  variable configData

  if {([info exists ${NamespaceCurrent}::${module}::dataOptions]) && \
      ([createDir $configData(datapath)])} then {
    foreach {name data} [array get ${NamespaceCurrent}::${module}::dataOptions] {
      foreach {type file desc} $data {break}
      if {([info exists type]) && ([info exists file]) && \
          ([info exists desc])} then {
        if {[set result [backupFile [file join $configData(datapath) $file] $verbose]]} then {
          if {($result >= 1) && ($verbose >= 1)} then {
            wpLog o * [mc "Backing up %s data file..." $desc]
          }
        } elseif {$verbose >= 0} then {
          wpLog o * [mc "Error backing up %s data file!" $desc]
        }
      }
    }
  }
  return
}

##
## Handle data for a list of modules
##
## Args:    action {load|save|backup}, module list, force {0,1},
##           verbose {-1,0,1}
## Returns: nothing
##
proc moduleData {action modules {force 0} {verbose 0}} {
  if {![string compare * $modules]} then {
    set modules [listModules 1]
  }
  switch -exact -- $action {
    save {
      foreach module $modules {
        if {([moduleExists $module]) && ([moduleLoaded $module])} then {
          moduleDataSave $module $force $verbose
        }
      }
    }
    load {
      foreach module $modules {
        if {([moduleExists $module]) && ([moduleLoaded $module])} then {
          moduleDataLoad $module $force $verbose
        }
      }
    }
    backup {
      foreach module $modules {
        if {([moduleExists $module]) && ([moduleLoaded $module])} then {
          moduleDataBackup $module $force $verbose
        }
      }
    }
  }
  return
}

##
## Process commands to add to the command table for the given module
##
## Args:    array, module, origin, commands, {verbose 0}
## Returns: nothing
##
proc buildModuleCommandTable {arrayName module origin commands {verbose 0}} {
  upvar 1 $arrayName array

  foreach command $commands {
    set arrayIndexName $module,$command

    if {![info exists array($arrayIndexName)]} then {
      if {$verbose >= 2} then {
        wpLog o * [mc "Adding command: `%s' from `%s'" $command $origin]
      }
      set array($arrayIndexName) ${origin}::$command
    } elseif {$verbose >= 0} then {
      wpLog o * [mc "Warning: ignoring duplicate command `%s', conflicts with `%s'" $arrayIndexName $array($arrayIndexName)]
    }
  }
  return
}

##
## Builds command matching table from module database
##
## Args:    none
## Returns: nothing
##
proc buildCommandTable {{verbose 0}} {
  variable NamespaceCurrent
  variable ModuleName
  variable ExportedCommands
  variable commandTable

  # Add commands for ::wp
  buildModuleCommandTable newCommandTable $ModuleName $NamespaceCurrent $ExportedCommands $verbose

  # Add commands for ::wp::[module]
  foreach module [listModules] {
    if {[string compare "" [set provides [getModuleDatabaseData $module provides]]]} then {
      buildModuleCommandTable newCommandTable $module "${NamespaceCurrent}::$module" $provides $verbose
    }
  }

  # Get rid of the old command table if it exists
  if {[info exists commandTable]} then {
    unset commandTable
  }

  # Copy new data into the command table
  array set commandTable [array get newCommandTable]
  return
}

##
## Return full namespace path for the given command
##
## Args:    command
## Returns: full namespace path for the given command if it exists
##          nothing otherwise
##
proc whichCommand {command} {
  variable NamespaceCurrent
  variable commandTable

  # Search for 'module::command'
  if {([regexp -- {(.*)::(.*)} $command "" tmpModule tmpCommand]) &&
      ([info exists $tmpModule]) &&
      ([info exists $tmpCommand]) &&
      ([info exits commandTable($tmpModule,$tmpCommand)])} then {
    return $commandTable($tmpModule,$tmpCommand)
  # Search for 'module,command'
  } elseif {([regexp -- {,} $command]) &&
            ([info exists commandTable($command)])} then {
    return $commandTable($command)
  } else {
    set names [array names commandTable *,$command]
    if {[llength $names] == 1} then {
      return $commandTable($names)
    } else {
# FIXME: multiple matches ...
    }
  }
  return
}

##
## Return module name that provides the given command
##
## Args:    command
## Returns: name of module that provides the given command
##          nothing otherwise
##
proc whichCommandModule {command} {
  variable NamespaceCurrent
  variable ModuleName
  variable commandTable

  # Search for 'module::command'
  if {([regexp -- {(.*)::(.*)} $command "" tmpModule tmpCommand]) &&
      ([info exists $tmpModule]) &&
      ([info exists $tmpCommand]) &&
      ([info exits commandTable($tmpModule,$tmpCommand)])} then {
    set lookup $commandTable($tmpModule,$tmpCommand)
  } else {
    set names [array names commandTable *,$command]
    if {[llength $names] == 1} then {
      set lookup $commandTable($names)
    } else {
# FIXME: multiple matches ...
    }
  }
  if {[info exists lookup]} then {
    if {![string compare ${NamespaceCurrent}::$command $lookup]} then {
      return $ModuleName
    }
    return [namespace tail [namespace qualifiers $lookup]]
  }
  return
}

##
## Return full namespace path for the given module
##
## Args:    module
## Returns: full namespace path for the given module if it's loaded
##          nothing otherwise
##
proc whichModule {module} {
  variable NamespaceCurrent
  variable ModuleName

  if {![string compare $ModuleName $module]} then {
    return $NamespaceCurrent
  } elseif {[moduleLoaded $module]} then {
    return ${NamespaceCurrent}::$module
  }
  return
}

##
## Check if the given [module] config option exists
##
## Args:    module, option
## Returns: 1 if the given module config option exists
##
proc configExists {module {option ""}} {
  variable NamespaceCurrent
  variable configData

  if {[string compare "" $module]} then {
    set where "${NamespaceCurrent}::${module}::"
  } else {
    set where ""
  }
  if {[string compare "" $option]} then {
    if {[info exists ${where}configData($option)]} then {
      return 1
    }
  } elseif {[info exists ${where}configData]} then {
    return 1
  }
  return 0
}

##
## Compare the given version to eggdrop's version
##
## Args:    version
## Returns: 0 if eggdrop's version is older then the given version
##          1 if eggdrop's version matches the given version
##          2 if eggdrop's version is newer then the given version
##          -1 if the given version is invalid
##
proc compareVersion {version} {
  global numversion

  if {([string compare "" $version]) && \
      ([info exists numversion])} then {
    if {[regexp -- {[.]} $version]} then {
      regsub -all -- {[.]} $version 0 version
      set version ${version}00
    }
    if {[regexp -- {[^0-9]} $version]} then {
      return -1
    } elseif {$numversion == $version} then {
      return 1
    } elseif {$numversion > $version} then {
      return 2
    }
  }
  return 0
}

##
## Log module information
##
## Args:    level, channel, text
## Returns: nothing
##
proc wpLog {level channel text} {
  set module [namespace tail [uplevel 1 [list namespace current]]]
  if {(![string compare "" $module]) || \
      (![string compare wp $module])} then {
    putloglev $level $channel "Wolfpack: $text"
  } else {
    putloglev $level $channel "Wolfpack: \[$module\] $text"
  }
  return
}

##
## Log Tcl call trace
##
## Args:    none
## Returns: nothing
##
proc callTrace {{loglevel d}} {
  wpLog $loglevel * "Tcl Call Trace"
  for {set x [expr [info level] - 1]} {$x > 0} {incr x -1} {
    wpLog $loglevel * "$x: [info level $x]"
  }
  return
}

##
## Build list of children namespace for the given namespace
##
## Args:    namespace
## Returns: list of child namespaces
##
proc listNamespaceChildren {namespace} {
  set children [namespace children $namespace]
  foreach child $children {
    append children " [listNamespaceChildren $child]"
  }
  return $children
}

##
## Add namespace qualifier onto the given command name
##
## Args:    command
## Returns: fully qualified command if found
##
proc qualifyCommandName {command} {
  set qualified [uplevel [expr [info level] - 1] [list if 1 "namespace which $command"]]
  if {[string compare "" $qualified]} then {
    return $qualified
  }
  return $command
}

##
## Return seconds elapsed if 'microseconds' >= 0
##
## Args:    microseconds
## Returns: ' (d.fff seconds elapsed)'
##
proc secondsElapsed {microseconds} {
  if {$microseconds >= 0} then {
    return [mc " (%.3f seconds elapsed)" [expr $microseconds / 1000000.0]]
  }
  return
}

##
## Load the given list of modules
##
## Args:    modules, {verbose 0}
## Returns: nothing
##
proc loadModules {modules {verbose 0}} {
  foreach module $modules {
    if {[catch {moduleLoad $module $verbose} result]} then {
      wpLog o * [mc "Error: unable to load module `%s': %s" $module $result]
    }
  }
  return
}

##
## Temporary proc to emulate msgcat
##
## Args:    string
## Returns: string
##
proc mc {src args} {
  if {[llength $args]} then {
    return [uplevel 1 [linsert $args 0 format $src]]
  }
  return $src
}

# FIXME: eliminate 'bind' and 'unbind' with '[namespace code ...]' ?

##
## Wrapper for Eggdrop's bind command that adds
## a fully qualified namespace to the given proc
##
## Args:    type, flags, mask, {proc ""}
## Returns: result of global 'bind' command
##
proc bind {type flags mask {proc ""}} {
  if {[string compare "" $proc]} then {
    return [::bind $type $flags $mask [qualifyCommandName $proc]]
  }
  return [::bind $type $flags $mask]
}

##
## Wrapper for Eggdrop's unbind command that adds
## a fully qualified namespace to the given proc
##
## Args:    type, flags, mask, proc
## Returns: result of global 'unbind' command
##
proc unbind {type flags mask proc} {
  return [::unbind $type $flags $mask [qualifyCommandName $proc]]
}

##
## Evaluate command line arguments
##
## Args:    none
## Returns: nothing
##
proc EvalArgs {argc argv argv0} {
  variable ModuleVersion
  variable optionData

  for {set index 0} {$index < $argc} {incr index} {
    set option [lindex $argv $index]
    set nextoption [lindex $argv [expr $index + 1]]

    switch -regexp -- $option {
      {^--$} {
        break
      }
      {^--cfgfile$} {
        if {([string compare "" $nextoption]) && \
            (![regexp -- {-} $nextoption])} then {
          set optionData(cfgfile) $nextoption
          incr index
        } else {
          listAppendIf noparms "--cfgfile"
        }
      }
      {^--config$} {
        set optionData(config) 1
      }
      {^--update$} {
        set optionData(update) 1
      }
      {^--noupdate$} {
        set optionData(noupdate) 1
      }
      {^--rebuild$} {
        set optionData(rebuild) 1
      }
      {^--time$} {
        set optionData(time) 1
      }
      {^--include$} {
        if {([string compare "" $nextoption]) && \
            (![regexp -- {-} $nextoption])} then {
          listAppendIf optionData(include) $nextoption
          incr index
        } else {
          listAppendIf noparms "--include"
        }
      }
      {^--exclude$} {
        if {([string compare "" $nextoption]) && \
            (![regexp -- {-} $nextoption])} then {
          listAppendIf optionData(exclude) $nextoption
          incr index
        } else {
          listAppendIf noparms "--exclude"
        }
      }
      {^--module$} {
        if {([string compare "" $nextoption]) && \
            (![regexp -- {-} $nextoption])} then {
          listAppendIf optionData(module) $nextoption
          incr index
        } else {
          listAppendIf noparms "--module"
        }
      }
      {^--verbose$} {
        incr optionData(verbose)
      }
      {^--quiet$} {
        incr optionData(quiet) -1
      }
      {^--debug$} {
        set optionData(debug) 1
      }
      {^--help$} {
        ShowUsage $argv0
        exit
      }
      {^--version$} {
        puts [mc "%s version %s" [file tail $argv0] $ModuleVersion]
        exit
      }
      {^--license$} {
        ShowLicense
        exit
      }
      {^-[^-]*$} {
        set suboptions [split $option ""]
        set sublength [llength [split $suboptions]]
        for {set subindex 0} {$subindex < $sublength} {incr subindex} {
          set suboption [lindex $suboptions $subindex]
          switch -exact -- $suboption {
            - {
              continue
            }
            f {
              # Next arg in argv should be a filename: '-f filename.conf',
              # so break out of the suboption loop after this option
              if {([string compare "" $nextoption]) && \
                  (![regexp -- {-} $nextoption])} then {
                set optionData(cfgfile) $nextoption
                incr index
                break
              } else {
                listAppendIf noparms "-f"
              }
            }
            c {
              set optionData(config) 1
            }
            u {
              set optionData(update) 1
            }
            n {
              set optionData(noupdate) 1
            }
            r {
              set optionData(rebuild) 1
            }
            t {
              set optionData(time) 1
            }
            i {
              # Next arg in argv should be a module: '-i module',
              # so break out of the suboption loop after this option
              if {([string compare "" $nextoption]) && \
                  (![regexp -- {-} $nextoption])} then {
                listAppendIf optionData(include) $nextoption
                incr index
                break
              } else {
                listAppendIf noparms "-i"
              }
            }
            x {
              # Next arg in argv should be a module: '-x module',
              # so break out of the suboption loop after this option
              if {([string compare "" $nextoption]) && \
                  (![regexp -- {-} $nextoption])} then {
                listAppendIf optionData(exclude) $nextoption
                incr index
                break
              } else {
                listAppendIf noparms "-x"
              }
            }
            m {
              # Next arg in argv should be a module: '-m module',
              # so break out of the suboption loop after this option
              if {([string compare "" $nextoption]) && \
                  (![regexp -- {-} $nextoption])} then {
                listAppendIf optionData(module) $nextoption
                incr index
                break
              } else {
                listAppendIf noparms "-m"
              }
            }
            v {
              incr optionData(verbose)
            }
            q {
              incr optionData(quiet) -1
            }
            d {
              set optionData(debug) 1
            }
            H {
              ShowUsage $argv0
              exit
            }
            V {
              puts [mc "%s version %s" [file tail $argv0] $ModuleVersion]
              exit
            }
            default {
              listAppendIf invalidopt "-$suboption"
            }
          }
        }
      }
      default {
        listAppendIf invalidopt $option
      }
    }
  }

  # Complain about invalid command line arguments
  if {[info exists invalidopt]} then {
    foreach option $invalidopt {
      puts stderr [mc "%s: unrecognized option `%s'" [file tail $argv0] $option]
    }
    set exit 1
  }

  # Complain about missing parameters
  if {[info exists noparms]} then {
    foreach option $noparms {
      puts stderr [mc "%s: option requires a parameter `%s'" [file tail $argv0] $option]
    }
    set exit 1
  }

  if {[info exists exit]} then {
    exit 1
  }
}

##
## Show usage information
##
## Args:    argv0
## Returns: nothing
##
# FIXME: is this the best way to support msgcat?
proc ShowUsage {argv0} {
  puts [mc "Usage: %s <options>" [file tail $argv0]]
  puts [mc "  Valid options:"]
  puts [mc "  -f, --cfgfile <file>  use configuration file `file'"]
  puts [mc "  -c, --config          start interactive configuration"]
  puts [mc "  -u, --update          update module database"]
  puts [mc "  -n, --noupdate        don't update module database even if its outdated"]
  puts [mc "  -r, --rebuild         force complete rebuild of module database"]
  puts [mc "  -t, --time            time compare/update/rebuild of module database"]
  puts [mc "  -i, --include <file>  include module `file' when updating database"]
  puts [mc "  -x, --exclude <file>  exclude module `file' when updating database"]
  puts [mc "  -m, --module <file>   only update database for module `file'"]
  puts [mc "  -v, --verbose         use more than once for more verbose operation"]
  puts [mc "  -q, --quiet           use more than once for quieter operation"]
  puts [mc "  -d, --debug           start debug mode with tclsh"]
  puts [mc "  -H, --help            show this help"]
  puts [mc "  -V, --version         show version information"]
  puts [mc "      --license         show copyright and license information"]
  return
}

##
## Show copyright and license information
##
## Args:    none
## Returns: nothing
##
# FIXME: should msgcat be used for license? [NOT copyright]
proc ShowLicense {} {
  variable Copyright
  variable License

  puts "\n    $Copyright\n"
  foreach line [split $License \n] {
    puts "    $line"
  }
  puts ""
  return
}

##
## Show copyright and license information
##
## Args:    {mode eggdrop}
## Returns: nothing
##
# FIXME: should msgcat be used for license? [NOT copyright]
proc ShowInitNotice {{mode eggdrop}} {
  variable Copyright

  wpLog o * "$Copyright"
  wpLog o * "Wolfpack comes with ABSOLUTELY NO WARRANTY."
  wpLog o * "This is free software, and you are welcome to redistribute it"

  switch -exact -- $mode {
    interactive {
      wpLog o * "under certain conditions. See the file COPYING or type"
      wpLog o * "`license' for details."
    }
    tclsh {
      wpLog o * "under certain conditions. See the file COPYING or type"
      wpLog o * "`./wolfpack.tcl --license' for details."
    }
    eggdrop {
      wpLog o * "under certain conditions. See the file COPYING or type"
      wpLog o * "'.wpconf license' from the partyline for details."
    }
  }
  return
}

##
## Build array of module option data
##
## Args:    verbose
## Returns: nothing
##
proc ParseModuleOptionData {{verbose 0}} {
  variable configData
  variable optionData
  variable modulePath
  variable moduleOptionData

  # Add valid files to the include, exclude, and module lists
  foreach option "include exclude module" {
    if {[string compare "" $optionData($option)]} then {
      if {[string compare "" $moduleOptionData($option)]} then {
        set moduleOptionData($option) ""
      }

      foreach file $optionData($option) {
        foreach dir $modulePath {
          set fullfile [file join $dir $file]
          if {([file exists $fullfile]) && ([file readable $fullfile])} then {
            lappend moduleOptionData($option) "$file $fullfile"
            break
          }
        }
      }
    }
  }

  # Build list of excluded files
  if {[string compare "" $moduleOptionData(exclude)]} then {
    set moduleOptionData(excludedfiles) ""

    if {[string compare "" $moduleOptionData(include)]} then {
      foreach exclude $moduleOptionData(exclude) {
        foreach {excludeFile excludeFullFile} $exclude {break}
        set found 0
        foreach include $moduleOptionData(include) {
          foreach {includeFile includeFullFile} $include {break}
          if {![string compare $excludeFullFile $includeFullFile]} then {
            set found 1
            break
          }
        }
        if {!$found} then {
          lappend moduleOptionData(excludedfiles) $excludeFullFile
        }
      }
    } else {
      foreach exclude $moduleOptionData(exclude) {
        foreach {file fullfile} $exclude {break}
        lappend moduleOptionData(excludedfiles) $fullfile
      }
    }
  }

  # Build list of module files
  if {[string compare "" $moduleOptionData(module)]} then {
    set moduleOptionData(modulefiles) ""

    foreach module $moduleOptionData(module) {
      foreach {file fullfile} $module {break}
      lappend moduleOptionData(modulefiles) $fullfile
    }
  }
  return
}


###############################################################################
##
## Begin merge of config.tcl:
##


##
## Process a config command
##
## Args:    idx, text, {log 0}
## Returns: result of a config command
##
proc configEvalCommand {idx text {log 0}} {
  set text [split [string trimright $text]]
  set command [string tolower [string trimleft [lindex $text 0] .]]
  set arg [lrange $text 1 end]

# FIXME: global config code not fully implemented yet: set, info
#        should be able to be flag certain settings read-only
#        when loaded in eggdrop mode (paths, etc)

# FIXME: add commands and support:
#        reload -- reload module config file
#        status -- summary of modules, loaded, options, etc
#        update -- update modules
#        refresh -- refresh database when modules are added/removed/updated

  # Command name to proc mapping for command expansion
  array set CommandTable {
    set configSet
    list configList
    info configInfo
    default configDefault
    modules configModules
    enable configEnable
    disable configDisable
    save configSave
    help configHelp
    license configLicense
    quit configQuit
  }

  set result 0
  switch -exact -- [expandText $command \
                    [lsort [array names CommandTable]] 0 expandedCommand] {
    -1 { ;# Ambiguous command
      putdcc $idx [mc "Ambiguous command."]
    }
    0 {	;# Unknown command
      putdcc $idx [mc "What?  You need '%s'" help]
    }
    1 -
    2 {	;# Exact or partial match
      # All commands should return 0 unless they exit the control loop or fileevent
      set result [$CommandTable($expandedCommand) $idx $arg]
      if {(!$result) && ($log)} then {
        # FIXME: add code to log wpconf changes to other SessionData idx
        # NOTE: we only log this for Eggdrop partyline wpconf
        wpLog c * "#[idx2hand $idx]# $expandedCommand $arg"
      }
    }
  }
  return $result
}


##
## Interactive configuration: '.wpconf' from partyline
##
## Args:    handle, idx, arg
## Returns: 1
##
## Important variables:
##   SessionData(idx) "module namespace"
##
proc dcc:wpconf {hand idx arg} {
  variable NamespaceCurrent
  variable SessionData

  # Check to make sure user is a permanent owner
# FIXME: 'dccCheckOwner' is in dcctools.tcl
#  if {![dccCheckOwner $hand $idx]} then {
#    return 0
#  }

# FIXME: config.tcl had a way to store the default module for a user
  set module ""
  set namespace "::${NamespaceCurrent}::"

#  # Sanity checks
#  if {![moduleExists $module]} then {
#    putdcc $idx [mc "Warning: Stored default module %s does not exist." $module]
#    putdcc $idx [mc "Default set to global settings."]
#    set module ""
#  } elseif {![configExists $module]} then {
#    putdcc $idx [mc "Warning: Stored default module %s has no config settings." $module]
#    putdcc $idx [mc "Default set to global settings."]
#    set module ""
#  }
#
#  # Module will be "" for global settings
#  if {![string compare "" $module]} then {
#    set namespace "::${NamespaceCurrent}::"
#  } else {
#    set namespace "::${NamespaceCurrent}::${module}::"
#  }

  set SessionData($idx) [list $module $namespace]

# FIXME: improve noninteractive code;
#        eliminate 'SessionData($idx)' for noninteractive code ?
#        require .set to use 'module' instead of requiring .mod <module> ?

  # Noninteractive mode: '.wpconf <command> [args]'
  if {[string compare "" $arg]} then {
    configEvalCommand $idx $arg
  } else {
    putdcc $idx [mc "Entering %s configuration system..." wolfpack]
    # putdcc can't handle ""
    putdcc $idx " "
    putdcc $idx [mc "Type '%s' for help." help]
    putdcc $idx "${module}>"

    control $idx [namespace code control:config]
  }
  return 1
}


##
## Configure module settings: control
##
## Args:    idx, text
## Returns: 0 during command loop,
##          1 to exit back to partyline
##
proc control:config {idx text} {
  variable SessionData

  set result [configEvalCommand $idx $text 1]
  if {!$result} then {
    putdcc $idx "[lindex $SessionData($idx) 0]>"
  }
  return $result
}


##
## Interactive configuration: './wolfpack.tcl -c'
##
## Args:    none
## Returns: nothing
##
proc WpConfig {} {
  variable NamespaceCurrent
  variable SessionData

# FIXME: config.tcl had a way to store the default module for a user

  set module ""
  set namespace "::${NamespaceCurrent}::"

  set SessionData(-1) [list $module $namespace]

  fileevent stdin readable ${NamespaceCurrent}::WpConfigReadStdin
  puts [mc "Entering %s configuration system..." wolfpack]
  puts [mc "Type '%s' for help." help]
  puts -nonewline "${module}> "
  flush stdout
  vwait forever
}


##
## Read stdin and process commands
##
## Args:    none
## Returns: nothing
##
proc WpConfigReadStdin {} {
  variable SessionData
  variable moduleDatabaseDataChanged
  variable configData
  variable configDataChanged
  variable configFile

# FIXME: add readline-like support ?

  set result 0
  set stdin [gets stdin]

  if {[string compare "" $stdin]} then {
    # All commands should return 0 unless they cause config to exit
    set result [configEvalCommand -1 $stdin]
  }

  if {(!$result) && (![eof stdin])} then {
    puts -nonewline "[lindex $SessionData(-1) 0]> "
    flush stdout
  } else {

    # Save module database
    if {$moduleDatabaseDataChanged} then {
      saveModuleDatabase
    }

    # Save configuration data
    if {$configDataChanged} then {
      arraySave configData $configFile 0 [mc "configuration file "]
    }

    # NOTE: not saving module data here because it hasn't been processed
    moduleConfig save *

    puts ""
    flush stdout
    exit
  }
  return
}


##
## Configure module settings: set
##
## Args:    idx, arg
## Returns: 0
##
proc configSet {idx arg} {
  variable NamespaceCurrent
  variable SessionData

# FIXME: should we remove redundant whitespace in 'tmp'
#        so things like 'set foo   bar' will work?
  set tmp [split [lindex $arg 0] :]
  set tmpModule [lindex $tmp 0]
  set tmpOption [lindex $tmp 1]
  set tmpWhat [join [lrange $arg 1 end]]

  foreach {defaultModule defaultNamespace} $SessionData($idx) {break}

  # 'set' (display all settings for default module)
  if {![string compare "" $tmpModule]} then {
    set module $defaultModule
    set option ""
    set what ""
    set namespace $defaultNamespace

  # 'set module:option what'
  # 'set module:option' (display setting for given module:option)
  } elseif {[configExists $tmpModule $tmpOption]} then {
    set module $tmpModule
    set option $tmpOption
    set what $tmpWhat

  # NOTE: If a config option for the default module and a module
  #       from the module list have the same name, the config
  #       option for the default module takes presidence.

  # 'set option what'
  # 'set option' (display setting for given option)
  } elseif {[configExists $defaultModule $tmpModule]} then {
    set module $defaultModule
    set option $tmpModule
    set what $tmpWhat
    set namespace $defaultNamespace

  } elseif {[moduleExists $tmpModule]} then {
    if {![moduleLoaded $tmpModule]} then {
      putdcc $idx [mc "Error: module: %s is not loaded" $tmpModule]
    } elseif {![configExists $tmpModule]} then {
      putdcc $idx [mc "Error: no config options for module: %s" $tmpModule]
    } elseif {[string compare "" $tmpOption]} then {
      if {[string compare $defaultModule $tmpModule]} then {
        putdcc $idx [mc "Error: unknown option for module: %s: %s" $tmpModule $tmpOption]
      } else {
        putdcc $idx [mc "Error: unknown module option: %s" $tmpOption]
      }

    # 'set module' (display all settings for given module)
    } else {
      set module $tmpModule
      set option ""
      set what ""
    }
  } else {
    putdcc $idx [mc "Error: unknown module or option: %s" $tmpModule]
  }

  if {[info exists module]} then {
    if {![info exists namespace]} then {
      set namespace "${NamespaceCurrent}::${module}::"
    }

    if {![string compare "" $option]} then {

      # FIXME: make formatting width dynamic?
      putdcc $idx [mc "Module settings for module: %s" $module]
      putdcc $idx [mc "Option       Value"]
      foreach option [lsort [array names ${namespace}configData]] {
        putdcc $idx [format "%-12s %s" $option [set ${namespace}configData($option)]]
      }
    } elseif {![string compare "" $what]} then {
      putdcc $idx [mc "Currently: %s" [set ${namespace}configData($option)]]
    } else {

      # FIXME: finish checkData code and check againt configDefaults checks
      #foreach {type default errorText exampleText helpText} [set ${namespace}configDefaults($option)] {break}

      if {[string compare [join $what] [set ${namespace}configData($option)]]} then {
        set ${namespace}configData($option) [join $what]
        set ${namespace}configDataChanged 1

        if {![string compare $defaultModule $module]} then {
          putdcc $idx [mc "Set %s to: %s" $option $what]
        } else {
          putdcc $idx [mc "Set %s:%s to: %s" $module $option $what]
        }
      } else {
        putdcc $idx [mc "Error: %s was already set to: %s" $option $what]
      }
    }
  }
  return 0
}


##
## Configure module settings: list
##
## Args:    idx, arg
## Returns: 0
##
proc configList {idx arg} {
  variable NamespaceCurrent
  variable SessionData

  set tmpModule [lindex $arg 0]

  if {![string compare "" $tmpModule]} then {
    foreach {module namespace} $SessionData($idx) {break}
  } elseif {![moduleExists $tmpModule]} then {
    putdcc $idx [mc "Error: unknown module: %s" $tmpModule]
  } elseif {![moduleLoaded $tmpModule]} then {
    putdcc $idx [mc "Error: module: %s is not loaded" $tmpModule]
  } elseif {![configExists $tmpModule]} then {
    putdcc $idx [mc "Error: no config options for module: %s" $tmpModule]
  } else {
    set module $tmpModule
  }

  if {[info exists module]} then {

    # Module will be "" for global settings
    if {![string compare "" $module]} then {
      set namespace "::${NamespaceCurrent}::"
    } else {
      set namespace "::${NamespaceCurrent}::${module}::"
    }

    # FIXME: make formatting width dynamic?
    putdcc $idx [mc "Module option info for module: %s" $module]
    putdcc $idx [mc "Option       Info"]
    foreach option [lsort [array names ${namespace}configData]] {
      # FIXME: make use of some of the other fields
      foreach {type default errorText exampleText helpText} [set ${namespace}configDefaults($option)] {break}
      if {[string compare "" $helpText]} then {
        putdcc $idx [format "%-12s %s" $option $helpText]
      } else {
        putdcc $idx [format "%-12s %s" $option "(no info)"]
      }
    }
  }
  return 0
}


##
## Configure module settings: info
##
## Args:    idx, arg
## Returns: 0
##
proc configInfo {idx arg} {
  variable SessionData

  set tmpModule [lindex $arg 0]

  if {![string compare "" $tmpModule]} then {
    foreach {module namespace} $SessionData($idx) {break}
  } elseif {![moduleExists $tmpModule]} then {
    putdcc $idx [mc "Error: unknown module: %s" $tmpModule]
  } else {
    set module $tmpModule
  }

  if {[info exists module]} then {
    if {![string compare "" $module]} then {
      putdcc $idx [mc "Global info not implemented."]
    } else {
      putdcc $idx [mc "Info for module: %s" $module]
      putdcc $idx " "
      putdcc $idx [mc "Filename: %s" [getModuleDatabaseData $module file]]
      putdcc $idx [mc "MD5: %s" [getModuleDatabaseData $module md5sum]]
      putdcc $idx [mc "Version: %s" [getModuleDatabaseData $module version]]
      if {[getModuleDatabaseData $module load]} then {
        set enabled [mc "yes"]
      } else {
        set enabled [mc "no"]
      }
      putdcc $idx [mc "Enabled: %s" $enabled]
      if {[string compare "" [set config [getModuleDatabaseData $module config]]]} then {
        putdcc $idx [mc "Config file: %s" $config]
      }
      if {[string compare "" [set author [getModuleDatabaseData $module author]]]} then {
        putdcc $idx [mc "Author: %s" $author]
      }
      if {[string compare "" [set license [getModuleDatabaseData $module license]]]} then {
        putdcc $idx [mc "License: %s" $license]
      }
      if {[string compare "" [set provides [getModuleDatabaseData $module provides]]]} then {
        putdcc $idx [mc "Provided commands:"]
        foreach line [splitStringList $provides 65 " " " "] {
          putdcc $idx "  $line"
        }
      }
      if {[string compare "" [set requires [getModuleDatabaseData $module requires]]]} then {
        putdcc $idx [mc "Required commands:"]
        foreach line [splitStringList $requires 65 " " " "] {
          putdcc $idx "  $line"
        }
      }
      putdcc $idx [mc "Description:"]
      foreach line [splitStringList [getModuleDatabaseData $module description] 65 " " " "] {
        putdcc $idx "  $line"
      }
    }
  }
  return 0
}


##
## Configure module settings: default
##
## Args:    idx, arg
## Returns: 0
##
proc configDefault {idx arg} {
  variable NamespaceCurrent
  variable SessionData
  variable configData

  set tmpModule [lindex $arg 0]

  if {![string compare "" $tmpModule]} then {
    # Global settings (::wp::configData)
    set module ""
  } elseif {![moduleExists $tmpModule]} then {
    putdcc $idx [mc "Error: unknown module: %s" $tmpModule]
  } elseif {![moduleLoaded $tmpModule]} then {
    putdcc $idx [mc "Error: module: %s is not loaded" $tmpModule]
  } elseif {![configExists $tmpModule]} then {
    putdcc $idx [mc "Error: no config options for module: %s" $tmpModule]
  } else {
    set module $tmpModule
  }

  if {[info exists module]} then {
    # Module will be "" for global settings
    if {![string compare "" $module]} then {
      set SessionData($idx) [list $module "::${NamespaceCurrent}::"]
      putdcc $idx [mc "Default set to: global settings"]
    } else {
      set SessionData($idx) [list $module "::${NamespaceCurrent}::${module}::"]
      putdcc $idx [mc "Default module now: %s" $module]
    }
  }
  return 0
}


##
## Configure module settings: modules
##
## Args:    idx, arg
## Returns: 0
##
proc configModules {idx arg} {

  set what [lindex $arg 0]
  if {![string compare "" $what]} then {
    set what "all"
  }

  switch -exact -- $what {
    * -
    all {
      set modules [listModules]
      putdcc $idx [mc "Available modules:"]
    }
    enabled {
      set modules ""
      foreach module [listModules] {
        if {[getModuleDatabaseData $module load]} {
          lappend modules $module
        }
      }
      if {[string compare "" $modules]} then {
        putdcc $idx [mc "Enabled modules:"]
      } else {
        putdcc $idx [mc "No modules enabled"]
      }
    }
    disabled {
      set modules ""
      foreach module [listModules] {
        if {![getModuleDatabaseData $module load]} {
          lappend modules $module
        }
      }
      if {[string compare "" $modules]} then {
        putdcc $idx [mc "Disabled modules:"]
      } else {
        putdcc $idx [mc "No modules disabled"]
      }
    }
    default {
      putdcc $idx [mc "Error: 'option' must be one of: %s, %s, %s" all enabled disabled]
    }
  }

  if {[info exists modules]} then {
    set fmtlen [listMaxElementLength $modules]
    if {$fmtlen < 6} then {
      set fmtlen 6 ;# 'Module'
    }

# FIXME: 'mc' text isn't done right
    putdcc $idx "[format "%-${fmtlen}s  %s" [mc Module] [mc Version]]"
    foreach module $modules {
      set version [getModuleDatabaseData $module version]
      putdcc $idx "[format "%-${fmtlen}s  %s" $module $version]"
    }
  }
  return 0
}


##
## Configure module settings: enable
##
## Args:    idx, arg
## Returns: 0
##
proc configEnable {idx arg} {
  set module [lindex $arg 0]

  if {![string compare "" $module]} then {
    putdcc $idx [mc "Usage: %s <module>" enable]
  } elseif {![moduleExists $module]} then {
    putdcc $idx [mc "Error: unknown module: %s" $module]
  } elseif {[moduleLoaded $module]} then {
    putdcc $idx [mc "Error: %s is already enabled" $module]
  } else {
    putdcc $idx [mc "Loading module: %s" $module]
    if {[catch {moduleLoad $module} result]} then {
      putdcc $idx [mc "Error loading %s: %s" $module $result]

      if {$idx != -1} then {
        wpLog o * [mc "Error loading %s: %s" $module $result]
      }
    }
  }
  return 0
}


##
## Configure module settings: disable
##
## Args:    idx, arg
## Returns: 0
##
proc configDisable {idx arg} {
  variable NamespaceCurrent
  variable SessionData
  variable configData

  set module [lindex $arg 0]

  if {![string compare "" $module]} then {
    putdcc $idx [mc "Usage: %s <module>" disable]
  } elseif {![moduleExists $module]} then {
    putdcc $idx [mc "Error: unknown module: %s" $module]
  } elseif {![moduleLoaded $module]} then {
    putdcc $idx [mc "Error: %s is already disabled" $module]
  } else {
    putdcc $idx [mc "Disabling module: %s" $module]
    if {[catch {moduleUnload $module} result]} then {
      putdcc $idx [mc "Error unloading %s: %s" $module $result]

      if {$idx != -1} then {
        wpLog o * [mc "Error unloading %s: %s" $module $result]
      }
    }

    # Set default module to something sane if we just disabled/unloaded it

    foreach {sessionModule sessionNamespace} $SessionData($idx) {break}

    if {![string compare $module $sessionModule]} then {
      set SessionData($idx) [list "" "::${NamespaceCurrent}::"]
      putdcc $idx [mc "Default module %s disabled, defaulting to global settings." $module]
    }
  }
  return 0
}


##
## Configure module settings: save
##
## Args:    idx, arg
## Returns: 0
##
proc configSave {idx arg} {
  variable moduleDatabaseDataChanged
  variable configData
  variable configDataChanged
  variable configFile

  # Save module database
  if {$moduleDatabaseDataChanged} then {
    putdcc $idx [mc "Saving module database"]
    saveModuleDatabase 1
  }

  # Save configuration data
  if {$configDataChanged} then {
    arraySave configData $configFile 1 [mc "configuration file "]
  }

  set module [lindex $arg 0]

  if {![string compare "" $module]} then {
    putdcc $idx [mc "Saving changed config data for all modules"]
    moduleConfig save * 0 1
  } else {
    if {![moduleExists $module]} then {
      putdcc $idx [mc "Error: unknown module: %s" $module]
    } elseif {![moduleLoaded $module]} then {
      putdcc $idx [mc "Error: module: %s is not loaded" $module]
    } elseif {![configExists $module]} then {
      putdcc $idx [mc "Error: no config options for module: %s" $module]
    } else {
      putdcc $idx [mc "Saving changed config data for module: %s" $module]
      moduleConfigSave $module 0 1
    }
  }
  return 0
}


##
## Configure module settings: help
##
## Args:    idx, arg
## Returns: 0
##
proc configHelp {idx arg} {

  set what [lindex $arg 0]
  if {![string compare "" $what]} then {
    set what "help"
  }

  switch -exact -- $what {
    set {
      putdcc $idx [mc "###  %s <module>:<option> \[what\]" set]
      putdcc $idx [mc "     %s <option> \[what\]" set]
      putdcc $idx [mc "     %s \[module\]" set]
      putdcc $idx [mc "   Sets a global or module option to the specified value."]
      putdcc $idx [mc "   Shows current setting if no new value is given."]
    }
    list {
      putdcc $idx [mc "###  %s \[module\]" list]
      putdcc $idx [mc "   Shows options avaliable for the given module,"]
      putdcc $idx [mc "   or your default module if none is given."]
    }
    info {
      putdcc $idx [mc "###  %s \[module\]" info]
      putdcc $idx [mc "   Shows info for the given module,"]
      putdcc $idx [mc "   or your default module if none is given."]
    }
    default {
      putdcc $idx [mc "### %s \[module\]" default]
      putdcc $idx [mc "   Changes your default module."]
      putdcc $idx [mc "   Defaults to gloabl settings if no module is given."]
    }
    modules {
      putdcc $idx [mc "###  %s \[what\]" modules]
      putdcc $idx [mc "   Shows info for modules"]
      putdcc $idx [mc "   Available options: %s, %s, %s" all enabled disabled]
      putdcc $idx [mc "     %s:      Shows all available modules" all]
      putdcc $idx [mc "     %s:  Shows enabled modules" enabled]
      putdcc $idx [mc "     %s: Shows disabled modules" disabled]
    }
    enable {
      putdcc $idx [mc "###  %s <module>" enable]
      putdcc $idx [mc "   Enable and load the given module."]
    }
    disable {
      putdcc $idx [mc "###  %s <module>" disable]
      putdcc $idx [mc "   Disable and unload the given module."]
    }
    save {
      putdcc $idx [mc "###  %s \[module\]" save]
      putdcc $idx [mc "   Saves config info for the given module,"]
      putdcc $idx [mc "   or all modules if none is given."]
    }
    help {
      putdcc $idx [mc "###  Avaliable configuration commands:"]
      putdcc $idx [mc "  %s       <module>:<option> \[what\] | <option> \[what\]" set]
      putdcc $idx [mc "  %s      \[module\]" list]
      putdcc $idx [mc "  %s      \[module\]" info]
      putdcc $idx [mc "  %s   \[module\]" default]
      putdcc $idx [mc "  %s   \[what\]" modules]
      putdcc $idx [mc "  %s    <module>" enable]
      putdcc $idx [mc "  %s   <module>" disable]
      putdcc $idx [mc "  %s      \[module\]" save]
      putdcc $idx [mc "  %s      \[command\]" help]
      putdcc $idx [mc "  %s" license]
      putdcc $idx [mc "  %s" quit]
      putdcc $idx [mc "You can get help on individual commands: '%s <command>'" help]
    }
    license {
      putdcc $idx [mc "###  %s" license]
      putdcc $idx  [mc "   Show program copyright and license."]
    }
    quit {
      putdcc $idx [mc "###  %s" quit]

      if {$idx == -1} then {
        putdcc $idx [mc "   Quits interactive configuration."]
      } else {
        putdcc $idx [mc "   Quits the wpconf system and returns you to the partyline."]
      }
    }
    default {
      putdcc $idx [mc "No help available on that."]
    }
  }
  return 0
}


##
## Configure module settings: show copyright and license information
##
## Args:    idx, arg
## Returns: 0
##
# FIXME: should msgcat be used for license? [NOT copyright]
proc configLicense {idx arg} {
  variable Copyright
  variable License

# FIXME: use '$arg' == module name to display copyright/license of modules

  putdcc $idx " "
  putdcc $idx "    $Copyright"
  putdcc $idx " "
  foreach line [split $License \n] {
    putdcc $idx "    $line"
  }
  putdcc $idx " "
  return 0
}


##
## Configure module settings: quit
##
## Args:    idx, arg
## Returns: 1
##
proc configQuit {idx arg} {
  variable SessionData

  if {$idx != -1} then {
    putdcc $idx [mc "*** Ja mata!"]
    putdcc $idx [mc "Returning you to command mode..."]

    unset SessionData($idx)
  }
  return 1
}


##
## Save module config and data files
##
## Args:    hand, idx, text {module}
## Returns: nothing
##
proc WpSave {hand idx text} {
  variable moduleDatabaseDataChanged
  variable configData
  variable configDataChanged
  variable configFile

  set what [lindex [split [string trimright $text]] 1]

  if {![string compare "" $what]} then {
    set module "*"
    set force 0
  } elseif {![moduleExists $what]} then {
    putdcc $idx [mc "Error: unknown module: %s" $what]
  } elseif {![moduleLoaded $what]} then {
    putdcc $idx [mc "Error: module: %s is not loaded" $what]
  } else {
    set module $what
    set force 1
  }

  if {[info exists module]} then {

    # Save module database
    if {$moduleDatabaseDataChanged} then {
      putdcc $idx [mc "Saving module database"]
      saveModuleDatabase 1
    }

    # Save configuration data
    if {$configDataChanged} then {
      arraySave configData $configFile 1 [mc "configuration file "]
    }

    moduleConfig save $module $force 1
    moduleData save $module $force 1
  }
  return
}

##
## Reload module config and data files
##
## Args:    hand, idx, text {module}
## Returns: nothing
##
proc WpReload {hand idx text} {
  set what [lindex [split [string trimright $text]] 1]

  if {![string compare "" $what]} then {
    set module "*"
    set force 0
  } elseif {![moduleExists $what]} then {
    putdcc $idx [mc "Error: unknown module: %s" $what]
  } elseif {![moduleLoaded $what]} then {
    putdcc $idx [mc "Error: module: %s is not loaded" $what]
  } else {
    set module $what
    set force 1
  }

  if {[info exists module]} then {
    loadModuleDatabase 1
    moduleConfigLoad $what 1 1
    moduleDataLoad $what 1 1
  }
  return
}


##
## Process dcc commands from the partyline
##
## Args:    idx, text
## Returns: text
##
proc filt:wpcommands {idx text} {
  set hand [idx2hand $idx]

# FIXME: this needs to separate partyline (dcc) and filesys (fil) stuff

  switch -regexp -- [string tolower [lindex $text 0]] {
    {^(\.|)save$} {
      if {[matchattr $hand m]} then {
        WpSave $hand $idx $text
      }
    }
    {^(\.|)reload$} {
      if {[matchattr $hand m]} then {
        WpReload $hand $idx $text
      }
    }
    {^(\.|)help$} {
# FIXME: this is where the new help system will hook into dcc commands
#      set text [WpHelp $hand $idx $text]
    }
  }
  return $text
}


##
## Save changed module config and data files once an hour: time
##
## Args:    handle, idx, arg {module}
## Returns: 1
##
proc time:hourly {minute hour day month year} {
  variable moduleDatabaseDataChanged
  variable configData
  variable configDataChanged
  variable configFile

  if {$moduleDatabaseDataChanged} then {
    saveModuleDatabase 1
  }

  # Save configuration data
  if {$configDataChanged} then {
    arraySave configData $configFile 1 [mc "configuration file "]
  }

  moduleConfig save * 0 1
  moduleData save * 0 1
  return 0
}


##
## Backup module data files every 24 hours: time
##
## Args:    handle, idx, arg {module}
## Returns: 1
##
proc time:daily {minute hour day month year} {
  moduleData backup * 0 1
  return 0
}


##
## Save changed module config and data files: evnt
##
## Args:    handle, idx, arg {module}
## Returns: 1
##
proc evnt:save {type} {
  variable moduleDatabaseDataChanged
  variable configData
  variable configDataChanged
  variable configFile

  if {$moduleDatabaseDataChanged} then {
    saveModuleDatabase 1
  }

  # Save configuration data
  if {$configDataChanged} then {
    arraySave configData $configFile 1 [mc "configuration file "]
  }

  moduleConfig save * 0 1
  moduleData save * 0 1
  return 0
}


# FIXME: implement help stuff for filt type bindings

# Help Data
array set helpData {
  dcc:wpconf {{} {enters wpconf module config system}}
}


##
## End merge of config.tcl
##
###############################################################################


##
## Inline startup and init code
##

# Eggdrop doesn't currently set argv0, so we use it to detect load type.
if {[info exists argv0]} then {
  set TclshMode 1
} else {
  set TclshMode 0
}

if {$TclshMode} then {

  # Eval command line arguments if loading with tclsh
  EvalArgs $argc $argv $argv0

  # Display copyright and license information
  if {$optionData(config)} then {
    ShowInitNotice interactive
  } else {
    ShowInitNotice tclsh
  }
} else {
  ShowInitNotice eggdrop
}

wpLog o * [mc "%s v%s loading..." wolfpack.tcl $ModuleVersion]

# Init md5Sum command
if {![md5Init]} then {
  wpLog o * [mc "Error: can't find a usable md5 command!"]
# FIXME: would it be safe to remove $NamespaceCurrent in this case?
  return 0
}

# Export commands and wrappers
if 1 "namespace export [join $ExportedCommands]"
if 1 "namespace export [join $ExportedWrappers]"

# Set missing variables to default values
if {![info exists configDataChanged]} then {
  set configDataChanged 0
}
if {![info exists moduleDatabaseDataChanged]} then {
  set moduleDatabaseDataChanged 0
}
if {![info exists moduleLoadedList]} then {
  set moduleLoadedList ""
}

# Find configuration file name
global env wp-config-file
if {[string compare "" $optionData(cfgfile)]} then {
  set configFile $optionData(cfgfile)
} elseif {([info exists wp-config-file]) && \
          ([string compare "" ${wp-config-file}])} then {
  set configFile ${wp-config-file}
} elseif {([info exists env(WP_CONFIG_FILE)]) && \
          ([string compare "" $env(WP_CONFIG_FILE)])} then {
  set configFile $env(WP_CONFIG_FILE)
} else {
  set configFile wolfpack.conf
}

wpLog o * [mc "Using configuration file: %s" $configFile]

# Load configuration data
arrayLoad configData $configFile 0 [mc "configuration file "]

# Unset unknown configuration variables
foreach name [array names configData] {
  if {![info exists configDefaults($name)]} then {
    unset configData($name)
    set configDataChanged 1
  }
}

# Set missing configuration variables to defaults
foreach {name data} [array get configDefaults] {
  if {![info exists configData($name)]} then {
    set configData($name) [lindex $data 1]
    set configDataChanged 1
  }
}

# Set oldConfigData configuration variables to match
foreach {name data} [array get configData] {
  if {![info exists oldConfigData($name)]} then {
    set oldConfigData($name) $data
  }
}

# Save configuration data if changed
if {$configDataChanged} then {
  arraySave configData $configFile 0 [mc "configuration file "]
}

# FIXME: if this changes in the config, it needs to be updated
# Set module path
set modulePath [split $configData(modulepath) :]

# Check verbose/quiet options
if {$optionData(quiet)} then {
  set Verbose $optionData(quiet)
} elseif {$configData(verbose)} then {
  set Verbose $configData(verbose)
} else {
  set Verbose $optionData(verbose)
}

# Build array of module option data
ParseModuleOptionData $Verbose

# Check update/noupdate/rebuild options
if {(($configData(update)) || ($optionData(update))) &&
    (!$optionData(noupdate))} then {
  set UpdateDatabase 1
} else {
  set UpdateDatabase 0
}
if {$optionData(rebuild)} then {
  set RebuildDatabase 1
} else {
  set RebuildDatabase 0
}

# Check time options
if {($optionData(time)) || ($configData(time))} then {

  # Sanity check: old eggdrop versions used [time] for a timestamp command
  if {[catch {time}]} then {
    set TimeOk 1
  } else {
    wpLog o * [mc "Warning: old Eggdrop '%s' command detected." time]
    set TimeOk 0
  }
} else {
  set TimeOk 0
}

# Load module database
if {$Verbose >= 0} then {
  wpLog o * [mc "Loading module database..."]
}

if {$TimeOk} then {
  set LoadTime [time {set LoadResult [loadModuleDatabase $Verbose]}]
} else {
  set LoadTime -1
  set LoadResult [loadModuleDatabase $Verbose]
}

set CreateDatabase 0

switch -exact -- $LoadResult {
  1 {
    if {$Verbose >= 0} then {
      wpLog o * [mc "Done.%s" [secondsElapsed [lindex $LoadTime 0]]]
    }
  }
  -1 {
    if {$Verbose >= 0} then {
      wpLog o * [mc "Warning: module database does not exist."]
    }
    set CreateDatabase 1
  }
  0 {
    if {$Verbose >= 0} then {
      wpLog o * [mc "ERROR: Unable to load module database!"]
    }
    # Something really bad happened, stop loading immediately.
    return
  }
}

# Compare and update module database if we are not going to rebuild it
if {($UpdateDatabase) && (!$RebuildDatabase) && (!$CreateDatabase)} then {

  if {$Verbose >= 0} then {
    wpLog o * [mc "Comparing and updating module database..."]
  }

  if {$TimeOk} then {
    set UpdateTime [time {updateModuleDatabase $Verbose}]
  } else {
    set UpdateTime -1
    updateModuleDatabase $Verbose
  }

  if {$Verbose >= 0} then {
    wpLog o * [mc "Done.%s" [secondsElapsed [lindex $UpdateTime 0]]]
  }

# Rebuild database if requested or it does not exist
} elseif {($RebuildDatabase) || ($CreateDatabase)} then {

  if {$Verbose >= 0} then {
    if {$CreateDatabase} then {
      wpLog o * [mc "Creating module database..."]
    } else {
      wpLog o * [mc "Rebuilding module database..."]
    }
  }

  # Rebuild/create module database
  if {$TimeOk} then {
    set RebuildTime [time {rebuildModuleDatabase $Verbose}]
  } else {
    set RebuildTime -1
    rebuildModuleDatabase $Verbose
  }

  if {$Verbose >= 0} then {
    wpLog o * [mc "Done.%s" [secondsElapsed [lindex $RebuildTime 0]]]
  }

# Print a warning if database compare and update is disabled
} elseif {$Verbose >= 0} then {
  wpLog o * [mc "Warning: not updating module database..."]
}

# Save module database if it has changed
if {$moduleDatabaseDataChanged} then {

  if {$Verbose >= 0} then {
    wpLog o * [mc "Saving module database..."]
  }

  if {$TimeOk} then {
    set SaveTime [time {set SaveResult [saveModuleDatabase $Verbose]}]
  } else {
    set SaveTime -1
    set SaveResult [saveModuleDatabase $Verbose]
  }

  if {$Verbose >= 0} then {
    if {$SaveResult} then {
      wpLog o * [mc "Done.%s" [secondsElapsed [lindex $SaveTime 0]]]
    } else {
      wpLog o * [mc "ERROR: Unable to save module database!"]
    }
  }
}


# Process code here if loading with eggdrop or
# if starting in debug or config mode with tclsh
if {(!$TclshMode) || ($optionData(debug)) || ($optionData(config))} then {

  # Build command table
  if {$Verbose >= 0} then {
    wpLog o * [mc "Building command table..."]
  }
  if {$TimeOk} then {
    set CommandTime [time {buildCommandTable $Verbose}]
  } else {
    set CommandTime -1
    buildCommandTable $Verbose
  }
  if {$Verbose >= 0} then {
    wpLog o * [mc "Done.%s" [secondsElapsed [lindex $CommandTime 0]]]
  }

  set loadList ""

  # Load debug module first if not in eggdrop mode
  if {$TclshMode} then {
    if {[moduleExists debug]} then {
      lappend loadList debug
    }

  # Load compat module first if in eggdrop mode
  } elseif {[moduleExists compat]} then {
    lappend loadList compat
  }

  # Load other modules next, moduleLoad will preload additional modules
  foreach name [lsort [array names moduleDatabaseData]] {
    # Don't load debug or compat multiple times
    if {(![regexp {^(compat|debug)$} $name]) &&
        ([getModuleDatabaseData $name load] == 1)} then {
      lappend loadList $name
    }
  }

  wpLog o * [mc "Loading modules..."]

  # Load modules
  if {$TimeOk} then {
    set LoadTime [time {loadModules $loadList $Verbose}]
  } else {
    set LoadTime -1
    loadModules $loadList $Verbose
  }

  wpLog o * [mc "Done.%s" [secondsElapsed [lindex $LoadTime 0]]]

  set ModuleLoadCount [llength $moduleLoadedList]
  if {$ModuleLoadCount} then {
    set ModuleTotalCount [llength [array names moduleDatabaseData]]
    wpLog o * [mc "Modules loaded (%d/%d): %s" $ModuleLoadCount $ModuleTotalCount $moduleLoadedList]
  } else {
    wpLog o * [mc "No modules loaded."]
  }

##########

# FIXME: this is temporary; for old config.tcl code above
bind dcc n|- wpconf dcc:wpconf
bind filt -|- * filt:wpcommands
bind time -|- {00 * * * *} time:hourly
bind time -|- {00 00 * * *} time:daily
bind evnt -|- save evnt:save

##########

  # ModulePostInit
  foreach module $moduleLoadedList {
    if {[string compare "" [info commands ${NamespaceCurrent}::${module}::ModulePostInit]]} then {
      ${NamespaceCurrent}::${module}::ModulePostInit
    }
  }

  # Display date/time during load to be logged for debugging purposes
  # NOTE: Can't use "%c (%Q)" as format string because %Q is exclusive
  set clockSeconds [clock seconds]
  wpLog o * [mc "loaded on %s (%s)" [clock format $clockSeconds -format "%c"] [clock format $clockSeconds -format "%Q"]]
  unset clockSeconds

  # Enter interactive configuration if requested
  if {($TclshMode) && ($optionData(config))} then {
    WpConfig
  }

  # Unload modules if started in debug mode
  if {$optionData(debug)} then {

# FIXME: code module dependency stuff first
#    wpLog o * [mc "Unloading modules..."]

    # Unload all modules except debug
#    foreach module $moduleLoadedList {
#      if {[string compare $module debug]} then {
#        if {[catch {moduleUnload $module $Verbose} result]} then {
#          wpLog o * [mc "Error: unable to unload module `%s': %s" $module $result]
#        }
#      }
#    }

    # Unload debug module last to make it display stats
    if {[moduleLoaded debug]} then {
      if {[catch {moduleUnload debug} result]} then {
        wpLog o * [mc "Error: unable to unload module `%s': %s" debug $result]
      }
    }
#    wpLog o * [mc "Done."]
  }
}

} ;# namespace ::wp

return
